@@ -17,8 +17,8 @@
  */
 
 #include "sw_splash.h"
-#include "wayland_helpers.h"
-
+#include "wayland_app.h"
+#include "wayland_shm.h"
 #include <wayland-client.h>
 
 #include <sys/types.h>
@@ -132,16 +132,10 @@ wl_buffer_listener const buffer_listener = {
 struct SwSplash::Self : SplashSession
 {
     Self()
-        : globals{
-              [](auto const&) {},
-              [](auto const&) {},
-              [](auto const&) {}}
     {
-        // Because we use the outputs exactly once, to enumerate the set of outputs at startup,
-        // we don't need to care about outputs changing or disappearing.
     }
 
-    Globals globals;
+    bool show_splash=true;
 
     DrawContext ctx;
 
@@ -161,6 +155,10 @@ SwSplash::SwSplash() : self{std::make_shared<Self>()} {}
 
 SwSplash::~SwSplash() = default;
 
+void SwSplash::enable (bool show_splash_opt){
+    self->show_splash = show_splash_opt;
+}
+
 void SwSplash::operator()(std::weak_ptr<mir::scene::Session> const& session)
 {
     std::lock_guard<decltype(self->mutex)> lock{self->mutex};
@@ -174,10 +172,13 @@ SwSplash::operator std::shared_ptr<SplashSession>() const
 
 void SwSplash::Self::operator()(struct wl_display* display)
 {
-    globals.init(display);
+    if (!show_splash)
+        return;
+
+    WaylandApp app{display};
 
     ctx.display = display;
-    ctx.surface = wl_compositor_create_surface(globals.compositor);
+    ctx.surface = wl_compositor_create_surface(app.compositor());
 
     static wl_shell_surface_listener const shell_surface_listener
     {
@@ -200,13 +201,13 @@ void SwSplash::Self::operator()(struct wl_display* display)
         [](auto, auto){}, // popup_done
     };
 
-    auto const shell_surface = wl_shell_get_shell_surface(globals.shell, ctx.surface);
+    auto const shell_surface = wl_shell_get_shell_surface(app.shell(), ctx.surface);
     wl_shell_surface_add_listener(shell_surface, &shell_surface_listener, &ctx);
     wl_shell_surface_set_fullscreen(shell_surface, WL_SHELL_SURFACE_FULLSCREEN_METHOD_DEFAULT, 0, nullptr);
     wl_surface_commit(ctx.surface);
     wl_display_roundtrip(display);
 
-    struct wl_shm_pool* shm_pool = make_shm_pool(globals.shm, ctx.width * ctx.height * 4, &ctx.content_area);
+    struct wl_shm_pool* shm_pool = make_shm_pool(app.shm(), ctx.width * ctx.height * 4, &ctx.content_area);
 
     for (auto& b: ctx.buffers)
     {
@@ -222,6 +223,7 @@ void SwSplash::Self::operator()(struct wl_display* display)
 
     auto const time_limit = std::chrono::steady_clock::now() + std::chrono::seconds(2);
 
+    //this is the splash screen
     do
     {
         wl_display_dispatch(display);
@@ -243,8 +245,6 @@ void SwSplash::Self::operator()(struct wl_display* display)
 
     wl_shell_surface_destroy(shell_surface);
     wl_surface_destroy(ctx.surface);
-    globals.teardown();
-    wl_display_roundtrip(display);
 }
 
 void SwSplash::operator()(struct wl_display* display)
