@@ -26,21 +26,23 @@
 #include <miral/runner.h>
 #include <miral/window_management_options.h>
 #include <miral/append_event_filter.h>
-#include <miral/debug_extension.h>
 #include <miral/internal_client.h>
 #include <miral/command_line_option.h>
 #include <miral/cursor_theme.h>
 #include <miral/keymap.h>
+#include <miral/toolkit_event.h>
 #include <miral/x11_support.h>
 #include <miral/wayland_extensions.h>
 
-#include <linux/input.h>
+
+#include <xkbcommon/xkbcommon-keysyms.h>
 #include <unistd.h>
 #include <boost/filesystem.hpp>
 
 int main(int argc, char const* argv[])
 {
     using namespace miral;
+    using namespace miral::toolkit;
 
     std::function<void()> shutdown_hook{[]{}};
 
@@ -58,7 +60,7 @@ int main(int argc, char const* argv[])
 
     ExternalClientLauncher external_client_launcher;
 
-    std::string terminal_cmd{"weston-terminal"};
+    std::string terminal_cmd{"miral-terminal"};
 
     auto const quit_on_ctrl_alt_bksp = [&](MirEvent const* event)
         {
@@ -77,17 +79,19 @@ int main(int argc, char const* argv[])
             if (!(mods & mir_input_event_modifier_alt) || !(mods & mir_input_event_modifier_ctrl))
                 return false;
 
-            switch (mir_keyboard_event_scan_code(kev))
+            switch (mir_keyboard_event_key_code(kev))
             {
-            case KEY_BACKSPACE:
+            case XKB_KEY_BackSpace:
                 runner.stop();
                 return true;
 
-            case KEY_T:
+            case XKB_KEY_t:
+            case XKB_KEY_T:
                 external_client_launcher.launch({terminal_cmd});
                 return false;
 
-            case KEY_X:
+            case XKB_KEY_x:
+            case XKB_KEY_X:
                 external_client_launcher.launch_using_x11({"xterm"});
                 return false;
 
@@ -97,7 +101,17 @@ int main(int argc, char const* argv[])
         };
 
     Keymap config_keymap;
-    DebugExtension debug_extensions;
+
+    auto run_startup_apps = [&](std::string const& apps)
+    {
+      for (auto i = begin(apps); i != end(apps); )
+      {
+          auto const j = find(i, end(apps), ':');
+          external_client_launcher.launch(std::vector<std::string>{std::string{i, j}});
+          if ((i = j) != end(apps)) ++i;
+      }
+    };
+
 
     return runner.run_with(
         {
@@ -109,9 +123,9 @@ int main(int argc, char const* argv[])
             external_client_launcher,
             launcher,
             config_keymap,
-            debug_extensions,
             AppendEventFilter{quit_on_ctrl_alt_bksp},
             StartupInternalClient{spinner},
+            CommandLineOption{run_startup_apps, "startup-apps", "Colon separated list of startup apps", ""},
             pre_init(CommandLineOption{[&](std::string const& typeface) { ::wallpaper::font_file(typeface); },
                                        "shell-wallpaper-font", "font file to use for wallpaper", ::wallpaper::font_file()}),
             CommandLineOption{[&](std::string const& cmd) { terminal_cmd = cmd; },
