@@ -48,27 +48,6 @@
  */
 /* This is C code. Not C++. */
 
-/**
- * Attributes of a surface that the client and server/shell may wish to
- * get or set over the wire.
- */
-typedef enum MirSurfaceAttrib
-{
-    /* Do not specify values...code relies on 0...N ordering. */
-    mir_surface_attrib_type,
-    mir_surface_attrib_state,
-    mir_surface_attrib_swapinterval, /**< \deprecated  Do not listen for events
-                                          reporting this attribute. Use the
-                                          "mir_*_get_swapinterval()" functions
-                                          instead if you wish query its value */
-    mir_surface_attrib_focus,
-    mir_surface_attrib_dpi,
-    mir_surface_attrib_visibility,
-    mir_surface_attrib_preferred_orientation,
-    /* Must be last */
-    mir_surface_attribs
-} MirSurfaceAttrib MIR_FOR_REMOVAL_IN_VERSION_1("use MirWindowAttrib");
-
 /**
  * Attributes of a window that the client and server/shell may wish to
  * get or set over the wire.
@@ -90,22 +69,6 @@ typedef enum MirWindowAttrib
     mir_window_attribs
 } MirWindowAttrib;
 
-typedef enum MirSurfaceType
-{
-    mir_surface_type_normal,       /**< AKA "regular"                       */
-    mir_surface_type_utility,      /**< AKA "floating"                      */
-    mir_surface_type_dialog,
-    mir_surface_type_overlay,      /**< \deprecated  Use "gloss" instead.   */
-    mir_surface_type_gloss = mir_surface_type_overlay,
-    mir_surface_type_freestyle,
-    mir_surface_type_popover,      /**< \deprecated  Choose "menu" or "tip" */
-    mir_surface_type_menu = mir_surface_type_popover,
-    mir_surface_type_inputmethod,  /**< AKA "OSK" or handwriting etc.       */
-    mir_surface_type_satellite,    /**< AKA "toolbox"/"toolbar"             */
-    mir_surface_type_tip,          /**< AKA "tooltip"                       */
-    mir_surface_types
-} MirSurfaceType MIR_FOR_REMOVAL_IN_VERSION_1("use MirWindowType");
-
 typedef enum MirWindowType
 {
     mir_window_type_normal,       /**< AKA "regular"                       */
@@ -121,23 +84,6 @@ typedef enum MirWindowType
     mir_window_types
 } MirWindowType;
 
-typedef enum MirSurfaceState
-{
-    mir_surface_state_unknown,
-    mir_surface_state_restored,
-    mir_surface_state_minimized,
-    mir_surface_state_maximized,
-    mir_surface_state_vertmaximized,
-    /* mir_surface_state_semimaximized,
-       Omitted for now, since it's functionally a subset of vertmaximized and
-       differs only in the X coordinate. */
-    mir_surface_state_fullscreen,
-    mir_surface_state_horizmaximized,
-    mir_surface_state_hidden,
-    mir_surface_state_attached,       /**< Used for panels, notifications and other windows attached to output edges */
-    mir_surface_states
-} MirSurfaceState MIR_FOR_REMOVAL_IN_VERSION_1("use MirWindowState");
-
 typedef enum MirWindowState
 {
     mir_window_state_unknown,
@@ -155,24 +101,12 @@ typedef enum MirWindowState
     mir_window_states
 } MirWindowState;
 
-typedef enum MirSurfaceFocusState
-{
-    mir_surface_unfocused = 0,
-    mir_surface_focused
-} MirSurfaceFocusState MIR_FOR_REMOVAL_IN_VERSION_1("use MirWindowFocusState");
-
 typedef enum MirWindowFocusState
 {
     mir_window_focus_state_unfocused = 0,
     mir_window_focus_state_focused
 } MirWindowFocusState;
 
-typedef enum MirSurfaceVisibility
-{
-    mir_surface_visibility_occluded = 0,
-    mir_surface_visibility_exposed
-} MirSurfaceVisibility MIR_FOR_REMOVAL_IN_VERSION_1("use MirWindowFocusState");
-
 typedef enum MirWindowVisibility
 {
     mir_window_visibility_occluded = 0,
@@ -469,8 +403,10 @@ typedef enum MirShellChrome
 typedef enum MirPointerConfinementState
 {
     mir_pointer_unconfined,
-    MIR_DEPRECATED_ENUM(mir_pointer_confined_to_surface, "mir_pointer_confined_to_window"),
-    mir_pointer_confined_to_window = mir_pointer_confined_to_surface,
+    mir_pointer_confined_oneshot,
+    mir_pointer_confined_persistent,
+    mir_pointer_locked_oneshot,
+    mir_pointer_locked_persistent,
 } MirPointerConfinementState;
 #pragma GCC diagnostic pop
 
