@@ -38,9 +38,8 @@ class Window;
 
 /// Enable configuration of the Wayland extensions enabled at runtime.
 ///
-/// This adds the command line option '--wayland-extensions' the corresponding
-/// MIR_SERVER_WAYLAND_EXTENSIONS environment variable, and the wayland-extensions
-/// config line.
+/// This adds the command line options '--wayland-extensions', '--add-wayland-extensions', '--drop-wayland-extensions'
+/// and the corresponding MIR_SERVER_* environment variables and config file options.
 ///   * The server can add support for additional extensions
 ///   * The server can specify the configuration defaults
 ///   * Mir's option handling allows the defaults to be overridden
@@ -51,33 +50,18 @@ public:
     /// Default to enabling the extensions recommended by Mir
     WaylandExtensions();
 
-    /// Initialize "enabled by default" to a custom set of extensions (colon
-    /// separated list).
-    /// \note This can only be a subset of supported_extensions()
-    /// \deprecated A better option is to use the default constructor, enable()
-    /// and disable(). You can call disable() on all recommended() extensions
-    /// if you want complete control over which are enabled
-    explicit WaylandExtensions(std::string const& default_value);
-
     void operator()(mir::Server& server) const;
 
-    /// All Wayland extensions currently supported (colon separated list).
-    /// This includes both the recommended_extensions() and any extensions that
-    /// have been added using add_extension().
-    /// \deprecated This is of no real use to the server, just for documenting
-    /// the configuration option.
-    auto supported_extensions() const -> std::string;
-
-    /// Default for extensions to enabled recommended by Mir (colon separated list)
-    /// \remark Since MirAL 2.5
-    /// \deprecated Instead of overridding the whole extension list in the constructor and using this to get the
-    /// recommended ones, you can now just enable() the extensions you want.
-    static auto recommended_extensions() -> std::string;
-
     ~WaylandExtensions();
     WaylandExtensions(WaylandExtensions const&);
     auto operator=(WaylandExtensions const&) -> WaylandExtensions&;
 
+    /// All Wayland extensions supported.
+    /// This includes both the supported() provided by Mir and any extensions
+    /// that have been added using add_extension().
+    /// \remark Since MirAL 3.0
+    auto all_supported() const -> std::set<std::string>;
+
     /// Context information useful for implementing Wayland extensions
     /// \remark Since MirAL 2.5
     class Context
@@ -130,6 +114,12 @@ public:
     /// Allows clients to retrieve additional information about outputs
     /// \remark Since MirAL 2.6
     static char const* const zxdg_output_manager_v1;
+
+    /// Allows a client to get information and gain control over all toplevels of all clients
+    /// Useful for taskbars and app switchers
+    /// Could allow a client to extract information about other programs the user is running
+    /// \remark Since MirAL 3.1
+    static char const* const zwlr_foreign_toplevel_manager_v1;
     /** @} */
 
     /// Add a bespoke Wayland extension both to "supported" and "enabled by default".
