@@ -1,5 +1,5 @@
 /*
- * Copyright © 2016 Canonical Ltd.
+ * Copyright © 2016-2020 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 or 3 as
@@ -56,60 +56,51 @@ struct WindowInfo
     auto window() const -> Window&;
 
     auto name() const -> std::string;
-    void name(std::string const& name);
 
     auto type() const -> MirWindowType;
-    void type(MirWindowType type);
 
     auto state() const -> MirWindowState;
-    void state(MirWindowState state);
 
     auto restore_rect() const -> mir::geometry::Rectangle;
-    void restore_rect(mir::geometry::Rectangle const& restore_rect);
 
     auto parent() const -> Window;
-    void parent(Window const& parent);
 
     auto children() const -> std::vector <Window> const&;
-    void add_child(Window const& child);
-    void remove_child(Window const& child);
 
+    /// These constrain the sizes a window may be resized to (both interactively and pragmatically). Clients can request
+    /// a min/max size, but it can be overridden by the window management policy. By default, minimum values are 0 and
+    /// maximum values are `std::numeric_limits<int>::max()`.
+    /// @{
     auto min_width() const -> mir::geometry::Width;
-    void min_width(mir::geometry::Width min_width);
-
     auto min_height() const -> mir::geometry::Height;
-    void min_height(mir::geometry::Height min_height);
-
     auto max_width() const -> mir::geometry::Width;
-    void max_width(mir::geometry::Width max_width);
-
     auto max_height() const -> mir::geometry::Height;
-    void max_height(mir::geometry::Height max_height);
+    /// @}
 
+    /// These control the size increments of the window. This is used in cases like a terminal that can only be resized
+    /// character-by-character. Current Wayland protocols do not support this property, so it generally wont be
+    /// requested by clients. By default, both are 1.
+    /// @{
     auto width_inc() const -> mir::geometry::DeltaX;
-    void width_inc(mir::geometry::DeltaX width_inc);
-
     auto height_inc() const -> mir::geometry::DeltaY;
-    void height_inc(mir::geometry::DeltaY height_inc);
+    /// @}
 
+    /// These constrain the possible aspect ratio of the window. Current Wayland protocols to not support this property,
+    /// so it generally wont be requested by clients. By default, min_aspect is
+    /// `{0U, std::numeric_limits<unsigned>::max()}` and max_aspect is `{std::numeric_limits<unsigned>::max(), 0U}`.
+    /// @{
     auto min_aspect() const -> AspectRatio;
-    void min_aspect(AspectRatio min_aspect);
-
     auto max_aspect() const -> AspectRatio;
-    void max_aspect(AspectRatio max_aspect);
+    /// @}
 
     bool has_output_id() const;
     auto output_id() const -> int;
-    void output_id(mir::optional_value<int> output_id);
 
     auto preferred_orientation() const -> MirOrientationMode;
-    void preferred_orientation(MirOrientationMode preferred_orientation);
 
     auto confine_pointer() const -> MirPointerConfinementState;
-    void confine_pointer(MirPointerConfinementState confinement);
 
     auto shell_chrome() const -> MirShellChrome;
-    void shell_chrome(MirShellChrome chrome);
 
     /// This can be used by client code to store window manager specific information
     auto userdata() const -> std::shared_ptr<void>;
@@ -118,21 +109,14 @@ struct WindowInfo
     void swap(WindowInfo& rhs) { std::swap(self, rhs.self); }
 
     auto depth_layer() const -> MirDepthLayer;
-    void depth_layer(MirDepthLayer depth_layer);
 
     /// Get the edges of the output that the window is attached to
     /// (only meaningful for windows in state mir_window_state_attached)
     auto attached_edges() const -> MirPlacementGravity;
-    /// Set the edges of the output that the window should be attached to
-    /// (only meaningful for windows in state mir_window_state_attached)
-    void attached_edges(MirPlacementGravity edges);
 
     /// Mir will try to avoid occluding the area covered by this rectangle (relative to the window)
     /// (only meaningful when the window is attached to an edge)
     auto exclusive_rect() const -> mir::optional_value<mir::geometry::Rectangle>;
-    /// Set the area to keep exclusive to this window
-    /// (only meaningful when the window is attached to an edge)
-    void exclusive_rect(mir::optional_value<mir::geometry::Rectangle> const& rect);
 
     /// Mir will not render anything outside this rectangle
     auto clip_area() const -> mir::optional_value<mir::geometry::Rectangle>;
@@ -143,10 +127,34 @@ struct WindowInfo
     /// \remark Since MirAL 2.8
     ///@{
     auto application_id() const -> std::string;
-    void application_id(std::string const& application_id);
     ///@}
 
 private:
+    friend class BasicWindowManager;
+    void name(std::string const& name);
+    void type(MirWindowType type);
+    void state(MirWindowState state);
+    void restore_rect(mir::geometry::Rectangle const& restore_rect);
+    void parent(Window const& parent);
+    void add_child(Window const& child);
+    void remove_child(Window const& child);
+    void min_width(mir::geometry::Width min_width);
+    void min_height(mir::geometry::Height min_height);
+    void max_width(mir::geometry::Width max_width);
+    void max_height(mir::geometry::Height max_height);
+    void width_inc(mir::geometry::DeltaX width_inc);
+    void height_inc(mir::geometry::DeltaY height_inc);
+    void min_aspect(AspectRatio min_aspect);
+    void max_aspect(AspectRatio max_aspect);
+    void output_id(mir::optional_value<int> output_id);
+    void preferred_orientation(MirOrientationMode preferred_orientation);
+    void confine_pointer(MirPointerConfinementState confinement);
+    void shell_chrome(MirShellChrome chrome);
+    void depth_layer(MirDepthLayer depth_layer);
+    void attached_edges(MirPlacementGravity edges);
+    void exclusive_rect(mir::optional_value<mir::geometry::Rectangle> const& rect);
+    void application_id(std::string const& application_id);
+
     struct Self;
     std::unique_ptr<Self> self;
 };
