@@ -22,7 +22,6 @@
 
 #include <boost/program_options/options_description.hpp>
 
-#include "mir/extension_description.h"
 #include "mir/module_properties.h"
 #include "mir/module_deleter.h"
 
@@ -55,14 +54,6 @@ class GraphicBufferAllocator;
 class GLConfig;
 class PlatformIpcOperations;
 class PlatformAuthentication;
-class NativeRenderingPlatform
-{
-protected:
-    NativeRenderingPlatform() = default;
-    virtual ~NativeRenderingPlatform() = default;
-    NativeRenderingPlatform(NativeRenderingPlatform const&) = delete;
-    NativeRenderingPlatform& operator=(NativeRenderingPlatform const&) = delete;
-};
 
 /**
  * \defgroup platform_enablement Mir platform enablement
@@ -87,27 +78,6 @@ public:
      */
     virtual UniqueModulePtr<GraphicBufferAllocator> create_buffer_allocator(
         Display const& output) = 0;
-
-    /**
-     * Creates an object capable of doing platform specific processing of buffers
-     * before they are sent or after they are received across IPC
-     */
-    virtual UniqueModulePtr<PlatformIpcOperations> make_ipc_operations() const = 0;
-
-    /**
-     * Access the native resource[s] used to connect to the rendering backend
-     * for this platform
-     */
-    virtual NativeRenderingPlatform* native_rendering_platform() = 0;
-};
-
-class NativeDisplayPlatform
-{
-protected:
-    NativeDisplayPlatform() = default;
-    virtual ~NativeDisplayPlatform() = default;
-    NativeDisplayPlatform(NativeDisplayPlatform const&) = delete;
-    NativeDisplayPlatform& operator=(NativeDisplayPlatform const&) = delete;
 };
 
 class DisplayPlatform
@@ -125,16 +95,6 @@ public:
     virtual UniqueModulePtr<Display> create_display(
         std::shared_ptr<DisplayConfigurationPolicy> const& initial_conf_policy,
         std::shared_ptr<GLConfig> const& gl_config) = 0;
-
-    /**
-     * Access the platform-specific resource[s] from the display.
-     */
-    virtual NativeDisplayPlatform* native_display_platform() = 0;
-
-    /**
-     * Get the extensions for the platform.
-     */
-    virtual std::vector<ExtensionDescription> extensions() const = 0;
 };
 
 class Platform : public DisplayPlatform,
@@ -164,13 +124,15 @@ enum PlatformPriority : uint32_t
     supported = 128,    /**< Capable of providing a functioning Platform on this device,
                          *    possibly with degraded performance or features.
                          */
+    hosted = 192,       /**< Capable of providing a fully-featured Platform on this device,
+                         *   running nested under some other display server rather than with
+                         *   exclusive hardware access.
+                         */
     best = 256          /**< Capable of providing a Platform with the best features and
                          *   performance this device is capable of.
                          */
 };
 
-//The host/guest platform concept is soon to be removed. Use CreateRenderingPlatform and
-//CreateDisplayPlatform instead
 typedef mir::UniqueModulePtr<mir::graphics::Platform>(*CreateHostPlatform)(
     std::shared_ptr<mir::options::Option> const& options,
     std::shared_ptr<mir::EmergencyCleanupRegistry> const& emergency_cleanup_registry,
@@ -186,37 +148,6 @@ typedef mir::graphics::PlatformPriority(*PlatformProbe)(
         mir::options::ProgramOption const& options);
 
 typedef mir::ModuleProperties const*(*DescribeModule)();
-
-typedef mir::UniqueModulePtr<mir::graphics::DisplayPlatform>(*CreateDisplayPlatform)(
-    std::shared_ptr<mir::options::Option> const& options,
-    std::shared_ptr<mir::EmergencyCleanupRegistry> const& emergency_cleanup_registry,
-    std::shared_ptr<mir::ConsoleServices> const& console,
-    std::shared_ptr<mir::graphics::DisplayReport> const& report,
-    std::shared_ptr<mir::logging::Logger> const& logger);
-
-typedef mir::UniqueModulePtr<mir::graphics::RenderingPlatform>(*CreateRenderingPlatform)(
-    std::shared_ptr<mir::options::Option> const& options,
-    std::shared_ptr<mir::graphics::PlatformAuthentication> const& platform_authentication);
-
-/** Signatures from the obsolete 0.27 ABI */
-namespace obsolete_0_27
-{
-constexpr char const* symbol_version = "MIR_GRAPHICS_PLATFORM_0.27";
-
-typedef mir::UniqueModulePtr<mir::graphics::Platform>(*CreateHostPlatform)(
-    std::shared_ptr<mir::options::Option> const& options,
-    std::shared_ptr<mir::EmergencyCleanupRegistry> const& emergency_cleanup_registry,
-    std::shared_ptr<mir::graphics::DisplayReport> const& report,
-    std::shared_ptr<mir::logging::Logger> const& logger);
-
-typedef mir::graphics::PlatformPriority(*PlatformProbe)(mir::options::ProgramOption const& options);
-
-typedef mir::UniqueModulePtr<mir::graphics::DisplayPlatform>(*CreateDisplayPlatform)(
-    std::shared_ptr<mir::options::Option> const& options,
-    std::shared_ptr<mir::EmergencyCleanupRegistry> const& emergency_cleanup_registry,
-    std::shared_ptr<mir::graphics::DisplayReport> const& report,
-    std::shared_ptr<mir::logging::Logger> const& logger);
-}
 }
 }
 
@@ -271,17 +202,6 @@ mir::graphics::PlatformPriority probe_graphics_platform(
 
 mir::ModuleProperties const* describe_graphics_module();
 
-mir::UniqueModulePtr<mir::graphics::DisplayPlatform> create_display_platform(
-    std::shared_ptr<mir::options::Option> const& options,
-    std::shared_ptr<mir::EmergencyCleanupRegistry> const& emergency_cleanup_registry,
-    std::shared_ptr<mir::ConsoleServices> const& console,
-    std::shared_ptr<mir::graphics::DisplayReport> const& report,
-    std::shared_ptr<mir::logging::Logger> const& logger);
-
-mir::UniqueModulePtr<mir::graphics::RenderingPlatform> create_rendering_platform(
-    std::shared_ptr<mir::options::Option> const& options,
-    std::shared_ptr<mir::graphics::PlatformAuthentication> const& platform_authentication);
-
 #if defined(__clang__)
 #pragma clang diagnostic pop
 #endif
