@@ -20,10 +20,8 @@
 #define MIR_TEST_DOUBLES_NULL_PLATFORM_H_
 
 #include "mir/graphics/platform.h"
-#include "mir/graphics/platform_ipc_package.h"
 #include "mir/graphics/graphic_buffer_allocator.h"
 #include "mir/test/doubles/null_display.h"
-#include "mir/test/doubles/null_platform_ipc_operations.h"
 
 namespace mir
 {
@@ -46,26 +44,6 @@ class NullPlatform : public graphics::Platform
     {
         return mir::make_module_ptr<NullDisplay>();
     }
-
-    graphics::NativeDisplayPlatform* native_display_platform() override
-    {
-        return nullptr;
-    }
-
-    mir::UniqueModulePtr<graphics::PlatformIpcOperations> make_ipc_operations() const override
-    {
-        return mir::make_module_ptr<NullPlatformIpcOperations>();
-    }
-
-    graphics::NativeRenderingPlatform* native_rendering_platform() override
-    {
-        return nullptr;
-    }
-
-    std::vector<ExtensionDescription> extensions() const override
-    {
-        return {};
-    }
 };
 }
 }
