@@ -139,26 +139,6 @@ catch (std::exception const& ex)
     MIR_LOG_UNCAUGHT_EXCEPTION(ex);
 }
 
-void mir_connection_get_platform(
-    MirConnection* connection,
-    MirPlatformPackage* platform_package)
-{
-    connection->populate(*platform_package);
-}
-
-void mir_connection_get_graphics_module(MirConnection *connection, MirModuleProperties *properties)
-try
-{
-    mir::require(mir_connection_is_valid(connection));
-    mir::require(properties != nullptr);
-
-    connection->populate_graphics_module(*properties);
-}
-catch (std::exception const& ex)
-{
-    MIR_LOG_UNCAUGHT_EXCEPTION(ex);
-}
-
 void mir_connection_set_lifecycle_event_callback(
     MirConnection* connection,
     MirLifecycleEventCallback callback,
@@ -183,14 +163,6 @@ void mir_connection_pong(MirConnection *connection, int32_t serial)
         connection->pong(serial);
 }
 
-MirDisplayConfiguration* mir_connection_create_display_config(
-    MirConnection* connection)
-{
-    if (connection)
-        return connection->create_copy_of_display_config();
-    return nullptr;
-}
-
 MirDisplayConfig* mir_connection_create_display_configuration(
     MirConnection* connection)
 {
@@ -213,41 +185,6 @@ void mir_connection_set_display_config_change_callback(
         connection->register_display_change_callback(callback, context);
 }
 
-void mir_display_config_destroy(MirDisplayConfiguration* configuration)
-{
-    mcl::delete_config_storage(configuration);
-}
-
-MirWaitHandle* mir_connection_apply_display_config(
-    MirConnection* connection,
-    MirDisplayConfiguration* display_configuration)
-{
-    try
-    {
-        return connection ? connection->configure_display(display_configuration) : nullptr;
-    }
-    catch (std::exception const& ex)
-    {
-        MIR_LOG_UNCAUGHT_EXCEPTION(ex);
-        return nullptr;
-    }
-}
-
-MirWaitHandle* mir_connection_set_base_display_config(
-    MirConnection* connection,
-    MirDisplayConfiguration const* display_configuration)
-{
-    try
-    {
-        return connection ? connection->set_base_display_configuration(display_configuration) : nullptr;
-    }
-    catch (std::exception const& ex)
-    {
-        MIR_LOG_UNCAUGHT_EXCEPTION(ex);
-        return nullptr;
-    }
-}
-
 MirInputConfig* mir_connection_create_input_config(
     MirConnection* connection)
 try
@@ -301,11 +238,6 @@ catch (std::exception const& ex)
     MIR_LOG_UNCAUGHT_EXCEPTION(ex);
 }
 
-void mir_input_config_destroy(MirInputConfig const* config)
-{
-    mir_input_config_release(config);
-}
-
 void mir_input_config_release(MirInputConfig const* config)
 {
     delete config;
@@ -365,12 +297,6 @@ MirEGLNativeDisplayType mir_connection_get_egl_native_display(
     return connection->egl_native_display();
 }
 
-MirPixelFormat mir_connection_get_egl_pixel_format(MirConnection* connection,
-    EGLDisplay disp, EGLConfig conf)
-{
-    return connection->egl_pixel_format(disp, conf);
-}
-
 void mir_connection_get_available_surface_formats(
     MirConnection* connection,
     MirPixelFormat* formats,
@@ -381,22 +307,6 @@ void mir_connection_get_available_surface_formats(
         connection->available_surface_formats(formats, format_size, *num_valid_formats);
 }
 
-MirWaitHandle* mir_connection_platform_operation(
-    MirConnection* connection,
-    MirPlatformMessage const* request,
-    MirPlatformOperationCallback callback, void* context)
-{
-    try
-    {
-        return connection->platform_operation(request, callback, context);
-    }
-    catch (std::exception const& ex)
-    {
-        MIR_LOG_UNCAUGHT_EXCEPTION(ex);
-        return nullptr;
-    }
-}
-
 void mir_connection_set_error_callback(
     MirConnection* connection,
     MirErrorCallback callback,
