@@ -1,5 +1,5 @@
 /*
- * Copyright © 2015-2017 Canonical Ltd.
+ * Copyright © 2015-2020 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 or 3 as
@@ -36,6 +36,21 @@
 using namespace mir;
 using namespace mir::geometry;
 
+auto miral::BasicWindowManager::DisplayArea::bounding_rectangle_of_contained_outputs() const -> Rectangle
+{
+    Rectangles box;
+    for (auto const& output : contained_outputs)
+    {
+        box.add(output.extents());
+    }
+    return box.bounding_rectangle();
+}
+
+auto miral::BasicWindowManager::DisplayArea::is_alive() const -> bool
+{
+    return !contained_outputs.empty();
+}
+
 struct miral::BasicWindowManager::Locker
 {
     explicit Locker(miral::BasicWindowManager* self);
@@ -74,7 +89,6 @@ miral::BasicWindowManager::BasicWindowManager(
     display_layout(display_layout),
     persistent_surface_store{persistent_surface_store},
     policy(build(WindowManagerTools{this})),
-    policy_application_zone_addendum{WindowManagementPolicy::ApplicationZoneAddendum::from(policy.get())},
     display_config_monitor{std::make_shared<DisplayConfigurationListeners>()}
 {
     display_config_monitor->add_listener(this);
@@ -153,16 +167,23 @@ auto miral::BasicWindowManager::add_surface(
     update_attached_and_fullscreen_sets(window_info, window_info.state());
 
     if (window_info.state() == mir_window_state_attached)
-        update_windows_for_outputs();
+        update_application_zones_and_attached_windows();
 
     policy->advise_new_window(window_info);
 
     std::shared_ptr<scene::Surface> const scene_surface = window_info.window();
-    scene_surface->add_observer(std::make_shared<shell::SurfaceReadyObserver>(
-        [this, &window_info](std::shared_ptr<scene::Session> const&, std::shared_ptr<scene::Surface> const&)
-            { Locker lock{this}; policy->handle_window_ready(window_info); },
-        session,
-        scene_surface));
+    if (scene_surface->visible())
+    {
+        policy->handle_window_ready(window_info);
+    }
+    else
+    {
+        scene_surface->add_observer(std::make_shared<shell::SurfaceReadyObserver>(
+            [this, &window_info](std::shared_ptr<scene::Session> const&, std::shared_ptr<scene::Surface> const&)
+                { Locker lock{this}; policy->handle_window_ready(window_info); },
+            session,
+            scene_surface));
+    }
 
     if (parent && spec.aux_rect().is_set() && spec.placement_hints().is_set())
     {
@@ -237,7 +258,7 @@ void miral::BasicWindowManager::remove_window(Application const& application, mi
     if (info.state() == mir_window_state_attached &&
         info.exclusive_rect().is_set())
     {
-        update_windows_for_outputs();
+        update_application_zones_and_attached_windows();
     }
 
     application->destroy_surface(info.window());
@@ -537,14 +558,6 @@ void miral::BasicWindowManager::ask_client_to_close(Window const& window)
         mir_surface->request_client_surface_close();
 }
 
-void miral::BasicWindowManager::force_close(Window const& window)
-{
-    auto application = window.application();
-
-    if (application && window)
-        remove_window(application, info_for(window));
-}
-
 auto miral::BasicWindowManager::active_window() const -> Window
 {
     return allow_active_window ? mru_active_windows.top() : Window{};
@@ -693,18 +706,31 @@ auto miral::BasicWindowManager::active_display_area() const -> std::shared_ptr<D
     return std::make_shared<DisplayArea>(Rectangle{{0, 0}, {100, 100}});
 }
 
+auto miral::BasicWindowManager::display_area_for_output_id(int output_id) const -> std::shared_ptr<DisplayArea>
+{
+    for (auto const& area : display_areas)
+    {
+        for (auto const& output : area->contained_outputs)
+        {
+            if (output.id() == output_id)
+            {
+                return area;
+            }
+        }
+    }
+
+    return {};
+}
+
 auto miral::BasicWindowManager::display_area_for(WindowInfo const& info) const -> std::shared_ptr<DisplayArea>
 {
     auto const window = info.window();
 
     if (info.has_output_id())
     {
-        for (auto const& area : display_areas)
+        if (auto const area = display_area_for_output_id(info.output_id()))
         {
-            if (area->output && area->output->id() == info.output_id())
-            {
-                return area;
-            }
+            return area;
         }
     }
 
@@ -882,6 +908,11 @@ auto miral::BasicWindowManager::active_output() -> geometry::Rectangle const
     return active_display_area()->area;
 }
 
+auto miral::BasicWindowManager::active_application_zone() -> Zone
+{
+    return active_display_area()->application_zone;
+}
+
 void miral::BasicWindowManager::raise_tree(Window const& root)
 {
     auto const& info = info_for(root);
@@ -1093,25 +1124,18 @@ void miral::BasicWindowManager::modify_window(WindowInfo& window_info, WindowSpe
         }
     }
 
-    Point new_pos = modifications.top_left().is_set() ? modifications.top_left().value() : window.top_left();
-
-    if (modifications.size().is_set())
+    if (modifications.size().is_set() || modifications.top_left().is_set() ||
+        modifications.min_width().is_set() || modifications.min_height().is_set() ||
+        modifications.max_width().is_set() || modifications.max_height().is_set() ||
+        modifications.width_inc().is_set() || modifications.height_inc().is_set())
     {
-        place_and_size(window_info, new_pos, modifications.size().value());
-    }
-    else if (modifications.min_width().is_set() || modifications.min_height().is_set() ||
-             modifications.max_width().is_set() || modifications.max_height().is_set() ||
-             modifications.width_inc().is_set() || modifications.height_inc().is_set())
-    {
-        Size new_size = window.size();
+        Point new_pos = modifications.top_left().is_set() ? modifications.top_left().value() : window.top_left();
+        Size new_size = modifications.size().is_set() ? modifications.size().value() : window.size();
 
+        // The new size constraints have already been applied to window_info and constrain_resize() will use them
         window_info.constrain_resize(new_pos, new_size);
         place_and_size(window_info, new_pos, new_size);
     }
-    else if (modifications.top_left().is_set())
-    {
-        place_and_size(window_info, new_pos, window.size());
-    }
 
     if (modifications.placement_hints().is_set())
     {
@@ -1146,7 +1170,7 @@ void miral::BasicWindowManager::modify_window(WindowInfo& window_info, WindowSpe
 
     if (application_zones_need_update)
     {
-        update_windows_for_outputs();
+        update_application_zones_and_attached_windows();
     }
 
     if (modifications.confine_pointer().is_set())
@@ -1394,16 +1418,15 @@ void miral::BasicWindowManager::update_attached_and_fullscreen_sets(WindowInfo&
 
 void miral::BasicWindowManager::set_state(miral::WindowInfo& window_info, MirWindowState value)
 {
-    auto const window = window_info.window();
-    auto const mir_surface = std::shared_ptr<scene::Surface>(window);
-
-    update_attached_and_fullscreen_sets(window_info, value);
-
     if (window_info.state() == value)
     {
         return;
     }
 
+    update_attached_and_fullscreen_sets(window_info, value);
+
+    auto const window = window_info.window();
+    auto const mir_surface = std::shared_ptr<scene::Surface>(window);
     bool const was_hidden = window_info.state() == mir_window_state_hidden ||
                             window_info.state() == mir_window_state_minimized;
 
@@ -1472,11 +1495,13 @@ void miral::BasicWindowManager::set_state(miral::WindowInfo& window_info, MirWin
         break;
 
     default:
-        auto const none_active = !active_window();
+        bool const can_become_active =
+            !active_window() ||
+            active_window().application() == window.application();
         window_info.state(value);
         mir_surface->configure(mir_window_attrib_state, value);
         mir_surface->show();
-        if (was_hidden && none_active)
+        if (was_hidden && can_become_active)
         {
             select_active_window(window);
         }
@@ -1734,17 +1759,12 @@ auto miral::BasicWindowManager::place_new_surface(WindowSpecification parameters
     std::shared_ptr<DisplayArea> display_area;
     if (parameters.output_id().is_set())
     {
-        for (auto const& area : display_areas)
-        {
-            if (area->output && area->output->id() == parameters.output_id())
-            {
-                display_area = area;
-                break;
-            }
-        }
+        display_area = display_area_for_output_id(parameters.output_id().value());
     }
     if (!display_area)
+    {
         display_area = active_display_area();
+    }
 
     auto const application_zone = display_area->application_zone.extents();;
     auto const height = parameters.size().value().height.as_int();
@@ -2476,18 +2496,71 @@ auto miral::BasicWindowManager::apply_exclusive_rect_to_application_zone(
     return zone;
 }
 
+auto miral::BasicWindowManager::add_output_to_display_areas(Locker const&, Output const& output)
+{
+    std::shared_ptr<DisplayArea> existing_area;
+
+    if (output.logical_group_id())
+    {
+        auto const area_iter = std::find_if(display_areas.begin(), display_areas.end(), [&](auto const& area)
+            {
+                return (
+                    area->logical_output_group_id &&
+                    area->logical_output_group_id.value() == output.logical_group_id());
+            });
+        if (area_iter != display_areas.end())
+        {
+            existing_area = *area_iter;
+        }
+    }
+
+    if (existing_area)
+    {
+        existing_area->contained_outputs.push_back(output);
+        existing_area->area = existing_area->bounding_rectangle_of_contained_outputs();
+    }
+    else
+    {
+        auto const new_area = std::make_shared<DisplayArea>(output);
+        if (output.logical_group_id())
+        {
+            new_area->logical_output_group_id = output.logical_group_id();
+        }
+        display_areas.push_back(new_area);
+    }
+}
+
+auto miral::BasicWindowManager::remove_output_from_display_areas(Locker const&, Output const& output)
+{
+    // Remove the output from all areas that contain it
+    for (auto const& area : display_areas)
+    {
+        auto const removed = std::remove_if(
+            area->contained_outputs.begin(),
+            area->contained_outputs.end(),
+            [&](Output const& area_output)
+            {
+                return area_output.is_same_output(output);
+            });
+        bool const output_removed{removed != area->contained_outputs.end()};
+        area->contained_outputs.erase(removed, area->contained_outputs.end());
+
+        if (output_removed)
+        {
+            area->area = area->bounding_rectangle_of_contained_outputs();
+        }
+    }
+}
+
 void miral::BasicWindowManager::advise_output_create(miral::Output const& output)
 {
     Locker lock{this};
 
     outputs.add(output.extents());
 
-    auto area = std::make_shared<DisplayArea>(output);
-    display_areas.push_back(area);
-
-    update_windows_for_outputs();
+    add_output_to_display_areas(lock, output);
+    application_zones_need_update = true;
     policy->advise_output_create(output);
-    policy_application_zone_addendum->advise_application_zone_create(area->application_zone);
 }
 
 void miral::BasicWindowManager::advise_output_update(miral::Output const& updated, miral::Output const& original)
@@ -2497,16 +2570,32 @@ void miral::BasicWindowManager::advise_output_update(miral::Output const& update
     outputs.remove(original.extents());
     outputs.add(updated.extents());
 
-    for (auto& area : display_areas)
+    if (original.logical_group_id() == updated.logical_group_id())
     {
-        if (area->output && area->output.value().is_same_output(original))
+        for (auto& area : display_areas)
         {
-            area->output = updated;
-            area->area = updated.extents();
+            bool update_extents{false};
+            for (auto& output : area->contained_outputs)
+            {
+                if (output.is_same_output(original))
+                {
+                    output = updated;
+                    update_extents = true;
+                }
+            }
+            if (update_extents)
+            {
+                area->area = area->bounding_rectangle_of_contained_outputs();
+            }
         }
     }
+    else
+    {
+        remove_output_from_display_areas(lock, original);
+        add_output_to_display_areas(lock, updated);
+    }
 
-    update_windows_for_outputs();
+    application_zones_need_update = true;
     policy->advise_output_update(updated, original);
 }
 
@@ -2514,41 +2603,59 @@ void miral::BasicWindowManager::advise_output_delete(miral::Output const& output
 {
     Locker lock{this};
 
-    std::vector<std::shared_ptr<DisplayArea>> removed_areas;
-    for (auto const& area : display_areas)
-    {
-        if (area->output && area->output.value().is_same_output(output))
-            removed_areas.push_back(area);
-    }
-
-    display_areas.erase(
-        std::remove_if(
-            display_areas.begin(),
-            display_areas.end(),
-            [&](auto area){
-                return area->output && area->output.value().is_same_output(output);
-            }),
-        display_areas.end());
-
     outputs.remove(output.extents());
 
+    remove_output_from_display_areas(lock, output);
+    application_zones_need_update = true;
+    policy->advise_output_delete(output);
+}
+
+void miral::BasicWindowManager::advise_output_end()
+{
+    Locker lock{this};
+
+    if (application_zones_need_update)
+    {
+        update_application_zones_and_attached_windows();
+        application_zones_need_update = false;
+    }
+}
+
+void miral::BasicWindowManager::update_application_zones_and_attached_windows()
+{
+    // Move all live areas to before the split and areas that should be removed to after
+    auto split = std::stable_partition(
+        display_areas.begin(),
+        display_areas.end(),
+        [&](std::shared_ptr<DisplayArea> const& area)
+        {
+            return area->is_alive();
+        });
+
+    // Move areas after the split into a new vector
+    std::vector<std::shared_ptr<DisplayArea>> removed_areas;
+    std::move(split, display_areas.end(), std::back_inserter(removed_areas));
+
+    // Clean up empty slots left behind
+    display_areas.erase(split, display_areas.end());
+
     for (auto const& area : removed_areas)
     {
+        // Move windows that were attached to the removed areas
         for (auto const& window : area->attached_windows)
         {
             auto info{info_for(window)};
             update_attached_and_fullscreen_sets(info, info.state());
         }
+
+        // Tell the policy about removed application zones
+        if (area->zone_policy_knows_about)
+        {
+            policy->advise_application_zone_delete(area->zone_policy_knows_about.value());
+        }
     }
 
-    update_windows_for_outputs();
-    for (auto& area : removed_areas)
-        policy_application_zone_addendum->advise_application_zone_delete(area->application_zone);
-    policy->advise_output_delete(output);
-}
-
-void miral::BasicWindowManager::update_windows_for_outputs()
-{
+    // Fullscreen surface should fill the whole area (does not depend on what the zones end up being)
     for (auto const& window : fullscreen_surfaces)
     {
         if (window)
@@ -2603,12 +2710,15 @@ void miral::BasicWindowManager::update_windows_for_outputs()
             place_attached_to_zone(*info_ptr, zone_rect, area->area);
         }
 
-        Zone new_zone{area->application_zone};
-        new_zone.extents(zone_rect);
-        if (!(new_zone == area->application_zone))
+        area->application_zone.extents(zone_rect);
+        if (!area->zone_policy_knows_about)
         {
-            policy_application_zone_addendum->advise_application_zone_update(new_zone, area->application_zone);
-            area->application_zone = new_zone;
+            policy->advise_application_zone_create(area->application_zone);
         }
+        if (area->zone_policy_knows_about && !(area->application_zone == area->zone_policy_knows_about.value()))
+        {
+            policy->advise_application_zone_update(area->application_zone, area->zone_policy_knows_about.value());
+        }
+        area->zone_policy_knows_about = area->application_zone;
     }
 }
