@@ -1,5 +1,5 @@
 /*
- * Copyright © 2015-2019 Canonical Ltd.
+ * Copyright © 2015-2020 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 or 3 as
@@ -122,7 +122,7 @@ public:
         std::shared_ptr<mir::scene::Session> const& session,
         std::shared_ptr<mir::scene::Surface> const& surface,
         uint64_t timestamp,
-        MirResizeEdge edge) override;
+        ::MirResizeEdge edge) override;
 
     int set_surface_attribute(
         std::shared_ptr<mir::scene::Session> const& /*application*/,
@@ -162,8 +162,6 @@ public:
 
     void ask_client_to_close(Window const& window) override;
 
-    void force_close(Window const& window) override;
-
     auto active_window() const -> Window override;
 
     auto select_active_window(Window const& hint) -> Window override;
@@ -181,6 +179,7 @@ public:
     auto window_at(mir::geometry::Point cursor) const -> Window override;
 
     auto active_output() -> mir::geometry::Rectangle const override;
+    auto active_application_zone() -> Zone override;
 
     void raise_tree(Window const& root) override;
     void start_drag_and_drop(WindowInfo& window_info, std::vector<uint8_t> const& handle) override;
@@ -201,20 +200,34 @@ private:
     {
         DisplayArea(Output const& output)
             : area{output.extents()},
-              application_zone{Zone{output.extents()}},
-              output{output}
+              application_zone{area},
+              contained_outputs{{output}}
         {
         }
 
         DisplayArea(Rectangle const& area)
             : area{area},
-              application_zone{Zone{area}}
+              application_zone{area}
         {
         }
 
-        Rectangle area; ///< The full area. If there is an output this is the same as the output's extents
+        /// Returns the bounding rectangle of the extents of all contained outputs
+        auto bounding_rectangle_of_contained_outputs() const -> Rectangle;
+
+        /// Returns if this area is currently being used (update_application_zones() will remove it otherwise)
+        auto is_alive() const -> bool;
+
+        Rectangle area; ///< The full area. If there is a single output, this is the same as the output's extents
+        /// The subset of the area where normal applications are generally placed (excludes, for example, panels)
         Zone application_zone;
-        std::experimental::optional<Output> output;
+        /// The last zone given to the policy, or nullopt if the policy hasn't been notified of this area's creation yet
+        std::experimental::optional<Zone> zone_policy_knows_about;
+        /// Often a single output
+        /// can be empty or (in the case of logical output groups) contain multiple outputs
+        /// if all outputs are removed the next call to update_application_zones() will drop this DisplayArea
+        std::vector<Output> contained_outputs;
+        /// Only set if this display area represents a logical group of multiple outputs
+        std::experimental::optional<int> logical_output_group_id;
         std::set<Window> attached_windows; ///< Maximized/anchored/etc windows attached to this area
     };
 
@@ -235,7 +248,6 @@ private:
     std::shared_ptr<DeadWorkspaces> const dead_workspaces{std::make_shared<DeadWorkspaces>()};
 
     std::unique_ptr<WindowManagementPolicy> const policy;
-    WindowManagementPolicy::ApplicationZoneAddendum* const policy_application_zone_addendum;
 
     std::mutex mutex;
     SessionInfoMap app_info;
@@ -247,7 +259,11 @@ private:
     miral::MRUWindowList mru_active_windows;
     bool allow_active_window = true;
     std::set<Window> fullscreen_surfaces;
-    std::vector<std::shared_ptr<DisplayArea>> display_areas; ///< For now these will map 1:1 to outputs, but this should not be assumed
+    /// Generally maps 1:1 with outputs, but this should not be assumed
+    /// For example, if multiple outputs are part of a logical output group they will have one big display area
+    std::vector<std::shared_ptr<DisplayArea>> display_areas;
+    /// If output configuration has changed and application zones need to be updated
+    bool application_zones_need_update{false};
 
     friend class Workspace;
     using wwbimap_t = boost::bimap<
@@ -292,6 +308,7 @@ private:
                  std::vector<std::shared_ptr<Workspace>> const& workspaces_containing_window);
     auto workspaces_containing(Window const& window) const -> std::vector<std::shared_ptr<Workspace>>;
     auto active_display_area() const -> std::shared_ptr<DisplayArea>;
+    auto display_area_for_output_id(int output_id) const -> std::shared_ptr<DisplayArea>; ///< returns null if not found
     auto display_area_for(WindowInfo const& info) const -> std::shared_ptr<DisplayArea>;
     /// Returns the application zone area after shrinking it for the exclusive zone if needed
     static auto apply_exclusive_rect_to_application_zone(
@@ -299,10 +316,16 @@ private:
         mir::geometry::Rectangle const& exclusive_rect_global_coords,
         MirPlacementGravity attached_edges) -> mir::geometry::Rectangle;
 
+    /// Returns the new display area (or null if none was created)
+    auto add_output_to_display_areas(Locker const&, Output const& output);
+    /// Returns any old display areas that have been removed
+    auto remove_output_from_display_areas(Locker const&, Output const& output);
     void advise_output_create(Output const& output) override;
     void advise_output_update(Output const& updated, Output const& original) override;
     void advise_output_delete(Output const& output) override;
-    void update_windows_for_outputs();
+    void advise_output_end() override;
+    /// Updates the application zones of all display areas and moves attached windows as needed
+    void update_application_zones_and_attached_windows();
 };
 }
 
