@@ -17,10 +17,12 @@
  */
 
 #include <miral/minimal_window_manager.h>
-
+#include <miral/toolkit_event.h>
 #include <linux/input.h>
 #include <gmpxx.h>
 
+using namespace miral::toolkit;
+
 namespace
 {
 unsigned int const shift_states =
