@@ -1,5 +1,5 @@
 /*
- * Copyright © 2016 Canonical Ltd.
+ * Copyright © 2016-2020 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 or 3 as
@@ -362,8 +362,7 @@ miral::WindowManagementTrace::WindowManagementTrace(
     WindowManagerTools const& wrapped,
     WindowManagementPolicyBuilder const& builder) :
     wrapped{wrapped},
-    policy(builder(WindowManagerTools{this})),
-    policy_application_zone_addendum{WindowManagementPolicy::ApplicationZoneAddendum::from(policy.get())}
+    policy(builder(WindowManagerTools{this}))
 {
 }
 
@@ -436,15 +435,6 @@ try {
 }
 MIRAL_TRACE_EXCEPTION
 
-void miral::WindowManagementTrace::force_close(miral::Window const& window)
-try {
-    log_input();
-    mir::log_info("%s -> %s", __func__, dump_of(window).c_str());
-    trace_count++;
-    wrapped.force_close(window);
-}
-MIRAL_TRACE_EXCEPTION
-
 auto miral::WindowManagementTrace::active_window() const -> Window
 try {
     log_input();
@@ -483,7 +473,19 @@ try {
     auto result = wrapped.active_output();
     std::stringstream out;
     out << result;
-    mir::log_info("%s -> ", __func__, out.str().c_str());
+    mir::log_info("%s -> %s", __func__, out.str().c_str());
+    trace_count++;
+    return result;
+}
+MIRAL_TRACE_EXCEPTION
+
+auto miral::WindowManagementTrace::active_application_zone() -> Zone
+try {
+    log_input();
+    auto result = wrapped.active_application_zone();
+    std::stringstream out;
+    out << result.extents();
+    mir::log_info("%s -> %s", __func__, out.str().c_str());
     trace_count++;
     return result;
 }
@@ -597,7 +599,7 @@ MIRAL_TRACE_EXCEPTION
 void miral::WindowManagementTrace::end_drag_and_drop()
 try {
     log_input();
-    mir::log_info("%s window_info=%s", __func__);
+    mir::log_info("%s", __func__);
     trace_count++;
     wrapped.end_drag_and_drop();
 }
@@ -631,7 +633,7 @@ MIRAL_TRACE_EXCEPTION
 void miral::WindowManagementTrace::add_tree_to_workspace(
     miral::Window const& window, std::shared_ptr<miral::Workspace> const& workspace)
 try {
-    mir::log_info("%s window=%s, workspace =%p", __func__, dump_of(window).c_str(), workspace.get());
+    mir::log_info("%s window=%s, workspace =%p", __func__, dump_of(window).c_str(), static_cast<void*>(workspace.get()));
     wrapped.add_tree_to_workspace(window, workspace);
 }
 MIRAL_TRACE_EXCEPTION
@@ -639,7 +641,7 @@ MIRAL_TRACE_EXCEPTION
 void miral::WindowManagementTrace::remove_tree_from_workspace(
     miral::Window const& window, std::shared_ptr<miral::Workspace> const& workspace)
 try {
-    mir::log_info("%s window=%s, workspace =%p", __func__, dump_of(window).c_str(), workspace.get());
+    mir::log_info("%s window=%s, workspace =%p", __func__, dump_of(window).c_str(), static_cast<void*>(workspace.get()));
     wrapped.remove_tree_from_workspace(window, workspace);
 }
 MIRAL_TRACE_EXCEPTION
@@ -647,7 +649,7 @@ MIRAL_TRACE_EXCEPTION
 void miral::WindowManagementTrace::move_workspace_content_to_workspace(
     std::shared_ptr<Workspace> const& to_workspace, std::shared_ptr<Workspace> const& from_workspace)
 try {
-    mir::log_info("%s to_workspace=%p, from_workspace=%p", __func__, to_workspace.get(), from_workspace.get());
+    mir::log_info("%s to_workspace=%p, from_workspace=%p", __func__, static_cast<void*>(to_workspace.get()), static_cast<void*>(from_workspace.get()));
     wrapped.move_workspace_content_to_workspace(to_workspace, from_workspace);
 }
 MIRAL_TRACE_EXCEPTION
@@ -663,7 +665,7 @@ MIRAL_TRACE_EXCEPTION
 void miral::WindowManagementTrace::for_each_window_in_workspace(
     std::shared_ptr<miral::Workspace> const& workspace, std::function<void(miral::Window const&)> const& callback)
 try {
-    mir::log_info("%s workspace =%p", __func__, workspace.get());
+    mir::log_info("%s workspace =%p", __func__, static_cast<void*>(workspace.get()));
     wrapped.for_each_window_in_workspace(workspace, callback);
 }
 MIRAL_TRACE_EXCEPTION
@@ -860,7 +862,7 @@ MIRAL_TRACE_EXCEPTION
 void miral::WindowManagementTrace::advise_adding_to_workspace(
     std::shared_ptr<miral::Workspace> const& workspace, std::vector<miral::Window> const& windows)
 try {
-    mir::log_info("%s workspace=%p, windows=%s", __func__, workspace.get(), dump_of(windows).c_str());
+    mir::log_info("%s workspace=%p, windows=%s", __func__, static_cast<void*>(workspace.get()), dump_of(windows).c_str());
     policy->advise_adding_to_workspace(workspace, windows);
 }
 MIRAL_TRACE_EXCEPTION
@@ -868,7 +870,7 @@ MIRAL_TRACE_EXCEPTION
 void miral::WindowManagementTrace::advise_removing_from_workspace(
     std::shared_ptr<miral::Workspace> const& workspace, std::vector<miral::Window> const& windows)
 try {
-    mir::log_info("%s workspace=%p, windows=%s", __func__, workspace.get(), dump_of(windows).c_str());
+    mir::log_info("%s workspace=%p, windows=%s", __func__, static_cast<void*>(workspace.get()), dump_of(windows).c_str());
     policy->advise_removing_from_workspace(workspace, windows);
 }
 MIRAL_TRACE_EXCEPTION
@@ -910,20 +912,20 @@ MIRAL_TRACE_EXCEPTION
 void miral::WindowManagementTrace::advise_application_zone_create(Zone const& application_zone)
 try {
     mir::log_info("%s application_zone=%s", __func__, dump_of(application_zone).c_str());
-    return policy_application_zone_addendum->advise_application_zone_create(application_zone);
+    return policy->advise_application_zone_create(application_zone);
 }
 MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_application_zone_update(Zone const& updated, Zone const& original)
 try {
     mir::log_info("%s updated=%s, original=%s", __func__, dump_of(updated).c_str(), dump_of(original).c_str());
-    return policy_application_zone_addendum->advise_application_zone_update(updated, original);
+    return policy->advise_application_zone_update(updated, original);
 }
 MIRAL_TRACE_EXCEPTION
 
 void miral::WindowManagementTrace::advise_application_zone_delete(Zone const& application_zone)
 try {
     mir::log_info("%s application_zone=%s", __func__, dump_of(application_zone).c_str());
-    return policy_application_zone_addendum->advise_application_zone_delete(application_zone);
+    return policy->advise_application_zone_delete(application_zone);
 }
 MIRAL_TRACE_EXCEPTION
