@@ -1,5 +1,5 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © 2013-2020 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License version 2 or 3,
@@ -42,9 +42,6 @@ char const* const mo::input_report_opt            = "input-report";
 char const* const mo::seat_report_opt            = "seat-report";
 char const* const mo::shared_library_prober_report_opt = "shared-library-prober-report";
 char const* const mo::shell_report_opt            = "shell-report";
-char const* const mo::host_socket_opt             = "host-socket";
-char const* const mo::nested_passthrough_opt      = "nested-passthrough";
-char const* const mo::name_opt                    = "name";
 char const* const mo::offscreen_opt               = "offscreen";
 char const* const mo::touchspots_opt              = "enable-touchspots";
 char const* const mo::cursor_opt                  = "cursor";
@@ -53,7 +50,10 @@ char const* const mo::debug_opt                   = "debug";
 char const* const mo::composite_delay_opt         = "composite-delay";
 char const* const mo::enable_key_repeat_opt       = "enable-key-repeat";
 char const* const mo::x11_display_opt             = "enable-x11";
+char const* const mo::x11_scale_opt               = "x11-scale";
 char const* const mo::wayland_extensions_opt      = "wayland-extensions";
+char const* const mo::add_wayland_extensions_opt  = "add-wayland-extensions";
+char const* const mo::drop_wayland_extensions_opt = "drop-wayland-extensions";
 char const* const mo::enable_mirclient_opt        = "enable-mirclient";
 
 char const* const mo::off_opt_value = "off";
@@ -149,13 +149,7 @@ mo::DefaultConfiguration::DefaultConfiguration(
     namespace po = boost::program_options;
 
     add_options()
-        (host_socket_opt, po::value<std::string>(),
-            "Host socket filename")
-        (server_socket_opt, po::value<std::string>()->default_value(::mir::default_server_socket),
-            "Socket filename [string:default=$XDG_RUNTIME_DIR/mir_socket or /tmp/mir_socket]")
-        (no_server_socket_opt, "Do not provide a socket filename for client connections")
         (arw_server_socket_opt, "Make socket filename globally rw (equivalent to chmod a=rw)")
-        (prompt_socket_opt, "Provide a \"..._trusted\" filename for prompt helper connections")
         (platform_graphics_lib, po::value<std::string>(),
             "Library to use for platform graphics support (default: autodetect)")
         (platform_input_lib, po::value<std::string>(),
@@ -191,11 +185,6 @@ mo::DefaultConfiguration::DefaultConfiguration(
             "frames from clients before compositing). Higher values result in "
             "lower latency but risk causing frame skipping. "
             "Default: A negative value means decide automatically.")
-        (name_opt, po::value<std::string>(),
-            "When nested, the name Mir uses when registering with the host.")
-        (nested_passthrough_opt, po::value<bool>()->default_value(true),
-            "When nested, attempt to pass a client's graphics content directly to the host"
-            " to avoid a composition pass")
         (offscreen_opt,
             "Render to offscreen buffers instead of the real outputs.")
         (touchspots_opt,
@@ -209,7 +198,6 @@ mo::DefaultConfiguration::DefaultConfiguration(
             "in unexpected ways] throw an exception (instead of a core dump)")
         (debug_opt, "Enable extra development debugging. "
             "This is only interesting for people doing Mir server or client development.")
-        (enable_mirclient_opt, "Enable deprecated mirclient socket (for running old clients)")
         (console_provider,
             po::value<std::string>()->default_value("auto"),
             "Console device handling\n"
@@ -353,7 +341,7 @@ void mo::DefaultConfiguration::parse_arguments(
         {
             std::ostringstream help_text;
             help_text << desc;
-            BOOST_THROW_EXCEPTION(mir::AbnormalExit(help_text.str()));
+            BOOST_THROW_EXCEPTION(mir::ExitWithOutput(help_text.str()));
         }
     }
     catch (po::error const& error)
