@@ -17,20 +17,19 @@
  *   Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
-#ifndef MIR_GRAPHICS_MESA_SHM_BUFFER_H_
-#define MIR_GRAPHICS_MESA_SHM_BUFFER_H_
+#ifndef MIR_GRAPHICS_GBM_SHM_BUFFER_H_
+#define MIR_GRAPHICS_GBM_SHM_BUFFER_H_
 
 #include "mir/graphics/buffer_basic.h"
 #include "mir/geometry/dimensions.h"
 #include "mir/geometry/size.h"
 #include "mir_toolkit/common.h"
-#include "mir/renderer/gl/texture_source.h"
 #include "mir/renderer/gl/texture_target.h"
 #include "mir_toolkit/mir_native_buffer.h"
 #include "mir/renderer/sw/pixel_source.h"
 #include "mir/graphics/texture.h"
 
-#include MIR_SERVER_GL_H
+#include <GLES2/gl2.h>
 
 #include <mutex>
 
@@ -42,97 +41,71 @@ namespace graphics
 {
 namespace common
 {
-/*
- * renderer::gl::TextureSource and graphics::gl::Texture both have
- * a bind() method. They need to do different things.
- *
- * Because we can't just override them based on their signature,
- * do the intermediate-base-class trick of having two proxy bases
- * which do nothing but rename bind() to something unique.
- */
+class EGLContextExecutor;
 
-class BindResolverTex : public gl::Texture
+class ShmBuffer :
+    public BufferBasic,
+    public NativeBufferBase,
+    public graphics::gl::Texture
 {
 public:
-    BindResolverTex() = default;
+    ~ShmBuffer() noexcept override;
 
-    void bind() override final
-    {
-        tex_bind();
-    }
-
-protected:
-    virtual void tex_bind() = 0;
-};
-
-class BindResolverTexTarget : public renderer::gl::TextureSource
-{
-public:
-    BindResolverTexTarget() = default;
-
-    void bind() override final
-    {
-        upload_to_texture();
-    }
-
-protected:
-    virtual void upload_to_texture() = 0;
-};
-
-class ShmBuffer : public BufferBasic, public NativeBufferBase,
-                  public BindResolverTexTarget,
-                  public renderer::gl::TextureTarget,
-                  public renderer::software::PixelSource,
-                  public BindResolverTex
-{
-public:
     static bool supports(MirPixelFormat);
 
-    ~ShmBuffer() noexcept;
-
     geometry::Size size() const override;
-    geometry::Stride stride() const override;
     MirPixelFormat pixel_format() const override;
-    void gl_bind_to_texture() override;
-    void upload_to_texture() override;
-    void secure_for_render() override;
-    void write(unsigned char const* data, size_t size) override;
-    void read(std::function<void(unsigned char const*)> const& do_with_pixels) override;
     NativeBufferBase* native_buffer_base() override;
 
-    void tex_bind() override;
+    void bind() override;
     gl::Program const& shader(gl::ProgramFactory& cache) const override;
     Layout layout() const override;
     void add_syncpoint() override;
-
-    //each platform will have to return the NativeBuffer type that the platform has defined.
-    virtual std::shared_ptr<graphics::NativeBuffer> native_buffer_handle() const override = 0;
-
-    void bind_for_write() override;
-    void commit() override;
-
 protected:
-    ShmBuffer(std::unique_ptr<ShmFile> shm_file,
-              geometry::Size const& size,
-              MirPixelFormat const& pixel_format);
-
-    std::shared_ptr<MirBufferPackage> to_mir_buffer_package() const;
+    ShmBuffer(
+        geometry::Size const& size,
+        MirPixelFormat const& format,
+        std::shared_ptr<EGLContextExecutor> egl_delegate);
 
+    /// \note This must be called with a current GL context
+    void upload_to_texture(void const* pixels, geometry::Stride const& stride);
 private:
-    ShmBuffer(ShmBuffer const&) = delete;
-    ShmBuffer& operator=(ShmBuffer const&) = delete;
-
-    std::unique_ptr<ShmFile> const shm_file;
     geometry::Size const size_;
     MirPixelFormat const pixel_format_;
-    geometry::Stride const stride_;
-    void* const pixels;
+    std::shared_ptr<EGLContextExecutor> const egl_delegate;
     std::mutex tex_id_mutex;
     GLuint tex_id{0};
 };
 
+class MemoryBackedShmBuffer :
+    public ShmBuffer,
+    public renderer::software::PixelSource
+{
+public:
+    MemoryBackedShmBuffer(
+        geometry::Size const& size,
+        MirPixelFormat const& pixel_format,
+        std::shared_ptr<EGLContextExecutor> egl_delegate);
+
+    void write(unsigned char const* data, size_t size) override;
+    void read(std::function<void(unsigned char const*)> const& do_with_pixels) override;
+    geometry::Stride stride() const override;
+
+    std::shared_ptr<NativeBuffer> native_buffer_handle() const override;
+
+    void bind() override;
+
+    MemoryBackedShmBuffer(MemoryBackedShmBuffer const&) = delete;
+    MemoryBackedShmBuffer& operator=(MemoryBackedShmBuffer const&) = delete;
+private:
+    geometry::Stride const stride_;
+    std::unique_ptr<unsigned char[]> const pixels;
+    std::mutex uploaded_mutex;
+    bool uploaded{false};
+};
+
 }
 }
 }
 
-#endif /* MIR_GRAPHICS_MESA_SHM_BUFFER_H_ */
+#endif /* MIR_GRAPHICS_GBM_SHM_BUFFER_H_ */
