@@ -23,11 +23,11 @@
 #include "buffer_texture_binder.h"
 #include "mir/anonymous_shm_file.h"
 #include "shm_buffer.h"
+#include "buffer_from_wl_shm.h"
 #include "mir/graphics/buffer_properties.h"
 #include "mir/renderer/gl/context_source.h"
 #include "mir/renderer/gl/context.h"
 #include "mir/graphics/display.h"
-#include "software_buffer.h"
 #include "wayland-eglstream-controller.h"
 #include "mir/graphics/egl_error.h"
 #include "mir/graphics/texture.h"
@@ -95,33 +95,24 @@ std::unique_ptr<mir::renderer::gl::Context> context_for_output(mg::Display const
 }
 
 mge::BufferAllocator::BufferAllocator(mg::Display const& output)
-    : ctx{context_for_output(output)}
+    : wayland_ctx{context_for_output(output)},
+      egl_delegate{
+          std::make_shared<mgc::EGLContextExecutor>(context_for_output(output))}
 {
 }
 
 mge::BufferAllocator::~BufferAllocator() = default;
 
-std::shared_ptr<mg::Buffer> mge::BufferAllocator::alloc_buffer(
-    BufferProperties const& buffer_properties)
-{
-    if (buffer_properties.usage == mg::BufferUsage::software)
-        return alloc_software_buffer(buffer_properties.size, buffer_properties.format);
-    BOOST_THROW_EXCEPTION(std::runtime_error("platform incapable of creating hardware buffers"));
-}
-
 std::shared_ptr<mg::Buffer> mge::BufferAllocator::alloc_software_buffer(geom::Size size, MirPixelFormat format)
 {
-    if (!mgc::ShmBuffer::supports(format))
+    if (!mgc::MemoryBackedShmBuffer::supports(format))
     {
         BOOST_THROW_EXCEPTION(
             std::runtime_error(
                 "Trying to create SHM buffer with unsupported pixel format"));
     }
 
-    auto const stride = geom::Stride{ MIR_BYTES_PER_PIXEL(format) * size.width.as_uint32_t() };
-    size_t const size_in_bytes = stride.as_int() * size.height.as_int();
-    return std::make_shared<mge::SoftwareBuffer>(
-        std::make_unique<mir::AnonymousShmFile>(size_in_bytes), size, format);
+    return std::make_shared<mgc::MemoryBackedShmBuffer>(size, format, egl_delegate);
 }
 
 std::vector<MirPixelFormat> mge::BufferAllocator::supported_pixel_formats()
@@ -130,11 +121,6 @@ std::vector<MirPixelFormat> mge::BufferAllocator::supported_pixel_formats()
     return {mir_pixel_format_argb_8888, mir_pixel_format_xrgb_8888};
 }
 
-std::shared_ptr<mg::Buffer> mge::BufferAllocator::alloc_buffer(geometry::Size, uint32_t, uint32_t)
-{
-    BOOST_THROW_EXCEPTION(std::runtime_error("platform incapable of creating buffers"));
-}
-
 namespace
 {
 
@@ -338,19 +324,19 @@ void mge::BufferAllocator::create_buffer_eglstream_resource(
         EGL_NONE
     };
 
-    allocator->ctx->make_current();
+    allocator->wayland_ctx->make_current();
     auto dpy = eglGetCurrentDisplay();
 
-    auto stream = allocator->nv_extensions.eglCreateStreamAttribNV(dpy, attribs);
+    auto stream = allocator->nv_extensions(dpy).eglCreateStreamAttribNV(dpy, attribs);
 
     if (stream == EGL_NO_STREAM_KHR)
     {
         BOOST_THROW_EXCEPTION((mg::egl_error("Failed to create EGLStream from Wayland buffer")));
     }
 
-    BoundEGLStream::associate_stream(buffer, allocator->ctx, stream);
+    BoundEGLStream::associate_stream(buffer, allocator->wayland_ctx, stream);
 
-    allocator->ctx->release_current();
+    allocator->wayland_ctx->release_current();
 }
 
 struct wl_eglstream_controller_interface const mge::BufferAllocator::impl{
@@ -395,12 +381,12 @@ void mir::graphics::eglstream::BufferAllocator::bind_display(
     }
 
     auto context_guard = mir::raii::paired_calls(
-        [this]() { ctx->make_current(); },
-        [this]() { ctx->release_current(); });
+        [this]() { wayland_ctx->make_current(); },
+        [this]() { wayland_ctx->release_current(); });
 
     auto dpy = eglGetCurrentDisplay();
 
-    if (extensions.eglBindWaylandDisplayWL(dpy, display) != EGL_TRUE)
+    if (extensions(dpy).eglBindWaylandDisplayWL(dpy, display) != EGL_TRUE)
     {
         BOOST_THROW_EXCEPTION((mg::egl_error("Failed to bind Wayland EGL display")));
     }
@@ -428,7 +414,20 @@ void mir::graphics::eglstream::BufferAllocator::bind_display(
         BOOST_THROW_EXCEPTION((std::runtime_error{message.str()}));
     }
 
-    mir::log_info("Bound EGLStreams-backed WaylandAllocator display");
+    mir::log_info("Bound EGLStreams-backed Wayland display");
+}
+
+void mir::graphics::eglstream::BufferAllocator::unbind_display(wl_display* display)
+{
+    auto context_guard = mir::raii::paired_calls(
+        [this]() { wayland_ctx->make_current(); },
+        [this]() { wayland_ctx->release_current(); });
+    auto dpy = eglGetCurrentDisplay();
+
+    if (extensions(dpy).eglUnbindWaylandDisplayWL(dpy, display) != EGL_TRUE)
+    {
+        BOOST_THROW_EXCEPTION((mg::egl_error("Failed to unbind Wayland EGL display")));
+    }
 }
 
 namespace
@@ -523,16 +522,16 @@ mir::graphics::eglstream::BufferAllocator::buffer_from_resource(
     std::function<void()>&& /*on_release*/)
 {
     auto context_guard = mir::raii::paired_calls(
-        [this]() { ctx->make_current(); },
-        [this]() { ctx->release_current(); });
+        [this]() { wayland_ctx->make_current(); },
+        [this]() { wayland_ctx->release_current(); });
     auto dpy = eglGetCurrentDisplay();
 
     EGLint width, height;
-    if (extensions.eglQueryWaylandBufferWL(dpy, buffer, EGL_WIDTH, &width) != EGL_TRUE)
+    if (extensions(dpy).eglQueryWaylandBufferWL(dpy, buffer, EGL_WIDTH, &width) != EGL_TRUE)
     {
         BOOST_THROW_EXCEPTION(mg::egl_error("Failed to query Wayland buffer width"));
     }
-    if (extensions.eglQueryWaylandBufferWL(dpy, buffer, EGL_HEIGHT, &height) != EGL_TRUE)
+    if (extensions(dpy).eglQueryWaylandBufferWL(dpy, buffer, EGL_HEIGHT, &height) != EGL_TRUE)
     {
         BOOST_THROW_EXCEPTION(mg::egl_error("Failed to query Wayland buffer height"));
     }
@@ -540,7 +539,7 @@ mir::graphics::eglstream::BufferAllocator::buffer_from_resource(
         [&]()
         {
             EGLint y_inverted;
-            if (extensions.eglQueryWaylandBufferWL(dpy, buffer, EGL_WAYLAND_Y_INVERTED_WL, &y_inverted) != EGL_TRUE)
+            if (extensions(dpy).eglQueryWaylandBufferWL(dpy, buffer, EGL_WAYLAND_Y_INVERTED_WL, &y_inverted) != EGL_TRUE)
             {
                 // If querying Y_INVERTED fails, we must have the default, GL, layout
                 return mg::gl::Texture::Layout::GL;
@@ -562,3 +561,15 @@ mir::graphics::eglstream::BufferAllocator::buffer_from_resource(
         geom::Size{width, height},
         layout);
 }
+
+auto mge::BufferAllocator::buffer_from_shm(
+    wl_resource* buffer,
+    std::shared_ptr<Executor> wayland_executor,
+    std::function<void()>&& on_consumed) -> std::shared_ptr<Buffer>
+{
+    return mg::wayland::buffer_from_wl_shm(
+        buffer,
+        std::move(wayland_executor),
+        egl_delegate,
+        std::move(on_consumed));
+}
