@@ -23,7 +23,6 @@
 #include "mir/options/option.h"
 #include "mir/graphics/graphic_buffer_allocator.h"
 #include "mir/graphics/display.h"
-#include "mir/graphics/platform_ipc_operations.h"
 #include "mir/fd.h"
 #include "mir/renderer/gl/egl_platform.h"
 
@@ -49,8 +48,6 @@ class RenderingPlatform : public graphics::RenderingPlatform
 public:
     UniqueModulePtr<GraphicBufferAllocator>
         create_buffer_allocator(Display const& output) override;
-    UniqueModulePtr<PlatformIpcOperations> make_ipc_operations() const override;
-    NativeRenderingPlatform* native_rendering_platform() override;
 };
 
 class DisplayPlatform : public graphics::DisplayPlatform
@@ -64,8 +61,6 @@ public:
     UniqueModulePtr<Display> create_display(
         std::shared_ptr<DisplayConfigurationPolicy> const& /*initial_conf_policy*/,
         std::shared_ptr<GLConfig> const& /*gl_config*/) override;
-    NativeDisplayPlatform* native_display_platform() override;
-    std::vector<ExtensionDescription> extensions() const override;
 
 private:
     std::shared_ptr<DisplayReport> const display_report;
@@ -80,7 +75,7 @@ public:
     Platform(
         std::shared_ptr<RenderingPlatform> const&,
         std::shared_ptr<DisplayPlatform> const&);
-    ~Platform() = default;
+    ~Platform();
 
     UniqueModulePtr<GraphicBufferAllocator>
         create_buffer_allocator(Display const& output) override;
@@ -88,12 +83,6 @@ public:
     UniqueModulePtr<Display> create_display(
         std::shared_ptr<DisplayConfigurationPolicy> const& /*initial_conf_policy*/,
         std::shared_ptr<GLConfig> const& /*gl_config*/) override;
-    NativeDisplayPlatform* native_display_platform() override;
-    std::vector<ExtensionDescription> extensions() const override;
-
-    UniqueModulePtr<PlatformIpcOperations> make_ipc_operations() const override;
-
-    NativeRenderingPlatform* native_rendering_platform() override;
 
 private:
     std::shared_ptr<RenderingPlatform> const rendering;
