@@ -22,6 +22,7 @@
 #include "utils.h"
 #include "mir/graphics/platform.h"
 #include "mir/options/option.h"
+#include "mir/options/configuration.h"
 #include "mir/module_deleter.h"
 #include "mir/assert_module_entry_point.h"
 #include "mir/libname.h"
@@ -29,6 +30,8 @@
 #include "mir/graphics/egl_error.h"
 #include "one_shot_device_observer.h"
 #include "mir/raii.h"
+#include "kms-utils/drm_mode_resources.h"
+#include "mir/graphics/egl_logger.h"
 
 #include <boost/throw_exception.hpp>
 #include <boost/exception/diagnostic_information.hpp>
@@ -82,37 +85,24 @@ EGLDeviceEXT find_device()
 }
 
 mir::UniqueModulePtr<mg::Platform> create_host_platform(
-    std::shared_ptr<mo::Option> const&,
+    std::shared_ptr<mo::Option> const& options,
     std::shared_ptr<mir::EmergencyCleanupRegistry> const&,
     std::shared_ptr<mir::ConsoleServices> const& console,
     std::shared_ptr<mg::DisplayReport> const& display_report,
-    std::shared_ptr<mir::logging::Logger> const&)
+    std::shared_ptr<mir::logging::Logger> const& logger)
 {
     mir::assert_entry_point_signature<mg::CreateHostPlatform>(&create_host_platform);
+
+    if (options->is_set(mo::debug_opt))
+    {
+        mg::initialise_egl_logger(logger);
+    }
+
     return mir::make_module_ptr<mge::Platform>(
         std::make_shared<mge::RenderingPlatform>(),
         std::make_shared<mge::DisplayPlatform>(*console, find_device(), display_report));
 }
 
-mir::UniqueModulePtr<mg::DisplayPlatform> create_display_platform(
-    std::shared_ptr<mo::Option> const&, 
-    std::shared_ptr<mir::EmergencyCleanupRegistry> const&,
-    std::shared_ptr<mir::ConsoleServices> const& console,
-    std::shared_ptr<mg::DisplayReport> const& display_report,
-    std::shared_ptr<mir::logging::Logger> const&) 
-{
-    mir::assert_entry_point_signature<mg::CreateDisplayPlatform>(&create_display_platform);
-    return mir::make_module_ptr<mge::DisplayPlatform>(*console, find_device(), display_report);
-}
-
-mir::UniqueModulePtr<mg::RenderingPlatform> create_rendering_platform(
-    std::shared_ptr<mir::options::Option> const&,
-    std::shared_ptr<mg::PlatformAuthentication> const&)
-{
-    mir::assert_entry_point_signature<mg::CreateRenderingPlatform>(&create_rendering_platform);
-    return mir::make_module_ptr<mge::RenderingPlatform>();
-}
-
 void add_graphics_platform_options(boost::program_options::options_description& /*config*/)
 {
     mir::assert_entry_point_signature<mg::AddPlatformOptions>(&add_graphics_platform_options);
@@ -230,10 +220,18 @@ mg::PlatformPriority probe_graphics_platform(
                         return false;
                     }
 
-                    int const drm_node_attrib[] = {
-                        EGL_DRM_MASTER_FD_EXT, static_cast<int>(drm_fd), EGL_NONE
-                    };
-                    EGLDisplay display = eglGetPlatformDisplayEXT(EGL_PLATFORM_DEVICE_EXT, device, drm_node_attrib);
+                    mg::kms::DRMModeResources kms_resources{drm_fd};
+                    if (
+                        kms_resources.num_connectors() == 0 ||
+                        kms_resources.num_encoders() == 0 ||
+                        kms_resources.num_crtcs() == 0)
+                    {
+                        mir::log_info("KMS support found, but device has no output hardware.");
+                        mir::log_info("This is probably a render-only hybrid graphics device");
+                        return false;
+                    }
+
+                    EGLDisplay display = eglGetPlatformDisplayEXT(EGL_PLATFORM_DEVICE_EXT, device, nullptr);
 
                     if (display == EGL_NO_DISPLAY)
                     {
@@ -264,9 +262,9 @@ mg::PlatformPriority probe_graphics_platform(
                         return false;
                     }
 
-                    eglBindAPI(MIR_SERVER_EGL_OPENGL_API);
+                    eglBindAPI(EGL_OPENGL_ES_API);
                     EGLint const config_attribs[] = {
-                        EGL_RENDERABLE_TYPE, MIR_SERVER_EGL_OPENGL_BIT,
+                        EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
                         EGL_SURFACE_TYPE, EGL_STREAM_BIT_KHR,
                         EGL_NONE
                     };
@@ -282,14 +280,12 @@ mg::PlatformPriority probe_graphics_platform(
                         [&ctx, display, config]()
                         {
                             EGLint const context_attr[] = {
-#if MIR_SERVER_EGL_OPENGL_BIT == EGL_OPENGL_ES2_BIT
                                 EGL_CONTEXT_CLIENT_VERSION, 2,
-#endif
                                 EGL_NONE
                             };
                             ctx = eglCreateContext(display, config, EGL_NO_CONTEXT, context_attr);
                         },
-                        [ctx, display]()
+                        [&ctx, display]()
                         {
                             if (ctx != EGL_NO_CONTEXT)
                             {
