@@ -206,7 +206,7 @@ public:
 
     mir::graphics::gl::Program&
         compile_fragment_shader(
-            void* id,
+            void const* id,
             char const* extension_fragment,
             char const* fragment_fragment) override
     {
@@ -224,13 +224,13 @@ public:
 
         std::stringstream opaque_fragment;
         opaque_fragment
+            << extension_fragment
+            << "\n"
             <<
             "#ifdef GL_ES\n"
             "precision mediump float;\n"
             "#endif\n"
             << "\n"
-            << extension_fragment
-            << "\n"
             << fragment_fragment
             << "\n"
             <<
@@ -241,13 +241,13 @@ public:
 
         std::stringstream alpha_fragment;
         alpha_fragment
+            << extension_fragment
+            << "\n"
             <<
             "#ifdef GL_ES\n"
             "precision mediump float;\n"
             "#endif\n"
             << "\n"
-            << extension_fragment
-            << "\n"
             << fragment_fragment
             << "\n"
             <<
@@ -290,9 +290,8 @@ private:
         glGetShaderiv(id, GL_COMPILE_STATUS, &ok);
         if (!ok)
         {
-            GLchar log[1024];
-            glGetShaderInfoLog(id, sizeof log - 1, NULL, log);
-            log[sizeof log - 1] = '\0';
+            GLchar log[1024] = "(No log info)";
+            glGetShaderInfoLog(id, sizeof log, NULL, log);
             glDeleteShader(id);
             BOOST_THROW_EXCEPTION(
                 std::runtime_error(
@@ -325,7 +324,7 @@ private:
     }
 
     ShaderHandle const vertex_shader;
-    std::vector<std::pair<void*, std::unique_ptr<::Program>>> programs;
+    std::vector<std::pair<void const*, std::unique_ptr<::Program>>> programs;
     // GL requires us to synchronise multi-threaded access to the shader APIs.
     std::mutex compilation_mutex;
 };
@@ -359,7 +358,7 @@ mrg::Renderer::Renderer(graphics::DisplayBuffer& display_buffer)
       texture_cache(mgl::DefaultProgramFactory().create_texture_cache()),
       display_transform(1)
 {
-    eglBindAPI(MIR_SERVER_EGL_OPENGL_API);
+    eglBindAPI(EGL_OPENGL_ES_API);
     EGLDisplay disp = eglGetCurrentDisplay();
     if (disp != EGL_NO_DISPLAY)
     {
