@@ -65,7 +65,8 @@ private:
     ScheduleMode schedule_mode;
     std::shared_ptr<Schedule> schedule;
     std::shared_ptr<MultiMonitorArbiter> const arbiter;
-    geometry::Size size; 
+    geometry::Size latest_buffer_size;
+    float scale_{1.0f};
     MirPixelFormat pf;
     bool first_frame_posted;
 
