@@ -38,9 +38,6 @@
 #include "mir/geometry/dimensions.h"
 #include "mir/graphics/display_configuration.h"
 #include "mir/graphics/pixel_format_utils.h"
-#include "mir/graphics/platform_ipc_operations.h"
-#include "mir/graphics/platform_ipc_package.h"
-#include "mir/graphics/platform_operation_message.h"
 #include "mir/graphics/gamma_curves.h"
 #include "mir/frontend/client_constants.h"
 #include "mir/frontend/event_sink.h"
@@ -99,26 +96,22 @@ mg::GammaCurve convert_string_to_gamma_curve(std::string const& str_bytes)
 
 mf::SessionMediator::SessionMediator(
     std::shared_ptr<mf::Shell> const& shell,
-    std::shared_ptr<mg::PlatformIpcOperations> const& ipc_operations,
     std::shared_ptr<mf::DisplayChanger> const& display_changer,
     std::vector<MirPixelFormat> const& surface_pixel_formats,
     std::shared_ptr<SessionMediatorObserver> const& observer,
     std::shared_ptr<mf::EventSinkFactory> const& sink_factory,
     std::shared_ptr<mf::MessageSender> const& message_sender,
     std::shared_ptr<MessageResourceCache> const& resource_cache,
-    std::shared_ptr<Screencast> const& screencast,
     ConnectionContext const& connection_context,
     std::shared_ptr<mi::CursorImages> const& cursor_images,
     std::shared_ptr<scene::CoordinateTranslator> const& translator,
     std::shared_ptr<scene::ApplicationNotRespondingDetector> const& anr_detector,
     std::shared_ptr<mir::cookie::Authority> const& cookie_authority,
     std::shared_ptr<mf::InputConfigurationChanger> const& input_changer,
-    std::vector<mir::ExtensionDescription> const& extensions,
     std::shared_ptr<mg::GraphicBufferAllocator> const& allocator,
     mir::Executor& executor) :
     client_pid_(0),
     shell(shell),
-    ipc_operations(ipc_operations),
     surface_pixel_formats(surface_pixel_formats),
     display_changer(display_changer),
     observer(observer),
@@ -126,14 +119,12 @@ mf::SessionMediator::SessionMediator(
     event_sink{sink_factory->create_sink(message_sender)},
     message_sender{message_sender},
     resource_cache(resource_cache),
-    screencast(screencast),
     connection_context(connection_context),
     cursor_images(cursor_images),
     translator{translator},
     anr_detector{anr_detector},
     cookie_authority(cookie_authority),
     input_changer(input_changer),
-    extensions(extensions),
     allocator{allocator},
     executor{executor}
 {
@@ -146,7 +137,6 @@ mf::SessionMediator::~SessionMediator() noexcept
         observer->session_error(mir_client_session->name(), __PRETTY_FUNCTION__, "connection dropped without disconnect");
         shell->close_session(mir_client_session);
     }
-    destroy_screencast_sessions();
 }
 
 void mf::SessionMediator::client_pid(int pid)
@@ -169,25 +159,20 @@ void mf::SessionMediator::connect(
 
     connection_context.handle_client_connect(scene_session);
 
-    auto ipc_package = ipc_operations->connection_ipc_package();
+    // Minimal implementation to make the stub client platform load, for tests
+    static mir::ModuleProperties const stub_module_descriptor = {
+        "mir:stub-graphics",
+        1, 0, 0,
+        "not a filename"
+    };
     auto platform = response->mutable_platform();
+    auto const module = platform->mutable_graphics_module();
 
-    for (auto& data : ipc_package->ipc_data)
-        platform->add_data(data);
-
-    for (auto& ipc_fds : ipc_package->ipc_fds)
-        platform->add_fd(ipc_fds);
-
-    if (auto const graphics_module = ipc_package->graphics_module)
-    {
-        auto const module = platform->mutable_graphics_module();
-
-        module->set_name(graphics_module->name);
-        module->set_major_version(graphics_module->major_version);
-        module->set_minor_version(graphics_module->minor_version);
-        module->set_micro_version(graphics_module->micro_version);
-        module->set_file(graphics_module->file);
-    }
+    module->set_name(stub_module_descriptor.name);
+    module->set_major_version(stub_module_descriptor.major_version);
+    module->set_minor_version(stub_module_descriptor.minor_version);
+    module->set_micro_version(stub_module_descriptor.micro_version);
+    module->set_file(stub_module_descriptor.file);
 
     auto display_config = display_changer->base_configuration();
     auto protobuf_config = response->mutable_display_configuration();
@@ -198,18 +183,6 @@ void mf::SessionMediator::connect(
     for (auto pf : surface_pixel_formats)
         response->add_surface_pixel_format(static_cast<::google::protobuf::uint32>(pf));
 
-    resource_cache->save_resource(response, ipc_package);
-
-    for ( auto const& ext : extensions )
-    {
-        if (ext.version.empty()) //malformed plugin, ignore
-            continue;
-        auto e = response->add_extension();
-        e->set_name(ext.name);
-        for(auto const& v : ext.version)
-            e->add_version(v);
-    }
-
     done->Run();
 }
 
@@ -461,7 +434,6 @@ void mf::SessionMediator::submit_buffer(
 
     mfd::ProtobufBufferPacker request_msg{const_cast<mir::protobuf::Buffer*>(&request->buffer())};
     auto b = buffer_cache.at(buffer_id);
-    ipc_operations->unpack_buffer(request_msg, *b);
 
     stream->submit_buffer(std::make_shared<AutoSendBuffer>(b, executor, event_sink));
 
@@ -513,10 +485,7 @@ void mf::SessionMediator::allocate_buffers(
 
             if (req.has_flags() && req.has_native_format())
             {
-                buffer = allocator->alloc_buffer(
-                    {req.width(), req.height()},
-                    req.native_format(),
-                    req.flags());
+                BOOST_THROW_EXCEPTION(std::runtime_error("Request for removed HW buffer interface"));
             }
             else
             {
@@ -529,9 +498,7 @@ void mf::SessionMediator::allocate_buffers(
                 }
                 else
                 {
-                    //legacy route, server-selected pf and usage
-                    buffer =
-                        allocator->alloc_buffer(mg::BufferProperties{size, pf, mg::BufferUsage::hardware});
+                    BOOST_THROW_EXCEPTION(std::runtime_error("Request for removed HW buffer interface"));
                 }
             }
 
@@ -642,7 +609,6 @@ void mf::SessionMediator::disconnect(
         observer->session_disconnect_called(mir_client_session->name());
 
         shell->close_session(mir_client_session);
-        destroy_screencast_sessions();
     }
     weak_mir_client_session.reset();
     weak_scene_session.reset();
@@ -911,89 +877,35 @@ void mf::SessionMediator::cancel_base_display_configuration_preview(
 }
 
 void mf::SessionMediator::create_screencast(
-    const mir::protobuf::ScreencastParameters* parameters,
-    mir::protobuf::Screencast* protobuf_screencast,
-    google::protobuf::Closure* done)
+    const mir::protobuf::ScreencastParameters*,
+    mir::protobuf::Screencast*,
+    google::protobuf::Closure*)
 {
-    auto const msg_type = mg::BufferIpcMsgType::full_msg;
-
-    geom::Rectangle const region{
-        {parameters->region().left(), parameters->region().top()},
-        {parameters->region().width(), parameters->region().height()}
-    };
-    geom::Size const size{parameters->width(), parameters->height()};
-    MirPixelFormat const pixel_format = static_cast<MirPixelFormat>(parameters->pixel_format());
-
-    int nbuffers = 1;
-    if (parameters->has_num_buffers())
-        nbuffers = parameters->num_buffers();
-
-    MirMirrorMode mirror_mode = mir_mirror_mode_none;
-    if (parameters->has_mirror_mode())
-        mirror_mode = static_cast<MirMirrorMode>(parameters->mirror_mode());
-
-    auto screencast_session_id = screencast->create_session(region, size, pixel_format, nbuffers, mirror_mode);
-
-    if (nbuffers > 0)
-    {
-        auto buffer = screencast->capture(screencast_session_id);
-        screencast_buffer_tracker.track_buffer(screencast_session_id, buffer.get());
-        pack_protobuf_buffer(
-            *protobuf_screencast->mutable_buffer_stream()->mutable_buffer(),
-            buffer.get(),
-            msg_type);
-    }
-
-    protobuf_screencast->mutable_screencast_id()->set_value(
-        screencast_session_id.as_value());
-    protobuf_screencast->mutable_buffer_stream()->set_pixel_format(pixel_format);
-    protobuf_screencast->mutable_buffer_stream()->mutable_id()->set_value(
-        screencast_session_id.as_value());
-    protobuf_screencast->mutable_buffer_stream()->set_pixel_format(pixel_format);
-
-    done->Run();
+    BOOST_THROW_EXCEPTION((std::runtime_error{"Screencast unsupported"}));
 }
 
 void mf::SessionMediator::release_screencast(
-    const mir::protobuf::ScreencastId* protobuf_screencast_id,
+    const mir::protobuf::ScreencastId*,
     mir::protobuf::Void*,
-    google::protobuf::Closure* done)
+    google::protobuf::Closure*)
 {
-    ScreencastSessionId const screencast_session_id{
-        protobuf_screencast_id->value()};
-    screencast->destroy_session(screencast_session_id);
-    screencast_buffer_tracker.remove_session(screencast_session_id);
-    done->Run();
+    BOOST_THROW_EXCEPTION((std::runtime_error{"Screencast unsupported"}));
 }
 
 void mf::SessionMediator::screencast_buffer(
-    const mir::protobuf::ScreencastId* protobuf_screencast_id,
-    mir::protobuf::Buffer* protobuf_buffer,
-    google::protobuf::Closure* done)
+    const mir::protobuf::ScreencastId*,
+    mir::protobuf::Buffer*,
+    google::protobuf::Closure*)
 {
-    ScreencastSessionId const screencast_session_id{
-        protobuf_screencast_id->value()};
-
-    auto buffer = screencast->capture(screencast_session_id);
-    bool const already_tracked = screencast_buffer_tracker.track_buffer(screencast_session_id, buffer.get());
-    auto const msg_type = already_tracked ?
-        mg::BufferIpcMsgType::update_msg : mg::BufferIpcMsgType::full_msg;
-    pack_protobuf_buffer(*protobuf_buffer,
-                         buffer.get(),
-                         msg_type);
-
-    done->Run();
+    BOOST_THROW_EXCEPTION((std::runtime_error{"Screencast unsupported"}));
 }
 
 void mf::SessionMediator::screencast_to_buffer(
-    mir::protobuf::ScreencastRequest const* request,
+    mir::protobuf::ScreencastRequest const*,
     mir::protobuf::Void*,
-    google::protobuf::Closure* done)
+    google::protobuf::Closure*)
 {
-    ScreencastSessionId const screencast_session_id{request->id().value()};
-    auto buffer = buffer_cache.at(mg::BufferID{request->buffer_id()});
-    screencast->capture(screencast_session_id, buffer);
-    done->Run();
+    BOOST_THROW_EXCEPTION((std::runtime_error{"Screencast unsupported"}));
 }
 
 void mf::SessionMediator::create_buffer_stream(
@@ -1169,36 +1081,16 @@ void mf::SessionMediator::translate_surface_to_screen(
 }
 
 void mf::SessionMediator::platform_operation(
-    mir::protobuf::PlatformOperationMessage const* request,
-    mir::protobuf::PlatformOperationMessage* response,
-    google::protobuf::Closure* done)
+    mir::protobuf::PlatformOperationMessage const*,
+    mir::protobuf::PlatformOperationMessage*,
+    google::protobuf::Closure*)
 {
     auto mir_client_session = weak_mir_client_session.lock();
 
     if (mir_client_session.get() == nullptr)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    mg::PlatformOperationMessage platform_request;
-    unsigned int const opcode = request->opcode();
-    platform_request.data.assign(request->data().begin(),
-                                 request->data().end());
-    for (auto const request_fd : request->fd())
-    {
-        platform_request.fds.emplace_back(request_fd);
-    }
-
-    auto const& platform_response = ipc_operations->platform_operation(opcode, platform_request);
-
-    response->set_opcode(opcode);
-    response->set_data(platform_response.data.data(),
-                       platform_response.data.size());
-    for (auto fd : platform_response.fds)
-    {
-        response->add_fd(fd);
-        resource_cache->save_fd(response, mir::Fd{fd});
-    }
-
-    done->Run();
+    BOOST_THROW_EXCEPTION((std::runtime_error{"Platform operations have been removed"}));
 }
 
 void mf::SessionMediator::start_prompt_session(
@@ -1266,20 +1158,6 @@ void mf::SessionMediator::request_persistent_surface_id(
     done->Run();
 }
 
-void mf::SessionMediator::pack_protobuf_buffer(
-    protobuf::Buffer& protobuf_buffer,
-    graphics::Buffer* graphics_buffer,
-    mg::BufferIpcMsgType buffer_msg_type)
-{
-    protobuf_buffer.set_buffer_id(graphics_buffer->id().as_value());
-
-    mfd::ProtobufBufferPacker packer{&protobuf_buffer};
-    ipc_operations->pack_buffer(packer, *graphics_buffer, buffer_msg_type);
-
-    for(auto const& fd : packer.fds())
-        resource_cache->save_fd(&protobuf_buffer, fd);
-}
-
 void mf::SessionMediator::configure_buffer_stream(
     mir::protobuf::StreamConfiguration const* request,
     mir::protobuf::Void*,
@@ -1432,19 +1310,6 @@ mf::SessionMediator::unpack_and_sanitize_display_configuration(
     return config;
 }
 
-void mf::SessionMediator::destroy_screencast_sessions()
-{
-    std::vector<ScreencastSessionId> ids_to_untrack;
-    screencast_buffer_tracker.for_each_session([this, &ids_to_untrack](ScreencastSessionId id)
-    {
-        screencast->destroy_session(id);
-        ids_to_untrack.push_back(id);
-    });
-
-    for (auto const& id : ids_to_untrack)
-        screencast_buffer_tracker.remove_session(id);
-}
-
 auto mf::detail::PromptSessionStore::insert(std::shared_ptr<PromptSession> const& session) -> PromptSessionId
 {
     std::lock_guard<decltype(mutex)> lock{mutex};
