@@ -63,18 +63,18 @@ inline void clear_pending_if_unchanged(mir::optional_value<T>& pending, T& cache
 }
 
 mf::WindowWlSurfaceRole::WindowWlSurfaceRole(
+    Executor& wayland_executor,
     WlSeat* seat,
     wl_client* client,
     WlSurface* surface,
     std::shared_ptr<msh::Shell> const& shell,
     OutputManager* output_manager)
-    : destroyed{std::make_shared<bool>(false)},
-      surface{surface},
+    : surface{surface},
       client{client},
       shell{shell},
       session{get_session(client)},
       output_manager{output_manager},
-      observer{std::make_shared<WaylandSurfaceObserver>(seat, surface, this)},
+      observer{std::make_shared<WaylandSurfaceObserver>(wayland_executor, seat, surface, this)},
       params{std::make_unique<scene::SurfaceCreationParameters>(
           scene::SurfaceCreationParameters().of_type(mir_window_type_freestyle))},
       committed_min_size{0, 0},
@@ -85,9 +85,8 @@ mf::WindowWlSurfaceRole::WindowWlSurfaceRole(
 
 mf::WindowWlSurfaceRole::~WindowWlSurfaceRole()
 {
+    mark_destroyed();
     surface->clear_role();
-    observer->disconnect();
-    *destroyed = true;
     if (auto const scene_surface = weak_scene_surface.lock())
     {
         shell->destroy_surface(session, scene_surface);
@@ -295,6 +294,18 @@ void mf::WindowWlSurfaceRole::set_server_side_decorated(bool server_side_decorat
     }
 }
 
+void mir::frontend::WindowWlSurfaceRole::set_type(MirWindowType type)
+{
+    if (weak_scene_surface.lock())
+    {
+        spec().type = type;
+    }
+    else
+    {
+        params->type = type;
+    }
+}
+
 void mf::WindowWlSurfaceRole::set_state_now(MirWindowState state)
 {
     if (auto const scene_surface = weak_scene_surface.lock())
@@ -333,17 +344,17 @@ auto mf::WindowWlSurfaceRole::current_size() const -> geom::Size
     return size;
 }
 
-std::experimental::optional<geom::Size> mf::WindowWlSurfaceRole::requested_window_size()
+auto mf::WindowWlSurfaceRole::requested_window_size() const -> std::experimental::optional<geom::Size>
 {
     return observer->requested_window_size();
 }
 
-auto mf::WindowWlSurfaceRole::window_state() -> MirWindowState
+auto mf::WindowWlSurfaceRole::window_state() const -> MirWindowState
 {
     return observer->state();
 }
 
-auto mf::WindowWlSurfaceRole::is_active() -> bool
+auto mf::WindowWlSurfaceRole::is_active() const -> bool
 {
     if (auto const scene_surface = weak_scene_surface.lock())
         return scene_surface->focus_state() == mir_window_focus_state_focused;
@@ -351,7 +362,7 @@ auto mf::WindowWlSurfaceRole::is_active() -> bool
         return false;
 }
 
-std::chrono::nanoseconds mf::WindowWlSurfaceRole::latest_timestamp()
+auto mf::WindowWlSurfaceRole::latest_timestamp() const -> std::chrono::nanoseconds
 {
     return observer->latest_timestamp();
 }
@@ -367,6 +378,17 @@ void mf::WindowWlSurfaceRole::commit(WlSurfaceState const& state)
 
     if (auto const scene_surface = weak_scene_surface.lock())
     {
+        bool const is_mapped = scene_surface->visible();
+        bool const should_be_mapped = static_cast<bool>(surface->buffer_size());
+        if (!is_mapped && should_be_mapped && scene_surface->state() == mir_window_state_hidden)
+        {
+            spec().state = mir_window_state_restored;
+        }
+        else if (is_mapped && !should_be_mapped)
+        {
+            spec().state = mir_window_state_hidden;
+        }
+
         if (!committed_size || size != committed_size.value())
         {
             spec().width = size.width;
@@ -405,27 +427,6 @@ void mf::WindowWlSurfaceRole::commit(WlSurfaceState const& state)
     pending_explicit_height = std::experimental::nullopt;
 }
 
-void mf::WindowWlSurfaceRole::visiblity(bool visible)
-{
-    auto const scene_surface = weak_scene_surface.lock();
-    if (!scene_surface)
-        return;
-
-    if (scene_surface->visible() == visible)
-        return;
-
-    if (visible)
-    {
-        if (scene_surface->state() == mir_window_state_hidden)
-            spec().state = mir_window_state_restored;
-    }
-    else
-    {
-        if (scene_surface->state() != mir_window_state_hidden)
-            spec().state = mir_window_state_hidden;
-    }
-}
-
 mir::shell::SurfaceSpecification& mf::WindowWlSurfaceRole::spec()
 {
     if (!pending_changes)
@@ -453,9 +454,12 @@ void mf::WindowWlSurfaceRole::create_scene_surface()
     if (params->max_height) committed_max_size.height = params->max_height.value();
 
     // The shell isn't guaranteed to respect the requested size
+    // TODO: make initial updates atomic somehow
     auto const content_size = scene_surface->content_size();
     if (content_size != params->size)
         observer->content_resized_to(scene_surface.get(), content_size);
+    if (is_active())
+        observer->attrib_changed(scene_surface.get(), mir_window_attrib_focus, 1);
 
     // Send wl_surface.enter events for every output
     // TODO: send enter/leave when the surface actually enters and leaves outputs
@@ -471,4 +475,3 @@ void mf::WindowWlSurfaceRole::create_scene_surface()
             }
         });
 }
-
