@@ -20,11 +20,14 @@
 
 #include "wayland_utils.h"
 #include "wl_surface.h"
+#include "wl_seat.h"
 
 #include "mir/executor.h"
 #include "mir/anonymous_shm_file.h"
 #include "mir/input/keymap.h"
+#include "mir/input/xkb_mapper.h"
 #include "mir/log.h"
+#include "mir/fatal.h"
 
 #include <xkbcommon/xkbcommon.h>
 #include <boost/throw_exception.hpp>
@@ -38,13 +41,11 @@ namespace mi = mir::input;
 mf::WlKeyboard::WlKeyboard(
     wl_resource* new_resource,
     mir::input::Keymap const& initial_keymap,
-    std::function<void(WlKeyboard*)> const& on_destroy,
     std::function<std::vector<uint32_t>()> const& acquire_current_keyboard_state)
     : Keyboard(new_resource, Version<6>()),
       keymap{nullptr, &xkb_keymap_unref},
       state{nullptr, &xkb_state_unref},
       context{xkb_context_new(XKB_CONTEXT_NO_FLAGS), &xkb_context_unref},
-      on_destroy{on_destroy},
       acquire_current_keyboard_state{acquire_current_keyboard_state}
 {
     // TODO: We should really grab the keymap for the focused surface when
@@ -59,50 +60,81 @@ mf::WlKeyboard::WlKeyboard(
      */
     set_keymap(initial_keymap);
 
-    // I don't know where to get "real" rate and delay args. These are better than nothing.
+    // 25 rate and 600 delay are the default in Weston and Sway
+    // At some point we will want to make this configurable
     if (version_supports_repeat_info())
-        send_repeat_info_event(30, 200);
+        send_repeat_info_event(25, 600);
 }
 
 mf::WlKeyboard::~WlKeyboard()
 {
-    on_destroy(this);
+    if (focused_surface)
+    {
+        focused_surface.value().remove_destroy_listener(destroy_listener_id);
+    }
 }
 
-void mf::WlKeyboard::key(std::chrono::milliseconds const& ms, WlSurface* surface, int scancode, bool down)
+void mf::WlKeyboard::event(MirKeyboardEvent const* event, WlSurface& surface)
 {
-    if (*focused_surface_destroyed || focused_surface != surface)
+    bool down;
+    switch (mir_keyboard_event_action(event))
     {
-        log_warning(
-            "Sending key to wl_surface@%u even though it was not explicitly given keyboard focus",
-            wl_resource_get_id(surface->resource));
-        focussed(surface, true);
+    case mir_keyboard_action_down:
+        down = true;
+        break;
+
+    case mir_keyboard_action_up:
+        down = false;
+        break;
+
+    default:
+        return;
     }
 
     auto const serial = wl_display_next_serial(wl_client_get_display(client));
-    /*
-     * HACK! Maintain our own XKB state, so we can serialise it for
-     * wl_keyboard_send_modifiers
-     */
-    xkb_key_direction const xkb_state = down ? XKB_KEY_DOWN : XKB_KEY_UP;
-    auto const wayland_state = down ? KeyState::pressed : KeyState::released;
+    auto const timestamp = mir_input_event_get_wayland_timestamp(mir_keyboard_event_input_event(event));
+    int const scancode = mir_keyboard_event_scan_code(event);
+    uint32_t const wayland_state = down ? KeyState::pressed : KeyState::released;
+    send_key_event(serial, timestamp, scancode, wayland_state);
 
-    xkb_state_update_key(state.get(), scancode + 8, xkb_state);
-    send_key_event(serial, ms.count(), scancode, wayland_state);
-    update_modifier_state();
+    if (as_nullable_ptr(focused_surface) == &surface)
+    {
+        /*
+         * HACK! Maintain our own XKB state, so we can serialise it for
+         * wl_keyboard_send_modifiers
+         */
+        xkb_key_direction const xkb_state = down ? XKB_KEY_DOWN : XKB_KEY_UP;
+        xkb_state_update_key(state.get(), scancode + 8, xkb_state);
+
+        update_modifier_state();
+    }
+    else
+    {
+        log_warning(
+            "Sending key 0x%2.2x %s to wl_surface@%u even though it was not explicitly given keyboard focus",
+            scancode, down ? "press" : "release", wl_resource_get_id(surface.resource));
+
+        focussed(surface, true);
+    }
 }
 
-void mf::WlKeyboard::focussed(WlSurface* surface, bool should_be_focused)
+void mf::WlKeyboard::focussed(WlSurface& surface, bool should_be_focused)
 {
-    bool const is_currently_focused = (*focused_surface_destroyed == false && focused_surface == surface);
+    if (client != surface.client)
+    {
+        fatal_error("WlKeyboard::focussed() called with a surface of the wrong client");
+    }
+
+    bool const is_currently_focused = (focused_surface && &focused_surface.value() == &surface);
 
     if (should_be_focused == is_currently_focused)
         return;
 
-    if (*focused_surface_destroyed == false)
+    if (focused_surface)
     {
+        focused_surface.value().remove_destroy_listener(destroy_listener_id);
         auto const serial = wl_display_next_serial(wl_client_get_display(client));
-        send_leave_event(serial, focused_surface->raw_resource());
+        send_leave_event(serial, focused_surface.value().raw_resource());
     }
 
     if (should_be_focused)
@@ -133,17 +165,22 @@ void mf::WlKeyboard::focussed(WlSurface* surface, bool should_be_focused)
                 keyboard_state.size() * sizeof(decltype(keyboard_state)::value_type));
         }
 
+        destroy_listener_id = surface.add_destroy_listener(
+            [this, &surface]()
+            {
+                focussed(surface, false);
+            });
+
         auto const serial = wl_display_next_serial(wl_client_get_display(client));
-        send_enter_event(serial, surface->raw_resource(), &key_state);
+        send_enter_event(serial, surface.raw_resource(), &key_state);
         wl_array_release(&key_state);
 
-        focused_surface = surface;
-        focused_surface_destroyed = surface->destroyed_flag();
+        focused_surface = mw::make_weak(&surface);
     }
     else
     {
-        focused_surface = nullptr;
-        focused_surface_destroyed = std::make_shared<bool>(true);
+        destroy_listener_id = {};
+        focused_surface = {};
     }
 }
 
@@ -159,26 +196,6 @@ void mf::WlKeyboard::update_keyboard_state(std::vector<uint32_t> const& keyboard
     update_modifier_state();
 }
 
-void mf::WlKeyboard::set_keymap(char const* const buffer, size_t length)
-{
-    mir::AnonymousShmFile shm_buffer{length};
-    memcpy(shm_buffer.base_ptr(), buffer, length);
-
-    send_keymap_event(KeymapFormat::xkb_v1,
-                      Fd{IntOwnedFd{shm_buffer.fd()}},
-                      length);
-
-    keymap = decltype(keymap)(xkb_keymap_new_from_buffer(
-        context.get(),
-        buffer,
-        length,
-        XKB_KEYMAP_FORMAT_TEXT_V1,
-        XKB_KEYMAP_COMPILE_NO_FLAGS),
-        &xkb_keymap_unref);
-
-    state = decltype(state)(xkb_state_new(keymap.get()), &xkb_state_unref);
-}
-
 void mf::WlKeyboard::set_keymap(mi::Keymap const& new_keymap)
 {
     xkb_rule_names const names = {
