@@ -74,18 +74,20 @@ void mf::WlSubcompositorInstance::get_subsurface(
 mf::WlSubsurface::WlSubsurface(wl_resource* new_subsurface, WlSurface* surface, WlSurface* parent_surface)
     : wayland::Subsurface(new_subsurface, Version<1>()),
       surface{surface},
-      parent{parent_surface->add_child(this)},
-      parent_destroyed{parent_surface->destroyed_flag()},
+      parent{parent_surface},
       synchronized_{true}
 {
+    parent_surface->add_subsurface(this);
     surface->set_role(this);
     surface->pending_invalidate_surface_data();
 }
 
 mf::WlSubsurface::~WlSubsurface()
 {
-    // unique pointer automatically removes `this` from parent child list
-
+    if (parent)
+    {
+        parent.value().remove_subsurface(this);
+    }
     surface->clear_role();
     refresh_surface_data_now();
 }
@@ -94,17 +96,31 @@ void mf::WlSubsurface::populate_surface_data(std::vector<shell::StreamSpecificat
                                              std::vector<mir::geometry::Rectangle>& input_shape_accumulator,
                                              geometry::Displacement const& parent_offset) const
 {
-    surface->populate_surface_data(buffer_streams, input_shape_accumulator, parent_offset);
+    if (surface->buffer_size())
+    {
+        // surface is mapped
+        surface->populate_surface_data(buffer_streams, input_shape_accumulator, parent_offset);
+    }
 }
 
-bool mf::WlSubsurface::synchronized() const
+auto mf::WlSubsurface::total_offset() const -> geom::Displacement
 {
-    return synchronized_ || parent->synchronized();
+    return parent ?
+        parent.value().offset() :
+        geom::Displacement{};
+}
+
+auto mf::WlSubsurface::synchronized() const -> bool
+{
+    bool const parent_synchronized = parent && parent.value().synchronized();
+    return synchronized_ || parent_synchronized;
 }
 
 auto mf::WlSubsurface::scene_surface() const -> std::experimental::optional<std::shared_ptr<scene::Surface>>
 {
-    return parent->scene_surface();
+    return parent ?
+        parent.value().scene_surface() :
+        std::experimental::nullopt;
 }
 
 void mf::WlSubsurface::parent_has_committed()
@@ -116,9 +132,9 @@ void mf::WlSubsurface::parent_has_committed()
     }
 }
 
-mf::WlSurface::Position mf::WlSubsurface::transform_point(geom::Point point)
+auto mf::WlSubsurface::subsurface_at(geom::Point point) -> std::experimental::optional<WlSurface*>
 {
-    return surface->transform_point(point);
+    return surface->subsurface_at(point);
 }
 
 void mf::WlSubsurface::set_position(int32_t x, int32_t y)
@@ -155,8 +171,10 @@ void mf::WlSubsurface::destroy()
 
 void mf::WlSubsurface::refresh_surface_data_now()
 {
-    if (!*parent_destroyed)
-        parent->refresh_surface_data_now();
+    if (parent)
+    {
+        parent.value().refresh_surface_data_now();
+    }
 }
 
 void mf::WlSubsurface::commit(WlSurfaceState const& state)
@@ -166,10 +184,22 @@ void mf::WlSubsurface::commit(WlSurfaceState const& state)
 
     cached_state.value().update_from(state);
 
+    if (cached_state.value().buffer)
+    {
+        auto const currently_mapped = static_cast<bool>(surface->buffer_size());
+        auto const pending_mapped = cached_state.value().buffer.value() != nullptr;
+        if (currently_mapped != pending_mapped)
+        {
+            cached_state.value().invalidate_surface_data();
+        }
+    }
+
     if (synchronized())
     {
-        if (cached_state.value().surface_data_needs_refresh() && !*parent_destroyed)
-            parent->pending_invalidate_surface_data();
+        if (cached_state.value().surface_data_needs_refresh() && parent)
+        {
+            parent.value().pending_invalidate_surface_data();
+        }
     }
     else
     {
@@ -179,9 +209,3 @@ void mf::WlSubsurface::commit(WlSurfaceState const& state)
         cached_state = std::experimental::nullopt;
     }
 }
-
-void mf::WlSubsurface::visiblity(bool visible)
-{
-    (void)visible;
-    log_warning("TODO: wl_subsurface.visiblity not implemented");
-}
