@@ -58,13 +58,13 @@ public:
                                std::vector<mir::geometry::Rectangle>& input_shape_accumulator,
                                geometry::Displacement const& parent_offset) const;
 
-    geometry::Displacement total_offset() const override { return parent->total_offset(); }
-    bool synchronized() const override;
+    auto total_offset() const -> geometry::Displacement override;
+    auto synchronized() const -> bool override;
     auto scene_surface() const -> std::experimental::optional<std::shared_ptr<scene::Surface>> override;
 
     void parent_has_committed();
 
-    WlSurface::Position transform_point(geometry::Point point);
+    auto subsurface_at(geometry::Point point) -> std::experimental::optional<WlSurface*>;
 
 private:
     void set_position(int32_t x, int32_t y) override;
@@ -77,13 +77,10 @@ private:
 
     void refresh_surface_data_now() override;
     virtual void commit(WlSurfaceState const& state) override;
-    virtual void visiblity(bool visible) override;
 
     WlSurface* const surface;
-    // manages parent/child relationship, but does not manage parent's memory
-    // see WlSurface::add_child() for details
-    std::unique_ptr<WlSurface, std::function<void(WlSurface*)>> const parent;
-    std::shared_ptr<bool> const parent_destroyed;
+    /// This class is responsible for removing itself from the parent's children list when needed
+    wayland::Weak<WlSurface> const parent;
     bool synchronized_;
     std::experimental::optional<WlSurfaceState> cached_state;
 };
