@@ -1,5 +1,5 @@
 /*
- * Copyright © 2018 Canonical Ltd.
+ * Copyright © 2018-2021 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3,
@@ -22,7 +22,6 @@
 #include "wl_surface_role.h"
 #include "wl_subcompositor.h"
 #include "wl_region.h"
-#include "wlshmbuffer.h"
 #include "deleted_for_resource.h"
 
 #include "wayland_wrapper.h"
@@ -34,12 +33,14 @@
 #include "mir/frontend/wayland.h"
 #include "mir/compositor/buffer_stream.h"
 #include "mir/executor.h"
-#include "mir/graphics/wayland_allocator.h"
+#include "mir/graphics/graphic_buffer_allocator.h"
+#include "mir/scene/surface.h"
 #include "mir/shell/surface_specification.h"
 #include "mir/log.h"
 
 #include <algorithm>
 #include <boost/throw_exception.hpp>
+#include <wayland-server-protocol.h>
 
 namespace mf = mir::frontend;
 namespace geom = mir::geometry;
@@ -84,15 +85,14 @@ bool mf::WlSurfaceState::surface_data_needs_refresh() const
 mf::WlSurface::WlSurface(
     wl_resource* new_resource,
     std::shared_ptr<Executor> const& executor,
-    std::shared_ptr<graphics::WaylandAllocator> const& allocator)
+    std::shared_ptr<graphics::GraphicBufferAllocator> const& allocator)
     : Surface(new_resource, Version<4>()),
         session{get_session(client)},
         stream{session->create_buffer_stream({{}, mir_pixel_format_invalid, graphics::BufferUsage::undefined})},
         allocator{allocator},
         executor{executor},
         null_role{this},
-        role{&null_role},
-        destroyed{std::make_shared<bool>(false)}
+        role{&null_role}
 {
     // wl_surface is specified to act in mailbox mode
     stream->allow_framedropping(true);
@@ -100,16 +100,6 @@ mf::WlSurface::WlSurface(
 
 mf::WlSurface::~WlSurface()
 {
-    *destroyed = true;
-
-    // so that unregister_destroy_listener calls invoked from destroy listeners don't screw up the iterator
-    auto listeners = move(destroy_listeners);
-    destroy_listeners.clear();
-    for (auto listener: listeners)
-    {
-        listener.second();
-    }
-
     role->destroy();
     session->destroy_buffer_stream(stream);
 }
@@ -119,23 +109,27 @@ bool mf::WlSurface::synchronized() const
     return role->synchronized();
 }
 
-mf::WlSurface::Position mf::WlSurface::transform_point(geom::Point point)
+auto mf::WlSurface::subsurface_at(geom::Point point) -> std::experimental::optional<WlSurface*>
 {
+    if (!buffer_size_)
+    {
+        // surface not mapped
+        return std::experimental::nullopt;
+    }
     point = point - offset_;
     // loop backwards so the first subsurface we find that accepts the input is the topmost one
     for (auto child_it = children.rbegin(); child_it != children.rend(); ++child_it)
     {
-        auto result = (*child_it)->transform_point(point);
-        if (result.is_in_input_region)
+        if (auto result = (*child_it)->subsurface_at(point))
             return result;
     }
     geom::Rectangle surface_rect = {geom::Point{}, buffer_size_.value_or(geom::Size{})};
-    for (auto& rect : input_shape.value_or(std::vector<geom::Rectangle>{{{}, buffer_size_.value_or(geom::Size{})}}))
+    for (auto& rect : input_shape.value_or(std::vector<geom::Rectangle>{surface_rect}))
     {
         if (rect.intersection_with(surface_rect).contains(point))
-            return {point, this, true};
+            return this;
     }
-    return {point, this, false};
+    return std::experimental::nullopt;
 }
 
 auto mf::WlSurface::scene_surface() const -> std::experimental::optional<std::shared_ptr<scene::Surface>>
@@ -160,22 +154,25 @@ void mf::WlSurface::set_pending_offset(std::experimental::optional<geom::Displac
     pending.offset = offset;
 }
 
-std::unique_ptr<mf::WlSurface, std::function<void(mf::WlSurface*)>> mf::WlSurface::add_child(WlSubsurface* child)
+void mf::WlSurface::add_subsurface(WlSubsurface* child)
 {
-    children.push_back(child);
+    if (std::find(children.begin(), children.end(), child) != children.end())
+    {
+        log_warning("Subsurface %p added to surface %p multiple times", static_cast<void*>(child), static_cast<void*>(this));
+        return;
+    }
 
-    return std::unique_ptr<WlSurface, std::function<void(WlSurface*)>>(
-        this,
-        [child=child, destroyed=destroyed](WlSurface* self)
-        {
-            if (*destroyed)
-                return;
-            // remove the child from the vector
-            self->children.erase(std::remove(self->children.begin(),
-                                             self->children.end(),
-                                             child),
-                                 self->children.end());
-        });
+    children.push_back(child);
+}
+
+void mf::WlSurface::remove_subsurface(WlSubsurface* child)
+{
+    children.erase(
+        std::remove(
+            children.begin(),
+            children.end(),
+            child),
+        children.end());
 }
 
 void mf::WlSurface::refresh_surface_data_now()
@@ -199,6 +196,15 @@ void mf::WlSurface::populate_surface_data(std::vector<shell::StreamSpecification
             rect = rect.intersection_with(surface_rect); // clip to surface
             input_shape_accumulator.push_back(rect);
         }
+
+        // If we have an explicity specified empty input shape all inport should be ignored
+        // however, if we give Mir an empty vector it will use a default input shape
+        // therefore we add a zero size rect to the vector
+        // TODO: sort this whole mess out
+        if (input_shape.value().empty())
+        {
+            input_shape_accumulator.push_back({{}, {}});
+        }
     }
     else
     {
@@ -211,16 +217,6 @@ void mf::WlSurface::populate_surface_data(std::vector<shell::StreamSpecification
     }
 }
 
-void mf::WlSurface::add_destroy_listener(void const* key, std::function<void()> listener)
-{
-    destroy_listeners[key] = listener;
-}
-
-void mf::WlSurface::remove_destroy_listener(void const* key)
-{
-    destroy_listeners.erase(key);
-}
-
 mf::WlSurface* mf::WlSurface::from(wl_resource* resource)
 {
     void* raw_surface = wl_resource_get_user_data(resource);
@@ -243,7 +239,6 @@ void mf::WlSurface::send_frame_callbacks()
 
 void mf::WlSurface::destroy()
 {
-    *destroyed = true;
     destroy_wayland_object();
 }
 
@@ -254,8 +249,6 @@ void mf::WlSurface::attach(std::experimental::optional<wl_resource*> const& buff
         mir::log_warning("Client requested unimplemented non-zero attach offset. Rendering will be incorrect.");
     }
 
-    role->visiblity(!!buffer);
-
     pending.buffer = buffer.value_or(nullptr);
 }
 
@@ -303,6 +296,36 @@ void mf::WlSurface::set_input_region(std::experimental::optional<wl_resource*> c
     }
 }
 
+namespace
+{
+MirPixelFormat wl_format_to_mir_format(uint32_t format)
+{
+    switch (format)
+    {
+        case WL_SHM_FORMAT_ARGB8888:
+            return mir_pixel_format_argb_8888;
+        case WL_SHM_FORMAT_XRGB8888:
+            return mir_pixel_format_xrgb_8888;
+        case WL_SHM_FORMAT_RGBA4444:
+            return mir_pixel_format_rgba_4444;
+        case WL_SHM_FORMAT_RGBA5551:
+            return mir_pixel_format_rgba_5551;
+        case WL_SHM_FORMAT_RGB565:
+            return mir_pixel_format_rgb_565;
+        case WL_SHM_FORMAT_RGB888:
+            return mir_pixel_format_rgb_888;
+        case WL_SHM_FORMAT_BGR888:
+            return mir_pixel_format_bgr_888;
+        case WL_SHM_FORMAT_XBGR8888:
+            return mir_pixel_format_xbgr_8888;
+        case WL_SHM_FORMAT_ABGR8888:
+            return mir_pixel_format_abgr_8888;
+        default:
+            return mir_pixel_format_invalid;
+    }
+}
+}
+
 void mf::WlSurface::commit(WlSurfaceState const& state)
 {
     // We're going to lose the value of state, so copy the frame_callbacks first. We have to maintain a list of
@@ -331,21 +354,36 @@ void mf::WlSurface::commit(WlSurfaceState const& state)
         }
         else
         {
-            auto const executor_send_frame_callbacks = [this, executor = executor, destroyed = destroyed]()
+            auto const executor_send_frame_callbacks = [executor = executor, weak_self = mw::make_weak(this)]()
                 {
-                    executor->spawn(run_unless(
-                        destroyed,
-                        [this]()
+                    executor->spawn([weak_self]()
                         {
-                            send_frame_callbacks();
-                        }));
+                            if (weak_self)
+                            {
+                                weak_self.value().send_frame_callbacks();
+                            }
+                        });
                 };
 
             std::shared_ptr<graphics::Buffer> mir_buffer;
 
-            if (wl_shm_buffer_get(buffer))
+            if (auto const shm_buffer = wl_shm_buffer_get(buffer))
             {
-                mir_buffer = WlShmBuffer::mir_buffer_from_wl_buffer(
+                auto const stride = wl_shm_buffer_get_stride(shm_buffer);
+                auto const width = wl_shm_buffer_get_width(shm_buffer);
+                auto const format = wl_format_to_mir_format(wl_shm_buffer_get_format(shm_buffer));
+                if (stride < width * MIR_BYTES_PER_PIXEL(format)) {
+                    wl_resource_post_error(
+                        buffer,
+                        WL_SHM_ERROR_INVALID_STRIDE,
+                        "Stride (%u) is less than width × bytes per pixel (%u×%u). "
+                        "Did you accidentally specify stride in pixels?",
+                        stride, width, MIR_BYTES_PER_PIXEL(format));
+
+                    BOOST_THROW_EXCEPTION((
+                                              std::runtime_error{"Buffer has invalid stride"}));
+                }
+                mir_buffer = allocator->buffer_from_shm(
                     buffer,
                     executor,
                     std::move(executor_send_frame_callbacks));
@@ -427,6 +465,19 @@ void mf::WlSurface::set_buffer_scale(int32_t scale)
     pending.scale = scale;
 }
 
+auto mf::WlSurface::confine_pointer_state() const -> MirPointerConfinementState
+{
+    if (auto const maybe_scene_surface = scene_surface())
+    {
+        if (auto const scene_surface = *maybe_scene_surface)
+        {
+            return scene_surface->confine_pointer_state();
+        }
+    }
+
+    return mir_pointer_unconfined;
+}
+
 mf::NullWlSurfaceRole::NullWlSurfaceRole(WlSurface* surface) :
     surface{surface}
 {
@@ -438,5 +489,4 @@ auto mf::NullWlSurfaceRole::scene_surface() const -> std::experimental::optional
 }
 void mf::NullWlSurfaceRole::refresh_surface_data_now() {}
 void mf::NullWlSurfaceRole::commit(WlSurfaceState const& state) { surface->commit(state); }
-void mf::NullWlSurfaceRole::visiblity(bool /*visible*/) {}
 void mf::NullWlSurfaceRole::destroy() {}
