@@ -1,5 +1,5 @@
 /*
- * Copyright © 2018 Canonical Ltd.
+ * Copyright © 2018-2021 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3,
@@ -36,7 +36,7 @@ class Executor;
 
 namespace graphics
 {
-class WaylandAllocator;
+class GraphicBufferAllocator;
 }
 namespace scene
 {
@@ -80,6 +80,7 @@ struct WlSurfaceState
     // if it's nullptr, there is a new buffer and it is a null buffer, which should replace the current buffer
     std::experimental::optional<wl_resource*> buffer;
 
+    std::experimental::optional<int> scale;
     std::experimental::optional<geometry::Displacement> offset;
     std::experimental::optional<std::experimental::optional<std::vector<geometry::Rectangle>>> input_shape;
     std::vector<std::shared_ptr<Callback>> frame_callbacks;
@@ -99,7 +100,6 @@ public:
     auto scene_surface() const -> std::experimental::optional<std::shared_ptr<scene::Surface>> override;
     void refresh_surface_data_now() override;
     void commit(WlSurfaceState const& state) override;
-    void visiblity(bool /*visible*/) override;
     void destroy() override;
 
 private:
@@ -109,40 +109,32 @@ private:
 class WlSurface : public wayland::Surface
 {
 public:
-    struct Position
-    {
-        mir::geometry::Point position;
-        WlSurface* surface;
-        bool is_in_input_region;
-    };
-
     WlSurface(wl_resource* new_resource,
               std::shared_ptr<mir::Executor> const& executor,
-              std::shared_ptr<mir::graphics::WaylandAllocator> const& allocator);
+              std::shared_ptr<graphics::GraphicBufferAllocator> const& allocator);
 
     ~WlSurface();
 
-    std::shared_ptr<bool> destroyed_flag() const { return destroyed; }
     geometry::Displacement offset() const { return offset_; }
     geometry::Displacement total_offset() const { return offset_ + role->total_offset(); }
     std::experimental::optional<geometry::Size> buffer_size() const { return buffer_size_; }
     bool synchronized() const;
-    Position transform_point(geometry::Point point);
+    auto subsurface_at(geometry::Point point) -> std::experimental::optional<WlSurface*>;
     wl_resource* raw_resource() const { return resource; }
     auto scene_surface() const -> std::experimental::optional<std::shared_ptr<scene::Surface>>;
 
     void set_role(WlSurfaceRole* role_);
     void clear_role();
     void set_pending_offset(std::experimental::optional<geometry::Displacement> const& offset);
-    std::unique_ptr<WlSurface, std::function<void(WlSurface*)>> add_child(WlSubsurface* child);
+    void add_subsurface(WlSubsurface* child);
+    void remove_subsurface(WlSubsurface* child);
     void refresh_surface_data_now();
     void pending_invalidate_surface_data() { pending.invalidate_surface_data(); }
     void populate_surface_data(std::vector<shell::StreamSpecification>& buffer_streams,
                                std::vector<mir::geometry::Rectangle>& input_shape_accumulator,
                                geometry::Displacement const& parent_offset) const;
     void commit(WlSurfaceState const& state);
-    void add_destroy_listener(void const* key, std::function<void()> listener);
-    void remove_destroy_listener(void const* key);
+    auto confine_pointer_state() const -> MirPointerConfinementState;
 
     std::shared_ptr<scene::Session> const session;
     std::shared_ptr<compositor::BufferStream> const stream;
@@ -150,7 +142,7 @@ public:
     static WlSurface* from(wl_resource* resource);
 
 private:
-    std::shared_ptr<mir::graphics::WaylandAllocator> const allocator;
+    std::shared_ptr<mir::graphics::GraphicBufferAllocator> const allocator;
     std::shared_ptr<mir::Executor> const executor;
 
     NullWlSurfaceRole null_role;
@@ -162,8 +154,6 @@ private:
     std::experimental::optional<geometry::Size> buffer_size_;
     std::vector<std::shared_ptr<WlSurfaceState::Callback>> frame_callbacks;
     std::experimental::optional<std::vector<mir::geometry::Rectangle>> input_shape;
-    std::map<void const*, std::function<void()>> destroy_listeners;
-    std::shared_ptr<bool> const destroyed;
 
     void send_frame_callbacks();
 
