@@ -27,6 +27,8 @@
 #include <functional>
 #include <chrono>
 
+struct MirTouchEvent;
+
 namespace mir
 {
 class Executor;
@@ -38,29 +40,37 @@ class WlSurface;
 class WlTouch : public wayland::Touch
 {
 public:
-    WlTouch(
-        wl_resource* new_resource,
-        std::function<void(WlTouch*)> const& on_destroy);
+    WlTouch(wl_resource* new_resource);
 
     ~WlTouch();
 
+    /// Convert the Mir event into Wayland events and send them to the client. root_surface is the one that received
+    /// the Mir event, but the final Wayland event may be sent to a subsurface.
+    void event(MirTouchEvent const* event, WlSurface& root_surface);
+
+private:
+    struct TouchedSurface
+    {
+        wayland::Weak<WlSurface> surface;
+        wayland::DestroyListenerId destroy_listener_id;
+    };
+
+    /// Maps touch IDs to the surfaces the touch is on
+    std::unordered_map<int32_t, TouchedSurface> touch_id_to_surface;
+    bool needs_frame{false};
+
     void down(
+        uint32_t serial,
         std::chrono::milliseconds const& ms,
         int32_t touch_id,
-        WlSurface* parent,
-        geometry::Point const& position_on_parent);
+        WlSurface& root_surface,
+        std::pair<float, float> const& root_position);
     void motion(
         std::chrono::milliseconds const& ms,
         int32_t touch_id,
-        WlSurface* parent,
-        geometry::Point const& position_on_parent);
-    void up(std::chrono::milliseconds const& ms, int32_t touch_id);
-    void frame();
-
-private:
-    std::function<void(WlTouch*)> on_destroy;
-    std::unordered_map<int32_t, WlSurface*> focused_surface_for_ids;
-    bool can_send_frame{false};
+        std::pair<float, float> const& root_position);
+    void up(uint32_t serial, std::chrono::milliseconds const& ms, int32_t touch_id);
+    void maybe_frame();
 
     void release() override;
 };
