@@ -24,6 +24,8 @@
 #include "mir/frontend/mir_client_session.h"
 #include "mir/frontend/wayland.h"
 #include "mir/shell/surface_specification.h"
+#include "mir/shell/shell.h"
+#include "mir/scene/surface.h"
 #include "mir/log.h"
 
 #include <boost/throw_exception.hpp>
@@ -39,7 +41,7 @@ namespace mir
 namespace frontend
 {
 
-class XdgSurfaceStable : wayland::XdgSurface
+class XdgSurfaceStable : mw::XdgSurface
 {
 public:
     static XdgSurfaceStable* from(wl_resource* surface);
@@ -58,23 +60,22 @@ public:
 
     void send_configure();
 
-    std::experimental::optional<WindowWlSurfaceRole*> const& window_role();
+    mw::Weak<WindowWlSurfaceRole> const& window_role();
 
-    using wayland::XdgSurface::client;
-    using wayland::XdgSurface::resource;
+    using mw::XdgSurface::client;
+    using mw::XdgSurface::resource;
 
 private:
     void set_window_role(WindowWlSurfaceRole* role);
 
-    std::experimental::optional<WindowWlSurfaceRole*> window_role_;
-    std::shared_ptr<bool> window_role_destroyed;
+    mw::Weak<WindowWlSurfaceRole> window_role_;
     WlSurface* const surface;
 
 public:
     XdgShellStable const& xdg_shell;
 };
 
-class XdgToplevelStable : wayland::XdgToplevel, public WindowWlSurfaceRole
+class XdgToplevelStable : mw::XdgToplevel, public WindowWlSurfaceRole
 {
 public:
     XdgToplevelStable(
@@ -111,7 +112,7 @@ private:
     XdgSurfaceStable* const xdg_surface;
 };
 
-class XdgPositionerStable : public wayland::XdgPositioner, public shell::SurfaceSpecification
+class XdgPositionerStable : public mw::XdgPositioner, public shell::SurfaceSpecification
 {
 public:
     XdgPositionerStable(wl_resource* new_resource);
@@ -130,7 +131,7 @@ private:
 }
 namespace mf = mir::frontend;  // Keep CLion's parsing happy
 
-class mf::XdgShellStable::Instance : public wayland::XdgWmBase
+class mf::XdgShellStable::Instance : public mw::XdgWmBase
 {
 public:
     Instance(wl_resource* new_resource, mf::XdgShellStable* shell);
@@ -148,10 +149,12 @@ private:
 
 mf::XdgShellStable::XdgShellStable(
     struct wl_display* display,
+    Executor& wayland_executor,
     std::shared_ptr<msh::Shell> shell,
     WlSeat& seat,
     OutputManager* output_manager)
     : Global(display, Version<1>()),
+      wayland_executor{wayland_executor},
       shell{shell},
       seat{seat},
       output_manager{output_manager}
@@ -195,7 +198,7 @@ void mf::XdgShellStable::Instance::pong(uint32_t serial)
 mf::XdgSurfaceStable* mf::XdgSurfaceStable::from(wl_resource* surface)
 {
     auto* tmp = wl_resource_get_user_data(surface);
-    return static_cast<XdgSurfaceStable*>(static_cast<wayland::XdgSurface*>(tmp));
+    return static_cast<XdgSurfaceStable*>(static_cast<mw::XdgSurface*>(tmp));
 }
 
 mf::XdgSurfaceStable::XdgSurfaceStable(wl_resource* new_resource, WlSurface* surface, XdgShellStable const& xdg_shell)
@@ -225,11 +228,15 @@ void mf::XdgSurfaceStable::get_popup(
     if (parent_surface)
     {
         XdgSurfaceStable* parent_xdg_surface = XdgSurfaceStable::from(parent_surface.value());
-        std::experimental::optional<WindowWlSurfaceRole*> parent_window_role = parent_xdg_surface->window_role();
+        auto const& parent_window_role = parent_xdg_surface->window_role();
         if (parent_window_role)
-            parent_role = static_cast<WlSurfaceRole*>(parent_window_role.value());
+        {
+            parent_role = static_cast<WlSurfaceRole*>(&parent_window_role.value());
+        }
         else
+        {
             log_warning("Parent window of a popup has no role");
+        }
     }
 
     auto popup = new XdgPopupStable{new_popup, this, parent_role, positioner, surface};
@@ -238,11 +245,11 @@ void mf::XdgSurfaceStable::get_popup(
 
 void mf::XdgSurfaceStable::set_window_geometry(int32_t x, int32_t y, int32_t width, int32_t height)
 {
-    if (auto& role = window_role())
+    if (window_role_)
     {
-        role.value()->set_pending_offset(geom::Displacement{-x, -y});
-        role.value()->set_pending_width(geom::Width{width});
-        role.value()->set_pending_height(geom::Height{height});
+        window_role_.value().set_pending_offset(geom::Displacement{-x, -y});
+        window_role_.value().set_pending_width(geom::Width{width});
+        window_role_.value().set_pending_height(geom::Height{height});
     }
 }
 
@@ -254,28 +261,23 @@ void mf::XdgSurfaceStable::ack_configure(uint32_t serial)
 
 void mf::XdgSurfaceStable::send_configure()
 {
-    auto const serial = wl_display_next_serial(wl_client_get_display(wayland::XdgSurface::client));
+    auto const serial = wl_display_next_serial(wl_client_get_display(mw::XdgSurface::client));
     send_configure_event(serial);
 }
 
-std::experimental::optional<mf::WindowWlSurfaceRole*> const& mf::XdgSurfaceStable::window_role()
+mw::Weak<mf::WindowWlSurfaceRole> const& mf::XdgSurfaceStable::window_role()
 {
-    if (window_role_ && *window_role_destroyed)
-    {
-        window_role_ = std::experimental::nullopt;
-        window_role_destroyed = nullptr;
-    }
-
     return window_role_;
 }
 
 void mf::XdgSurfaceStable::set_window_role(WindowWlSurfaceRole* role)
 {
-    if (window_role())
+    if (window_role_)
+    {
         log_warning("XdgSurfaceStable::window_role set multiple times");
+    }
 
-    window_role_ = role;
-    window_role_destroyed = role->destroyed_flag();
+    window_role_ = mw::make_weak(role);
 }
 
 // XdgPopupStable
@@ -286,21 +288,28 @@ mf::XdgPopupStable::XdgPopupStable(
     std::experimental::optional<WlSurfaceRole*> parent_role,
     struct wl_resource* positioner,
     WlSurface* surface)
-    : wayland::XdgPopup(new_resource, Version<1>()),
+    : mw::XdgPopup(new_resource, Version<1>()),
       WindowWlSurfaceRole(
+          xdg_surface->xdg_shell.wayland_executor,
           &xdg_surface->xdg_shell.seat,
           mw::XdgPopup::client,
           surface,
           xdg_surface->xdg_shell.shell,
           xdg_surface->xdg_shell.output_manager),
+      shell{xdg_surface->xdg_shell.shell},
       xdg_surface{xdg_surface}
 {
     auto specification = static_cast<mir::shell::SurfaceSpecification*>(
                                 static_cast<XdgPositionerStable*>(
-                                    static_cast<wayland::XdgPositioner*>(
+                                    static_cast<mw::XdgPositioner*>(
                                         wl_resource_get_user_data(positioner))));
 
-    specification->type = mir_window_type_freestyle;
+    if (specification->aux_rect)
+    {
+        aux_rect = specification->aux_rect.value();
+    }
+
+    specification->type = mir_window_type_gloss;
     specification->placement_hints = mir_placement_hints_slide_any;
     if (parent_role)
     {
@@ -313,10 +322,37 @@ mf::XdgPopupStable::XdgPopupStable(
     apply_spec(*specification);
 }
 
+void mf::XdgPopupStable::set_aux_rect_offset_now(geom::Displacement const& new_aux_rect_offset)
+{
+    if (aux_rect_offset == new_aux_rect_offset)
+    {
+        return;
+    }
+
+    aux_rect_offset = new_aux_rect_offset;
+
+    shell::SurfaceSpecification spec;
+    spec.aux_rect = aux_rect;
+    spec.aux_rect.value().top_left += aux_rect_offset;
+
+    auto const scene_surface_{scene_surface()};
+    if (scene_surface_)
+    {
+        shell->modify_surface(
+            scene_surface_.value()->session().lock(),
+            scene_surface_.value(),
+            spec);
+    }
+    else
+    {
+        apply_spec(spec);
+    }
+}
+
 void mf::XdgPopupStable::grab(struct wl_resource* seat, uint32_t serial)
 {
     (void)seat, (void)serial;
-    // TODO
+    set_type(mir_window_type_menu);
 }
 
 void mf::XdgPopupStable::destroy()
@@ -324,9 +360,15 @@ void mf::XdgPopupStable::destroy()
     destroy_wayland_object();
 }
 
-void mf::XdgPopupStable::handle_resize(const std::experimental::optional<geometry::Point>& new_top_left,
+void mf::XdgPopupStable::handle_resize(const std::experimental::optional<geometry::Point>& new_top_left_,
                                        const geometry::Size& new_size)
 {
+    auto new_top_left{new_top_left_};
+    if (new_top_left)
+    {
+        new_top_left.value() -= aux_rect_offset;
+    }
+
     bool const needs_configure = (new_top_left != cached_top_left) || (new_size != cached_size);
 
     if (new_top_left)
@@ -351,7 +393,7 @@ void mf::XdgPopupStable::handle_close_request()
 
 auto mf::XdgPopupStable::from(wl_resource* resource) -> XdgPopupStable*
 {
-    auto popup = dynamic_cast<XdgPopupStable*>(wayland::XdgPopup::from(resource));
+    auto popup = dynamic_cast<XdgPopupStable*>(mw::XdgPopup::from(resource));
     if (!popup)
         BOOST_THROW_EXCEPTION(std::runtime_error("Invalid resource given to XdgPopupStable::from()"));
     return popup;
@@ -362,8 +404,9 @@ auto mf::XdgPopupStable::from(wl_resource* resource) -> XdgPopupStable*
 mf::XdgToplevelStable::XdgToplevelStable(wl_resource* new_resource, XdgSurfaceStable* xdg_surface, WlSurface* surface)
     : mw::XdgToplevel(new_resource, Version<1>()),
       WindowWlSurfaceRole(
+          xdg_surface->xdg_shell.wayland_executor,
           &xdg_surface->xdg_shell.seat,
-          wayland::XdgToplevel::client,
+          mw::XdgToplevel::client,
           surface,
           xdg_surface->xdg_shell.shell,
           xdg_surface->xdg_shell.output_manager),
@@ -560,7 +603,7 @@ void mf::XdgToplevelStable::send_toplevel_configure()
 mf::XdgToplevelStable* mf::XdgToplevelStable::from(wl_resource* surface)
 {
     auto* tmp = wl_resource_get_user_data(surface);
-    return static_cast<XdgToplevelStable*>(static_cast<wayland::XdgToplevel*>(tmp));
+    return static_cast<XdgToplevelStable*>(static_cast<mw::XdgToplevel*>(tmp));
 }
 
 // XdgPositionerStable
@@ -698,15 +741,15 @@ auto mf::XdgShellStable::get_window(wl_resource* surface) -> std::shared_ptr<sce
     if (mw::XdgSurface::is_instance(surface))
     {
         auto const xdgsurface = XdgSurfaceStable::from(surface);
-        if (auto const role = xdgsurface->window_role())
+        if (auto const& role = xdgsurface->window_role())
         {
-            if (auto const scene_surface = role.value()->scene_surface())
+            if (auto const scene_surface = role.value().scene_surface())
             {
                 return scene_surface.value();
             }
         }
 
-        log_debug("No window currently associated with wayland::XdgSurface %p", surface);
+        log_debug("No window currently associated with wayland::XdgSurface %p", static_cast<void*>(surface));
     }
 
     return {};
