@@ -38,7 +38,7 @@ namespace mir
 namespace frontend
 {
 
-class XdgSurfaceV6 : wayland::XdgSurfaceV6
+class XdgSurfaceV6 : mw::XdgSurfaceV6
 {
 public:
     static XdgSurfaceV6* from(wl_resource* surface);
@@ -54,23 +54,22 @@ public:
 
     void send_configure();
 
-    std::experimental::optional<WindowWlSurfaceRole*> const& window_role();
+    mw::Weak<WindowWlSurfaceRole> const& window_role();
 
-    using wayland::XdgSurfaceV6::client;
-    using wayland::XdgSurfaceV6::resource;
+    using mw::XdgSurfaceV6::client;
+    using mw::XdgSurfaceV6::resource;
 
 private:
     void set_window_role(WindowWlSurfaceRole* role);
 
-    std::experimental::optional<WindowWlSurfaceRole*> window_role_;
-    std::shared_ptr<bool> window_role_destroyed;
+    mw::Weak<WindowWlSurfaceRole> window_role_;
     WlSurface* const surface;
 
 public:
     XdgShellV6 const& xdg_shell;
 };
 
-class XdgPopupV6 : wayland::XdgPopupV6, public WindowWlSurfaceRole
+class XdgPopupV6 : mw::XdgPopupV6, public WindowWlSurfaceRole
 {
 public:
     XdgPopupV6(
@@ -98,7 +97,7 @@ private:
     XdgSurfaceV6* const xdg_surface;
 };
 
-class XdgToplevelV6 : wayland::XdgToplevelV6, public WindowWlSurfaceRole
+class XdgToplevelV6 : mw::XdgToplevelV6, public WindowWlSurfaceRole
 {
 public:
     XdgToplevelV6(wl_resource* new_resource, XdgSurfaceV6* xdg_surface, WlSurface* surface);
@@ -132,7 +131,7 @@ private:
     XdgSurfaceV6* const xdg_surface;
 };
 
-class XdgPositionerV6 : public wayland::XdgPositionerV6, public shell::SurfaceSpecification
+class XdgPositionerV6 : public mw::XdgPositionerV6, public shell::SurfaceSpecification
 {
 public:
     XdgPositionerV6(wl_resource* new_resource);
@@ -151,7 +150,7 @@ private:
 }
 namespace mf = mir::frontend;  // Keep CLion's parsing happy
 
-class mf::XdgShellV6::Instance : public wayland::XdgShellV6
+class mf::XdgShellV6::Instance : public mw::XdgShellV6
 {
 public:
     Instance(wl_resource* new_resource, mf::XdgShellV6* shell);
@@ -196,10 +195,12 @@ void mf::XdgShellV6::Instance::pong(uint32_t serial)
 
 mf::XdgShellV6::XdgShellV6(
     struct wl_display* display,
+    Executor& wayland_executor,
     std::shared_ptr<msh::Shell> shell,
     WlSeat& seat,
     OutputManager* output_manager) :
     Global(display, Version<1>()),
+    wayland_executor{wayland_executor},
     shell{shell},
     seat{seat},
     output_manager{output_manager}
@@ -216,7 +217,7 @@ void mf::XdgShellV6::bind(wl_resource* new_resource)
 mf::XdgSurfaceV6* mf::XdgSurfaceV6::from(wl_resource* surface)
 {
     auto* tmp = wl_resource_get_user_data(surface);
-    return static_cast<XdgSurfaceV6*>(static_cast<wayland::XdgSurfaceV6*>(tmp));
+    return static_cast<XdgSurfaceV6*>(static_cast<mw::XdgSurfaceV6*>(tmp));
 }
 
 mf::XdgSurfaceV6::XdgSurfaceV6(wl_resource* new_resource, WlSurface* surface,
@@ -246,11 +247,11 @@ void mf::XdgSurfaceV6::get_popup(wl_resource* new_popup, struct wl_resource* par
 
 void mf::XdgSurfaceV6::set_window_geometry(int32_t x, int32_t y, int32_t width, int32_t height)
 {
-    if (auto& role = window_role())
+    if (window_role_)
     {
-        role.value()->set_pending_offset(geom::Displacement{-x, -y});
-        role.value()->set_pending_width(geom::Width{width});
-        role.value()->set_pending_height(geom::Height{height});
+        window_role_.value().set_pending_offset(geom::Displacement{-x, -y});
+        window_role_.value().set_pending_width(geom::Width{width});
+        window_role_.value().set_pending_height(geom::Height{height});
     }
 }
 
@@ -262,28 +263,23 @@ void mf::XdgSurfaceV6::ack_configure(uint32_t serial)
 
 void mf::XdgSurfaceV6::send_configure()
 {
-    auto const serial = wl_display_next_serial(wl_client_get_display(wayland::XdgSurfaceV6::client));
+    auto const serial = wl_display_next_serial(wl_client_get_display(mw::XdgSurfaceV6::client));
     send_configure_event(serial);
 }
 
-std::experimental::optional<mf::WindowWlSurfaceRole*> const& mf::XdgSurfaceV6::window_role()
+mw::Weak<mf::WindowWlSurfaceRole> const& mf::XdgSurfaceV6::window_role()
 {
-    if (window_role_ && *window_role_destroyed)
-    {
-        window_role_ = std::experimental::nullopt;
-        window_role_destroyed = nullptr;
-    }
-
     return window_role_;
 }
 
 void mf::XdgSurfaceV6::set_window_role(WindowWlSurfaceRole* role)
 {
-    if (window_role())
+    if (window_role_)
+    {
         log_warning("XdgSurfaceV6::window_role set multiple times");
+    }
 
-    window_role_ = role;
-    window_role_destroyed = role->destroyed_flag();
+    window_role_ = mw::make_weak(role);
 }
 
 // XdgPopupV6
@@ -296,8 +292,9 @@ mf::XdgPopupV6::XdgPopupV6(
     WlSurface* surface)
     : mw::XdgPopupV6(new_resource, Version<1>()),
       WindowWlSurfaceRole(
+          xdg_surface->xdg_shell.wayland_executor,
           &xdg_surface->xdg_shell.seat,
-          wayland::XdgPopupV6::client,
+          mw::XdgPopupV6::client,
           surface,
           xdg_surface->xdg_shell.shell,
           xdg_surface->xdg_shell.output_manager),
@@ -305,13 +302,13 @@ mf::XdgPopupV6::XdgPopupV6(
 {
     auto specification = static_cast<mir::shell::SurfaceSpecification*>(
                                 static_cast<XdgPositionerV6*>(
-                                    static_cast<wayland::XdgPositionerV6*>(
+                                    static_cast<mw::XdgPositionerV6*>(
                                         wl_resource_get_user_data(positioner))));
 
-    auto parent_role = parent_surface->window_role();
-    auto parent_scene_surface = parent_role ? parent_role.value()->scene_surface() : std::experimental::nullopt;
+    auto const& parent_role = parent_surface->window_role();
+    auto parent_scene_surface = parent_role ? parent_role.value().scene_surface() : std::experimental::nullopt;
 
-    specification->type = mir_window_type_freestyle;
+    specification->type = mir_window_type_gloss;
     specification->placement_hints = mir_placement_hints_slide_any;
     if (parent_scene_surface)
         specification->parent = parent_scene_surface.value();
@@ -324,7 +321,7 @@ mf::XdgPopupV6::XdgPopupV6(
 void mf::XdgPopupV6::grab(struct wl_resource* seat, uint32_t serial)
 {
     (void)seat, (void)serial;
-    // TODO
+    set_type(mir_window_type_menu);
 }
 
 void mf::XdgPopupV6::destroy()
@@ -362,8 +359,9 @@ void mf::XdgPopupV6::handle_close_request()
 mf::XdgToplevelV6::XdgToplevelV6(struct wl_resource* new_resource, XdgSurfaceV6* xdg_surface, WlSurface* surface)
     : mw::XdgToplevelV6(new_resource, Version<1>()),
       WindowWlSurfaceRole(
+          xdg_surface->xdg_shell.wayland_executor,
           &xdg_surface->xdg_shell.seat,
-          wayland::XdgToplevelV6::client,
+          mw::XdgToplevelV6::client,
           surface,
           xdg_surface->xdg_shell.shell,
           xdg_surface->xdg_shell.output_manager),
@@ -560,7 +558,7 @@ void mf::XdgToplevelV6::send_toplevel_configure()
 mf::XdgToplevelV6* mf::XdgToplevelV6::from(wl_resource* surface)
 {
     auto* tmp = wl_resource_get_user_data(surface);
-    return static_cast<XdgToplevelV6*>(static_cast<wayland::XdgToplevelV6*>(tmp));
+    return static_cast<XdgToplevelV6*>(static_cast<mw::XdgToplevelV6*>(tmp));
 }
 
 // XdgPositionerV6
@@ -646,15 +644,15 @@ auto mf::XdgShellV6::get_window(wl_resource* surface) -> std::shared_ptr<scene::
     if (mw::XdgSurfaceV6::is_instance(surface))
     {
         auto const v6surface = XdgSurfaceV6::from(surface);
-        if (auto const role = v6surface->window_role())
+        if (auto const& role = v6surface->window_role())
         {
-            if (auto const scene_surface = role.value()->scene_surface())
+            if (auto const scene_surface = role.value().scene_surface())
             {
                 return scene_surface.value();
             }
         }
 
-        log_debug("No window currently associated with wayland::XdgSurfaceV6 %p", surface);
+        log_debug("No window currently associated with wayland::XdgSurfaceV6 %p", static_cast<void*>(surface));
     }
 
     return {};
