@@ -18,7 +18,9 @@
 
 #include "xcb_connection.h"
 
+#include "xwayland_log.h"
 #include "mir/log.h"
+#include "mir/c_memory.h"
 
 #include "boost/throw_exception.hpp"
 #include <sstream>
@@ -45,6 +47,130 @@ auto data_buffer_to_debug_string(
     ss << "]";
     return ss.str();
 }
+
+template<typename T, size_t length>
+constexpr size_t length_of(T(&)[length])
+{
+    return length;
+}
+
+/// Convert any sort of ID into a human-like readable (and memorable) name for debugging
+/// 0 results in Null, and IDs that are close together often rhyme
+auto number_to_readable_name(uint32_t number) -> std::string
+{
+    static const char* beginnings[]{"N", "Al", "Aver", "B", "Ch", "D", "Ell", "Ev", "F", "G", "Gr", "H", "Iv", "J", "K",
+        "L", "M", "Mr ", "Ms ", "Mx ", "P", "Pr", "Qu", "R", "S", "T", "Tr", "V", "W", "Z"};
+    static const char* middles[]{"u", "a", "ai", "ay", "e", "ee", "el", "en", "i", "ing", "o", "oo", "ou", "or"};
+    static const char* ends[]{"ll", "b", "ck", "cy", "d", "dy", "gh", "gia", "r", "l", "le", "liet", "lie", "mer", "nd",
+        "ny", "rd", "sly", "son", "ss", "thy", "te", "tt", "xon", "y", "yton", "zz", ""};
+    std::stringstream ss;
+    ss << beginnings[number % length_of(beginnings)];
+    number /= length_of(beginnings);
+    ss << middles[number % length_of(middles)];
+    number /= length_of(middles);
+    ss << ends[number % length_of(ends)];
+    return ss.str();
+}
+
+auto xcb_error_to_string(int error) -> std::string
+{
+    // see https://xcb.freedesktop.org/manual/group__XCB__Core__API.html#ga70a6bade94bd2824db552abcf5fbdbe3
+    switch (error)
+    {
+        case 0: return "no error";
+        case XCB_CONN_ERROR: return "XCB_CONN_ERROR";
+        case XCB_CONN_CLOSED_EXT_NOTSUPPORTED: return "XCB_CONN_CLOSED_EXT_NOTSUPPORTED";
+        case XCB_CONN_CLOSED_MEM_INSUFFICIENT: return "XCB_CONN_CLOSED_MEM_INSUFFICIENT";
+        case XCB_CONN_CLOSED_REQ_LEN_EXCEED: return "XCB_CONN_CLOSED_REQ_LEN_EXCEED";
+        case XCB_CONN_CLOSED_PARSE_ERR: return "XCB_CONN_CLOSED_PARSE_ERR";
+        case XCB_CONN_CLOSED_INVALID_SCREEN: return "XCB_CONN_CLOSED_INVALID_SCREEN";
+        case XCB_CONN_CLOSED_FDPASSING_FAILED: return "XCB_CONN_CLOSED_FDPASSING_FAILED";
+        default:;
+    }
+    return "unknwon XCB error " + std::to_string(error);
+}
+
+auto connect_to_fd(mir::Fd const& fd) -> xcb_connection_t*
+{
+    xcb_connection_t* connection = xcb_connect_to_fd(fd, nullptr);
+    if (auto const error = xcb_connection_has_error(connection))
+    {
+        xcb_disconnect(connection);
+        BOOST_THROW_EXCEPTION(std::runtime_error("xcb_connect_to_fd() failed: " + xcb_error_to_string(error)));
+    }
+    else
+    {
+        return connection;
+    }
+}
+
+template<typename T>
+auto value_handler(
+    mf::XCBConnection const* connection,
+    xcb_atom_t prop,
+    mf::XCBConnection::Handler<T> handler) -> mf::XCBConnection::Handler<xcb_get_property_reply_t*>
+{
+    return {
+        [connection, prop, on_success=move(handler.on_success)](xcb_get_property_reply_t const* reply)
+        {
+            uint8_t const expected_format = sizeof(T) * 8;
+            if (reply->format != expected_format)
+            {
+                BOOST_THROW_EXCEPTION(std::runtime_error(
+                    "Failed to read " + connection->query_name(prop) + " window property." +
+                    " Reply of type " + connection->query_name(reply->type) +
+                    " has a format " + std::to_string(reply->format) +
+                    " instead of expected " + std::to_string(expected_format)));
+            }
+
+            // The length returned by xcb_get_property_value_length() is in bytes for some reason
+            size_t const byte_len = xcb_get_property_value_length(reply);
+
+            if (byte_len != sizeof(T))
+            {
+                BOOST_THROW_EXCEPTION(std::runtime_error(
+                    "Failed to read " + connection->query_name(prop) + " window property." +
+                    " Reply of type " + connection->query_name(reply->type) +
+                    " has a byte length " + std::to_string(byte_len) +
+                    " instead of expected " + std::to_string(sizeof(T))));
+            }
+
+            on_success(*static_cast<T const*>(xcb_get_property_value(reply)));
+        },
+        move(handler.on_error)
+    };
+}
+
+template<typename T>
+auto vector_handler(
+    mf::XCBConnection const* connection,
+    xcb_atom_t prop,
+    mf::XCBConnection::Handler<std::vector<T>> handler) -> mf::XCBConnection::Handler<xcb_get_property_reply_t*>
+{
+    return {
+        [connection, prop, on_success=handler.on_success](xcb_get_property_reply_t const* reply)
+        {
+            uint8_t const expected_format = sizeof(T) * 8;
+            if (reply->format != expected_format)
+            {
+                BOOST_THROW_EXCEPTION(std::runtime_error(
+                    "Failed to read " + connection->query_name(prop) + " window property." +
+                    " Reply of type " + connection->query_name(reply->type) +
+                    " has a format " + std::to_string(reply->format) +
+                    " instead of expected " + std::to_string(expected_format)));
+            }
+
+            auto const start = static_cast<T const*>(xcb_get_property_value(reply));
+            // The length returned by xcb_get_property_value_length() is in bytes for some reason
+            size_t const len = xcb_get_property_value_length(reply) / sizeof(T);
+            auto const end = start + len;
+            std::vector<T> const values{start, end};
+
+            on_success(values);
+        },
+        move(handler.on_error)
+    };
+}
 }
 
 mf::XCBConnection::Atom::Atom(std::string const& name, XCBConnection* connection)
@@ -63,11 +189,10 @@ mf::XCBConnection::Atom::operator xcb_atom_t() const
         // The initial atomic check is faster, but we need to check again once we've got the lock
         if (atom == XCB_ATOM_NONE)
         {
-            auto const reply = xcb_intern_atom_reply(*connection, cookie, nullptr);
+            auto const reply = make_unique_cptr(xcb_intern_atom_reply(*connection, cookie, nullptr));
             if (!reply)
                 BOOST_THROW_EXCEPTION(std::runtime_error("Failed to look up atom " + name_));
             atom = reply->atom;
-            free(reply);
 
             std::lock_guard<std::mutex> lock{connection->atom_name_cache_mutex};
             connection->atom_name_cache[atom] = name_;
@@ -77,51 +202,11 @@ mf::XCBConnection::Atom::operator xcb_atom_t() const
     return atom;
 }
 
-mf::XCBConnection::XCBConnection(int fd)
-    : xcb_connection{xcb_connect_to_fd(fd, nullptr)},
+mf::XCBConnection::XCBConnection(Fd const& fd)
+    : fd{fd},
+      xcb_connection{connect_to_fd(fd)},
       xcb_screen{xcb_setup_roots_iterator(xcb_get_setup(xcb_connection)).data},
-      atom_name_cache{{XCB_ATOM_NONE, "None/Any"}},
-      wm_protocols{"WM_PROTOCOLS", this},
-      wm_take_focus{"WM_TAKE_FOCUS", this},
-      wm_delete_window{"WM_DELETE_WINDOW", this},
-      wm_state{"WM_STATE", this},
-      wm_change_state{"WM_CHANGE_STATE", this},
-      wm_s0{"WM_S0", this},
-      net_wm_cm_s0{"_NET_WM_CM_S0", this},
-      net_wm_name{"_NET_WM_NAME", this},
-      net_wm_pid{"_NET_WM_PID", this},
-      net_wm_state{"_NET_WM_STATE", this},
-      net_wm_state_maximized_vert{"_NET_WM_STATE_MAXIMIZED_VERT", this},
-      net_wm_state_maximized_horz{"_NET_WM_STATE_MAXIMIZED_HORZ", this},
-      net_wm_state_hidden{"_NET_WM_STATE_HIDDEN", this},
-      net_wm_state_fullscreen{"_NET_WM_STATE_FULLSCREEN", this},
-      net_wm_user_time{"_NET_WM_USER_TIME", this},
-      net_wm_icon_name{"_NET_WM_ICON_NAME", this},
-      net_wm_desktop{"_NET_WM_DESKTOP", this},
-      net_wm_window_type{"_NET_WM_WINDOW_TYPE", this},
-      net_wm_window_type_desktop{"_NET_WM_WINDOW_TYPE_DESKTOP", this},
-      net_wm_window_type_dock{"_NET_WM_WINDOW_TYPE_DOCK", this},
-      net_wm_window_type_toolbar{"_NET_WM_WINDOW_TYPE_TOOLBAR", this},
-      net_wm_window_type_menu{"_NET_WM_WINDOW_TYPE_MENU", this},
-      net_wm_window_type_utility{"_NET_WM_WINDOW_TYPE_UTILITY", this},
-      net_wm_window_type_splash{"_NET_WM_WINDOW_TYPE_SPLASH", this},
-      net_wm_window_type_dialog{"_NET_WM_WINDOW_TYPE_DIALOG", this},
-      net_wm_window_type_dropdown{"_NET_WM_WINDOW_TYPE_DROPDOWN_MENU", this},
-      net_wm_window_type_popup{"_NET_WM_WINDOW_TYPE_POPUP_MENU", this},
-      net_wm_window_type_tooltip{"_NET_WM_WINDOW_TYPE_TOOLTIP", this},
-      net_wm_window_type_notification{"_NET_WM_WINDOW_TYPE_NOTIFICATION", this},
-      net_wm_window_type_combo{"_NET_WM_WINDOW_TYPE_COMBO", this},
-      net_wm_window_type_dnd{"_NET_WM_WINDOW_TYPE_DND", this},
-      net_wm_window_type_normal{"_NET_WM_WINDOW_TYPE_NORMAL", this},
-      net_wm_moveresize{"_NET_WM_MOVERESIZE", this},
-      net_supporting_wm_check{"_NET_SUPPORTING_WM_CHECK", this},
-      net_supported{"_NET_SUPPORTED", this},
-      net_active_window{"_NET_ACTIVE_WINDOW", this},
-      motif_wm_hints{"_MOTIF_WM_HINTS", this},
-      clipboard{"CLIPBOARD", this},
-      clipboard_manager{"CLIPBOARD_MANAGER", this},
-      utf8_string{"UTF8_STRING", this},
-      wl_surface_id{"WL_SURFACE_ID", this}
+      atom_name_cache{{XCB_ATOM_NONE, "None/Any"}}
 {
 }
 
@@ -130,19 +215,12 @@ mf::XCBConnection::~XCBConnection()
     xcb_disconnect(xcb_connection);
 }
 
-mf::XCBConnection::operator xcb_connection_t*() const
+void mf::XCBConnection::verify_not_in_error_state() const
 {
-    return xcb_connection;
-}
-
-auto mf::XCBConnection::screen() const -> xcb_screen_t*
-{
-    return xcb_screen;
-}
-
-auto mf::XCBConnection::root_window() const -> xcb_window_t
-{
-    return xcb_screen->root;
+    if (auto const error = xcb_connection_has_error(xcb_connection))
+    {
+        BOOST_THROW_EXCEPTION(std::runtime_error("XCB connection shut down: " + xcb_error_to_string(error)));
+    }
 }
 
 auto mf::XCBConnection::query_name(xcb_atom_t atom) const -> std::string
@@ -153,16 +231,20 @@ auto mf::XCBConnection::query_name(xcb_atom_t atom) const -> std::string
     if (iter == atom_name_cache.end())
     {
         xcb_get_atom_name_cookie_t const cookie = xcb_get_atom_name(xcb_connection, atom);
-        xcb_get_atom_name_reply_t* const reply = xcb_get_atom_name_reply(xcb_connection, cookie, nullptr);
+        auto const reply = make_unique_cptr(xcb_get_atom_name_reply(xcb_connection, cookie, nullptr));
 
         std::string name;
 
         if (reply)
-            name = std::string{xcb_get_atom_name_name(reply), static_cast<size_t>(xcb_get_atom_name_name_length(reply))};
+        {
+            name = std::string{
+                xcb_get_atom_name_name(reply.get()),
+                static_cast<size_t>(xcb_get_atom_name_name_length(reply.get()))};
+        }
         else
+        {
             name = "Atom " + std::to_string(atom);
-
-        free(reply);
+        }
 
         atom_name_cache[atom] = name;
 
@@ -176,7 +258,7 @@ auto mf::XCBConnection::query_name(xcb_atom_t atom) const -> std::string
 
 auto mf::XCBConnection::reply_contains_string_data(xcb_get_property_reply_t const* reply) const -> bool
 {
-    return reply->type == XCB_ATOM_STRING || reply->type == utf8_string;
+    return reply->type == XCB_ATOM_STRING || reply->type == UTF8_STRING || reply->type == COMPOUND_TEXT;
 }
 
 auto mf::XCBConnection::string_from(xcb_get_property_reply_t const* reply) const -> std::string
@@ -184,7 +266,7 @@ auto mf::XCBConnection::string_from(xcb_get_property_reply_t const* reply) const
     if (!reply_contains_string_data(reply))
     {
         BOOST_THROW_EXCEPTION(std::runtime_error(
-            "Supplied reply is of type " + query_name(reply->type) + " and does not hold string data"));
+            "Supplied reply is of type \"" + query_name(reply->type) + "\" and does not hold string data"));
     }
 
     auto const result = std::string{
@@ -206,138 +288,114 @@ bool mf::XCBConnection::is_ours(uint32_t id) const
 auto mf::XCBConnection::read_property(
     xcb_window_t window,
     xcb_atom_t prop,
-    std::function<void(xcb_get_property_reply_t*)> action,
-    std::function<void()> on_error) -> std::function<void()>
+    bool delete_after_read,
+    uint32_t max_length,
+    Handler<xcb_get_property_reply_t*>&& handler) const -> std::function<void()>
 {
     xcb_get_property_cookie_t cookie = xcb_get_property(
         xcb_connection,
-        0, // don't delete
+        delete_after_read ? 1 : 0,
         window,
         prop,
         XCB_ATOM_ANY,
         0, // no offset
-        2048); // big buffer
+        max_length);
 
-    return [xcb_connection = xcb_connection, cookie, action, on_error]()
+    return [this, cookie, handler=std::move(handler), window, prop]()
         {
-            xcb_get_property_reply_t *reply = xcb_get_property_reply(xcb_connection, cookie, nullptr);
-            if (reply && reply->type != XCB_ATOM_NONE)
+            try
             {
-                try
+                Error error;
+                auto const reply = make_unique_cptr(xcb_get_property_reply(xcb_connection, cookie, &error.ptr));
+                if (reply && reply->type != XCB_ATOM_NONE)
                 {
-                    action(reply);
+                    handler.on_success(reply.get());
                 }
-                catch (...)
+                else if (reply)
                 {
-                    log(
-                        logging::Severity::warning,
-                        MIR_LOG_COMPONENT,
-                        std::current_exception(),
-                        "Failed to process property reply.");
+                    std::string message = "no reply data";
+                    if (verbose_xwayland_logging_enabled())
+                    {
+                        message +=  " for " + window_debug_string(window) + "." + query_name(prop);
+                    }
+                    handler.on_error(message);
+                }
+                else
+                {
+                    std::string message = "error reading property: ";
+                    if (verbose_xwayland_logging_enabled())
+                    {
+                        message = "error reading " + window_debug_string(window) + "." + query_name(prop) + ": ";
+                    }
+                    handler.on_error(message + error_debug_string(error.ptr));
                 }
             }
-            else
+            catch (...)
             {
-                try
-                {
-                    on_error();
-                }
-                catch (...)
-                {
-                    log(
-                        logging::Severity::warning,
-                        MIR_LOG_COMPONENT,
-                        std::current_exception(),
-                        "Failed to run property error callback.");
-                }
+                log(
+                    logging::Severity::warning,
+                    MIR_LOG_COMPONENT,
+                    "Exception thrown processing reply for property " +
+                    window_debug_string(window) + "." + query_name(prop));
             }
-            free(reply);
         };
 }
 
 auto mf::XCBConnection::read_property(
     xcb_window_t window,
     xcb_atom_t prop,
-    std::function<void(std::string const&)> action,
-    std::function<void()> on_error) -> std::function<void()>
+    Handler<xcb_get_property_reply_t*>&& handler) const -> std::function<void()>
 {
-    return read_property(
-        window,
-        prop,
-        [this, action](xcb_get_property_reply_t const* reply)
-        {
-            action(string_from(reply));
-        },
-        on_error);
+    return read_property(window, prop, false, 2048, std::move(handler));
 }
 
 auto mf::XCBConnection::read_property(
     xcb_window_t window,
     xcb_atom_t prop,
-    std::function<void(uint32_t)> action,
-    std::function<void()> on_error) -> std::function<void()>
+    Handler<std::string> handler) const -> std::function<void()>
 {
     return read_property(
         window,
         prop,
-        [this, prop, action](xcb_get_property_reply_t const* reply)
         {
-            uint8_t const expected_format = 32;
-            if (reply->format != expected_format)
+            [this, on_success=move(handler.on_success)](xcb_get_property_reply_t const* reply)
             {
-                BOOST_THROW_EXCEPTION(std::runtime_error(
-                    "Failed to read " + query_name(prop) + " window property." +
-                    " Reply of type " + query_name(reply->type) +
-                    " has a format " + std::to_string(reply->format) +
-                    " instead of expected " + std::to_string(expected_format)));
-            }
-
-            // The length returned by xcb_get_property_value_length() is in bytes for some reason
-            size_t const len = xcb_get_property_value_length(reply) / sizeof(uint32_t);
-
-            if (len != 1)
-            {
-                BOOST_THROW_EXCEPTION(std::runtime_error(
-                    "Failed to read " + query_name(prop) + " window property." +
-                    " Reply of type " + query_name(reply->type) +
-                    " has a value length " + std::to_string(len) +
-                    " instead of expected 1"));
-            }
-
-            action(*static_cast<uint32_t const*>(xcb_get_property_value(reply)));
-        },
-        on_error);
+                on_success(string_from(reply));
+            },
+            move(handler.on_error)
+        });
 }
 
 auto mf::XCBConnection::read_property(
     xcb_window_t window,
     xcb_atom_t prop,
-    std::function<void(std::vector<uint32_t>)> action,
-    std::function<void()> on_error) -> std::function<void()>
+    Handler<uint32_t> handler) const -> std::function<void()>
 {
-    return read_property(
-        window,
-        prop,
-        [this, action](xcb_get_property_reply_t const* reply)
-        {
-            uint8_t const expected_format = 32;
-            if (reply->format != expected_format)
-            {
-                BOOST_THROW_EXCEPTION(std::runtime_error(
-                    "Reply of type " + query_name(reply->type) +
-                    " has a format " + std::to_string(reply->format) +
-                    " instead of expected " + std::to_string(expected_format)));
-            }
+    return read_property(window, prop, value_handler(this, prop, handler));
+}
 
-            auto const start = static_cast<uint32_t const*>(xcb_get_property_value(reply));
-            // The length returned by xcb_get_property_value_length() is in bytes for some reason
-            size_t const len = xcb_get_property_value_length(reply) / sizeof(uint32_t);
-            auto const end = start + len;
-            std::vector<uint32_t> const values{start, end};
+auto mf::XCBConnection::read_property(
+    xcb_window_t window,
+    xcb_atom_t prop,
+    Handler<int32_t> handler) const -> std::function<void()>
+{
+    return read_property(window, prop, value_handler(this, prop, handler));
+}
 
-            action(values);
-        },
-        on_error);
+auto mf::XCBConnection::read_property(
+    xcb_window_t window,
+    xcb_atom_t prop,
+    Handler<std::vector<uint32_t>> handler) const -> std::function<void()>
+{
+    return read_property(window, prop, vector_handler(this, prop, handler));
+}
+
+auto mf::XCBConnection::read_property(
+    xcb_window_t window,
+    xcb_atom_t prop,
+    Handler<std::vector<int32_t>> handler) const -> std::function<void()>
+{
+    return read_property(window, prop, vector_handler(this, prop, handler));
 }
 
 void mf::XCBConnection::configure_window(
@@ -345,7 +403,7 @@ void mf::XCBConnection::configure_window(
     std::experimental::optional<geometry::Point> position,
     std::experimental::optional<geometry::Size> size,
     std::experimental::optional<xcb_window_t> sibling,
-    std::experimental::optional<xcb_stack_mode_t> stack_mode)
+    std::experimental::optional<xcb_stack_mode_t> stack_mode) const
 {
     std::vector<uint32_t> values;
     uint32_t mask = 0;
@@ -475,14 +533,65 @@ auto mf::XCBConnection::client_message_debug_string(xcb_client_message_event_t*
 
 auto mf::XCBConnection::window_debug_string(xcb_window_t window) const -> std::string
 {
-    if (!window)
+    if (window == XCB_WINDOW_NONE)
         return "null window";
     else if (window == xcb_screen->root)
         return "root window";
     else if (is_ours(window))
-        return "our window " + std::to_string(window);
+        return "our window " + number_to_readable_name(window);
     else
-        return "window " + std::to_string(window);
+        return "window " + number_to_readable_name(window);
+}
+
+auto mf::XCBConnection::error_debug_string(xcb_generic_error_t* error) const -> std::string
+{
+    std::string result;
+
+    if (!error)
+    {
+        return "(given XCB error is null)";
+    }
+
+    switch (error->error_code)
+    {
+    case XCB_REQUEST:
+    {
+        result = "not a valid request ";
+        break;
+    }
+
+    case XCB_VALUE:
+    {
+        auto const value_error{reinterpret_cast<xcb_value_error_t*>(error)};
+        result = std::to_string(value_error->bad_value) + " is an invalid value ";
+        break;
+    }
+
+    case XCB_WINDOW:
+    {
+        auto const window_error{reinterpret_cast<xcb_window_error_t*>(error)};
+        result =
+            window_debug_string(window_error->bad_value) +
+            " (ID " + std::to_string(window_error->bad_value) + ") does not exist ";
+        break;
+    }
+
+    case XCB_ATOM:
+    {
+        auto const atom_error{reinterpret_cast<xcb_atom_error_t*>(error)};
+        result = "atom " + std::to_string(atom_error->bad_value) + " does not exist ";
+        break;
+    }
+
+    default:
+        break;
+    }
+
+    result +=
+        "(error code " + std::to_string(error->error_code) +
+        ", opcode " + std::to_string(error->major_code) + "." + std::to_string(error->minor_code) + ")";
+
+    return result;
 }
 
 auto mf::XCBConnection::xcb_type_atom(XCBType type) const -> xcb_atom_t
@@ -492,10 +601,11 @@ auto mf::XCBConnection::xcb_type_atom(XCBType type) const -> xcb_atom_t
         case XCBType::ATOM:         return XCB_ATOM_ATOM;
         case XCBType::WINDOW:       return XCB_ATOM_WINDOW;
         case XCBType::CARDINAL32:   return XCB_ATOM_CARDINAL;
+        case XCBType::INTEGER32:    return XCB_ATOM_INTEGER;
         case XCBType::STRING:       return XCB_ATOM_STRING;
-        case XCBType::UTF8_STRING:  return utf8_string;
-        case XCBType::WM_STATE:     return wm_state;
-        case XCBType::WM_PROTOCOLS: return wm_protocols;
+        case XCBType::UTF8_STRING:  return UTF8_STRING;
+        case XCBType::WM_STATE:     return WM_STATE;
+        case XCBType::WM_PROTOCOLS: return WM_PROTOCOLS;
     }
 
     BOOST_THROW_EXCEPTION(std::runtime_error(
