@@ -22,6 +22,8 @@
 #include "mir/geometry/point.h"
 #include "mir/geometry/size.h"
 #include "mir/fatal.h"
+#include "mir/fd.h"
+#include "mir/log.h"
 
 #include <xcb/xcb.h>
 #include <string>
@@ -42,6 +44,7 @@ enum class XCBType
     ATOM,
     WINDOW,
     CARDINAL32, ///< also known as an unsigned int
+    INTEGER32,  ///< signed int
     STRING,
     UTF8_STRING,
     WM_STATE,
@@ -52,6 +55,7 @@ template<XCBType T> struct NativeXCBType                { typedef void type;};
 template<> struct NativeXCBType<XCBType::ATOM>          { typedef xcb_atom_t type; };
 template<> struct NativeXCBType<XCBType::WINDOW>        { typedef xcb_window_t type; };
 template<> struct NativeXCBType<XCBType::CARDINAL32>    { typedef uint32_t type; };
+template<> struct NativeXCBType<XCBType::INTEGER32>     { typedef int32_t type; };
 template<> struct NativeXCBType<XCBType::STRING>        { typedef char type; };
 template<> struct NativeXCBType<XCBType::UTF8_STRING>   { typedef char type; };
 template<> struct NativeXCBType<XCBType::WM_STATE>      { typedef uint32_t type; };
@@ -60,6 +64,7 @@ template<> struct NativeXCBType<XCBType::WM_PROTOCOLS>  { typedef uint32_t type;
 class XCBConnection
 {
 private:
+    Fd const fd;
     xcb_connection_t* const xcb_connection;
     xcb_screen_t* const xcb_screen;
 
@@ -76,7 +81,6 @@ public:
 
     private:
         Atom(Atom&) = delete;
-        Atom(Atom&&) = delete;
         Atom& operator=(Atom&) = delete;
 
         XCBConnection* const connection;
@@ -89,12 +93,56 @@ public:
         std::atomic<xcb_atom_t> mutable atom{XCB_ATOM_NONE};
     };
 
-    explicit XCBConnection(int fd);
+    struct Error
+    {
+        Error() = default;
+        Error(Error const&) = delete;
+        auto operator=(Error const&) -> Error const& = delete;
+        ~Error()
+        {
+            if (ptr)
+            {
+                free(ptr);
+            }
+        }
+
+        xcb_generic_error_t* ptr{nullptr};
+    };
+
+    template<typename T>
+    class Handler
+    {
+    public:
+        Handler(std::function<void(T const& value)>&& on_success)
+            : on_success{std::move(on_success)},
+              on_error{[](std::string const& message)
+                  {
+                      log_error("XCB error: %s", message.c_str());
+                  }}
+        {
+        }
+
+        Handler(
+            std::function<void(T const& value)>&& on_success,
+            std::function<void(std::string const& message)>&& on_error)
+            : on_success{move(on_success)},
+              on_error{move(on_error)}
+        {
+        }
+
+        std::function<void(T const& value)> on_success;
+        std::function<void(std::string const& message)> on_error;
+    };
+
+    explicit XCBConnection(Fd const& fd);
     ~XCBConnection();
 
-    operator xcb_connection_t*() const;
-    auto screen() const -> xcb_screen_t*;
-    auto root_window() const -> xcb_window_t;
+    /// Throws if the connection has been shut down
+    void verify_not_in_error_state() const;
+
+    operator xcb_connection_t*() const { return xcb_connection; }
+    auto screen() const -> xcb_screen_t* { return xcb_screen; }
+    auto root_window() const -> xcb_window_t { return xcb_screen->root; }
 
     /// Looks up an atom's name, or requests it from the X server if it is not already cached
     auto query_name(xcb_atom_t atom) const -> std::string;
@@ -110,51 +158,64 @@ public:
     auto read_property(
         xcb_window_t window,
         xcb_atom_t prop,
-        std::function<void(xcb_get_property_reply_t*)> action,
-        std::function<void()> on_error) -> std::function<void()>;
+        bool delete_after_read,
+        uint32_t max_length,
+        Handler<xcb_get_property_reply_t*>&& handler) const -> std::function<void()>;
 
     auto read_property(
         xcb_window_t window,
         xcb_atom_t prop,
-        std::function<void(std::string const&)> action,
-        std::function<void()> on_error = [](){}) -> std::function<void()>;
+        Handler<xcb_get_property_reply_t*>&& handler) const -> std::function<void()>;
 
     auto read_property(
         xcb_window_t window,
         xcb_atom_t prop,
-        std::function<void(uint32_t)> action,
-        std::function<void()> on_error = [](){}) -> std::function<void()>;
+        Handler<std::string> handler) const -> std::function<void()>;
 
     auto read_property(
         xcb_window_t window,
         xcb_atom_t prop,
-        std::function<void(std::vector<uint32_t>)> action,
-        std::function<void()> on_error = [](){}) -> std::function<void()>;
+        Handler<uint32_t> handler) const -> std::function<void()>;
+
+    auto read_property(
+        xcb_window_t window,
+        xcb_atom_t prop,
+        Handler<int32_t> handler) const -> std::function<void()>;
+
+    auto read_property(
+        xcb_window_t window,
+        xcb_atom_t prop,
+        Handler<std::vector<uint32_t>> handler) const -> std::function<void()>;
+
+    auto read_property(
+        xcb_window_t window,
+        xcb_atom_t prop,
+        Handler<std::vector<int32_t>> handler) const -> std::function<void()>;
     /// @}
 
     /// Set X11 window properties
     /// Safer and more fun than the C-style function provided by XCB
     /// @{
     template<XCBType type, typename T>
-    inline void set_property(xcb_window_t window, xcb_atom_t property, T data)
+    inline void set_property(xcb_window_t window, xcb_atom_t property, T data) const
     {
         set_property<type>(window, property, 1, &data);
     }
 
     template<XCBType type, typename T, size_t length>
-    inline void set_property(xcb_window_t window, xcb_atom_t property, T(&data)[length])
+    inline void set_property(xcb_window_t window, xcb_atom_t property, T(&data)[length]) const
     {
         set_property<type>(window, property, length, data);
     }
 
     template<XCBType type, typename T>
-    inline void set_property(xcb_window_t window, xcb_atom_t property, std::vector<T> const& data)
+    inline void set_property(xcb_window_t window, xcb_atom_t property, std::vector<T> const& data) const
     {
         set_property<type>(window, property, data.size(), data.data());
     }
 
     template<XCBType type>
-    inline void set_property(xcb_window_t window, xcb_atom_t property, std::string const& data)
+    inline void set_property(xcb_window_t window, xcb_atom_t property, std::string const& data) const
     {
         static_assert(
             type == XCBType::STRING || type == XCBType::UTF8_STRING,
@@ -164,7 +225,7 @@ public:
     /// @}
 
     /// Delete an X11 window property
-    inline void delete_property(xcb_window_t window, xcb_atom_t property)
+    inline void delete_property(xcb_window_t window, xcb_atom_t property) const
     {
         xcb_delete_property(xcb_connection, window, property);
     }
@@ -175,19 +236,19 @@ public:
         std::experimental::optional<geometry::Point> position,
         std::experimental::optional<geometry::Size> size,
         std::experimental::optional<xcb_window_t> sibling,
-        std::experimental::optional<xcb_stack_mode_t> stack_mode);
+        std::experimental::optional<xcb_stack_mode_t> stack_mode) const;
 
     /// Send client message
     /// @{
     template<XCBType type, typename T, size_t length>
-    void send_client_message(xcb_window_t window, uint32_t event_mask, T(&data)[length])
+    void send_client_message(xcb_window_t window, uint32_t event_mask, T(&data)[length]) const
     {
         static_assert(length * sizeof(T) <= sizeof(xcb_client_message_data_t), "Too much data");
         send_client_message<type>(window, event_mask, length, data);
     }
     /// @}
 
-    inline void flush()
+    inline void flush() const
     {
         xcb_flush(xcb_connection);
     }
@@ -197,53 +258,64 @@ public:
     auto reply_debug_string(xcb_get_property_reply_t* reply) const -> std::string;
     auto client_message_debug_string(xcb_client_message_event_t* event) const -> std::string;
     auto window_debug_string(xcb_window_t window) const -> std::string;
+    auto error_debug_string(xcb_generic_error_t* error) const -> std::string;
     /// @}
 
-    Atom const wm_protocols;
-    Atom const wm_take_focus;
-    Atom const wm_delete_window;
-    Atom const wm_state;
-    Atom const wm_change_state;
-    Atom const wm_s0;
-    Atom const net_wm_cm_s0;
-    Atom const net_wm_name;
-    Atom const net_wm_pid;
-    Atom const net_wm_state;
-    Atom const net_wm_state_maximized_vert;
-    Atom const net_wm_state_maximized_horz;
-    Atom const net_wm_state_hidden;
-    Atom const net_wm_state_fullscreen;
-    Atom const net_wm_user_time;
-    Atom const net_wm_icon_name;
-    Atom const net_wm_desktop;
-    Atom const net_wm_window_type;
-    Atom const net_wm_window_type_desktop;
-    Atom const net_wm_window_type_dock;
-    Atom const net_wm_window_type_toolbar;
-    Atom const net_wm_window_type_menu;
-    Atom const net_wm_window_type_utility;
-    Atom const net_wm_window_type_splash;
-    Atom const net_wm_window_type_dialog;
-    Atom const net_wm_window_type_dropdown;
-    Atom const net_wm_window_type_popup;
-    Atom const net_wm_window_type_tooltip;
-    Atom const net_wm_window_type_notification;
-    Atom const net_wm_window_type_combo;
-    Atom const net_wm_window_type_dnd;
-    Atom const net_wm_window_type_normal;
-    Atom const net_wm_moveresize;
-    Atom const net_supporting_wm_check;
-    Atom const net_supported;
-    Atom const net_active_window;
-    Atom const motif_wm_hints;
-    Atom const clipboard;
-    Atom const clipboard_manager;
-    Atom const utf8_string;
-    Atom const wl_surface_id;
+#define DECLARE_ATOM(name) Atom const name{#name, this}
+
+    DECLARE_ATOM(WM_PROTOCOLS);
+    DECLARE_ATOM(WM_TAKE_FOCUS);
+    DECLARE_ATOM(WM_DELETE_WINDOW);
+    DECLARE_ATOM(WM_STATE);
+    DECLARE_ATOM(WM_CHANGE_STATE);
+    DECLARE_ATOM(WM_NORMAL_HINTS);
+    DECLARE_ATOM(WM_S0);
+    DECLARE_ATOM(_NET_WM_CM_S0);
+    DECLARE_ATOM(_NET_WM_NAME);
+    DECLARE_ATOM(_NET_WM_PID);
+    DECLARE_ATOM(_NET_WM_STATE);
+    DECLARE_ATOM(_NET_WM_STATE_MAXIMIZED_VERT);
+    DECLARE_ATOM(_NET_WM_STATE_MAXIMIZED_HORZ);
+    DECLARE_ATOM(_NET_WM_STATE_HIDDEN);
+    DECLARE_ATOM(_NET_WM_STATE_FULLSCREEN);
+    DECLARE_ATOM(_NET_WM_USER_TIME);
+    DECLARE_ATOM(_NET_WM_ICON_NAME);
+    DECLARE_ATOM(_NET_WM_DESKTOP);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_DESKTOP);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_DOCK);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_TOOLBAR);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_MENU);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_UTILITY);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_SPLASH);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_DIALOG);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_DROPDOWN_MENU);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_POPUP_MENU);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_TOOLTIP);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_NOTIFICATION);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_COMBO);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_DND);
+    DECLARE_ATOM(_NET_WM_WINDOW_TYPE_NORMAL);
+    DECLARE_ATOM(_NET_WM_MOVERESIZE);
+    DECLARE_ATOM(_NET_SUPPORTING_WM_CHECK);
+    DECLARE_ATOM(_NET_SUPPORTED);
+    DECLARE_ATOM(_NET_ACTIVE_WINDOW);
+    DECLARE_ATOM(_MOTIF_WM_HINTS);
+    DECLARE_ATOM(CLIPBOARD);
+    DECLARE_ATOM(CLIPBOARD_MANAGER);
+    DECLARE_ATOM(UTF8_STRING);
+    DECLARE_ATOM(TEXT);
+    DECLARE_ATOM(COMPOUND_TEXT);
+    DECLARE_ATOM(TIMESTAMP);
+    DECLARE_ATOM(TARGETS);
+    DECLARE_ATOM(INCR);
+    DECLARE_ATOM(_WL_SELECTION);
+    DECLARE_ATOM(WL_SURFACE_ID);
+
+#undef DECLARE_ATOM
 
 private:
     XCBConnection(XCBConnection&) = delete;
-    XCBConnection(XCBConnection&&) = delete;
     XCBConnection& operator=(XCBConnection&) = delete;
 
     auto xcb_type_atom(XCBType type) const -> xcb_atom_t;
@@ -271,7 +343,7 @@ private:
         xcb_window_t window,
         xcb_atom_t property,
         size_t length,
-        T const* data)
+        T const* data) const
     {
         validate_xcb_type<type, T>();
         xcb_change_property(
@@ -314,7 +386,7 @@ private:
     }
 
     template<XCBType type, typename T>
-    void send_client_message(xcb_window_t window, uint32_t event_mask, size_t length, T const* data)
+    void send_client_message(xcb_window_t window, uint32_t event_mask, size_t length, T const* data) const
     {
         validate_xcb_type<type, T>();
         xcb_client_message_event_t event;
