@@ -21,18 +21,36 @@
 
 #include "mir/frontend/connector.h"
 
+#include <memory>
+#include <mutex>
+
 namespace mir
 {
+class Executor;
+namespace dispatch
+{
+class ReadableFd;
+class ThreadedDispatcher;
+}
 namespace frontend
 {
 class WaylandConnector;
 class XWaylandServer;
+class XWaylandSpawner;
+class XWaylandWM;
 class XWaylandConnector : public Connector
 {
 public:
+    /// Scale is a little complicated. It controls how big apps appear but *probably* not the scale they render at
+    /// (unless they respect the X11 DPI, which most don't). For sharp, correctly sized apps the output scale, XWayland
+    /// scale and the scale the application uses should all match. Application scale needs to be configured on a per-app
+    /// or even per-toolkit basis. GDK_SCALE is used by default for XWayland scale because many apps respect it, so it's
+    /// generally as correct as anything.
     XWaylandConnector(
-        std::shared_ptr<WaylandConnector> const& wayland_connector, std::string const& xwayland_path);
-    ~XWaylandConnector() override;
+        std::shared_ptr<WaylandConnector> const& wayland_connector,
+        std::string const& xwayland_path,
+        float scale);
+    ~XWaylandConnector();
 
     void start() override;
     void stop() override;
@@ -44,8 +62,17 @@ public:
     auto socket_name() const -> optional_value<std::string> override;
 
 private:
-    std::function<std::unique_ptr<XWaylandServer>()> const start_xwayland;
-    std::unique_ptr<XWaylandServer> xwayland_server;
+    std::shared_ptr<WaylandConnector> const wayland_connector;
+    std::string const xwayland_path;
+    float const scale;
+
+    void spawn();
+
+    std::mutex mutable mutex;
+    std::unique_ptr<XWaylandSpawner> spawner;
+    std::unique_ptr<XWaylandServer> server;
+    std::unique_ptr<XWaylandWM> wm;
+    std::unique_ptr<dispatch::ThreadedDispatcher> wm_event_thread;
 };
 } /* frontend */
 } /* mir */
