@@ -23,11 +23,16 @@
 #include "xwayland_surface.h"
 #include "xwayland_wm_shell.h"
 #include "xwayland_surface_role.h"
+#include "xwayland_cursors.h"
+#include "xwayland_client_manager.h"
+#include "xwayland_clipboard_source.h"
+#include "xwayland_clipboard_provider.h"
 
-#include "mir/dispatch/multiplexing_dispatchable.h"
-#include "mir/dispatch/readable_fd.h"
+#include "mir/c_memory.h"
 #include "mir/fd.h"
-#include "mir/terminate_with_current_exception.h"
+#include "mir/executor.h"
+#include "mir/frontend/surface_stack.h"
+#include "mir/scene/null_observer.h"
 
 #include <cstring>
 #include <poll.h>
@@ -35,71 +40,120 @@
 #include <unistd.h>
 #include <boost/throw_exception.hpp>
 
-#ifndef ARRAY_LENGTH
-#define ARRAY_LENGTH(a) mir::frontend::length_of(a)
-#endif
+namespace mf = mir::frontend;
+namespace ms = mir::scene;
+namespace geom = mir::geometry;
 
-#define CURSOR_ENTRY(x)        \
-    {                          \
-        (x), ARRAY_LENGTH((x)) \
+namespace
+{
+auto create_wm_window(mf::XCBConnection const& connection) -> xcb_window_t
+{
+    std::string const wm_name{"Mir XWM"};
+
+    xcb_window_t const wm_window = xcb_generate_id(connection);
+    xcb_create_window(
+        connection,
+        XCB_COPY_FROM_PARENT,
+        wm_window,
+        connection.root_window(),
+        0, 0, 10, 10, 0,
+        XCB_WINDOW_CLASS_INPUT_OUTPUT,
+        connection.screen()->root_visual,
+        0, NULL);
+
+    connection.set_property<mf::XCBType::WINDOW>(wm_window, connection._NET_SUPPORTING_WM_CHECK, wm_window);
+    connection.set_property<mf::XCBType::UTF8_STRING>(wm_window, connection._NET_WM_NAME, wm_name);
+
+    connection.set_property<mf::XCBType::WINDOW>(
+        connection.root_window(),
+        connection._NET_SUPPORTING_WM_CHECK,
+        wm_window);
+
+    /* Claim the WM_S0 selection even though we don't support
+     * the --replace functionality. */
+    xcb_set_selection_owner(connection, wm_window, connection.WM_S0, XCB_TIME_CURRENT_TIME);
+    xcb_set_selection_owner(connection, wm_window, connection._NET_WM_CM_S0, XCB_TIME_CURRENT_TIME);
+
+    return wm_window;
+}
+
+auto init_xfixes(mf::XCBConnection const& connection) -> xcb_query_extension_reply_t const*
+{
+    xcb_xfixes_query_version_cookie_t xfixes_cookie;
+    xcb_xfixes_query_version_reply_t *xfixes_reply;
+
+    xcb_prefetch_extension_data(connection, &xcb_xfixes_id);
+    xcb_prefetch_extension_data(connection, &xcb_composite_id);
+
+    auto const xfixes = xcb_get_extension_data(connection, &xcb_xfixes_id);
+    if (!xfixes || !xfixes->present)
+    {
+        mir::log_warning("xfixes not available");
+        return nullptr;
     }
 
-static const char *bottom_left_corners[] = {"bottom_left_corner", "sw-resize", "size_bdiag"};
+    xfixes_cookie = xcb_xfixes_query_version(connection, XCB_XFIXES_MAJOR_VERSION, XCB_XFIXES_MINOR_VERSION);
+    xfixes_reply = xcb_xfixes_query_version_reply(connection, xfixes_cookie, NULL);
 
-static const char *bottom_right_corners[] = {"bottom_right_corner", "se-resize", "size_fdiag"};
+    if (mir::verbose_xwayland_logging_enabled())
+    {
+        mir::log_debug("xfixes version: %d.%d", xfixes_reply->major_version, xfixes_reply->minor_version);
+    }
 
-static const char *bottom_sides[] = {"bottom_side", "s-resize", "size_ver"};
+    free(xfixes_reply);
+    return xfixes;
+}
 
-static const char *left_ptrs[] = {"left_ptr", "default", "top_left_arrow", "left-arrow"};
-
-static const char *left_sides[] = {"left_side", "w-resize", "size_hor"};
-
-static const char *right_sides[] = {"right_side", "e-resize", "size_hor"};
-
-static const char *top_left_corners[] = {"top_left_corner", "nw-resize", "size_fdiag"};
-
-static const char *top_right_corners[] = {"top_right_corner", "ne-resize", "size_bdiag"};
-
-static const char *top_sides[] = {"top_side", "n-resize", "size_ver"};
-
-struct cursor_alternatives
+auto focus_mode_to_string(uint32_t focus_mode) -> std::string
 {
-    const char **names;
-    size_t count;
+    switch (focus_mode)
+    {
+    case XCB_NOTIFY_MODE_NORMAL:        return "normal focus";
+    case XCB_NOTIFY_MODE_GRAB:          return "focus grabbed";
+    case XCB_NOTIFY_MODE_UNGRAB:        return "focus ungrabbed";
+    case XCB_NOTIFY_MODE_WHILE_GRABBED: return "focus while grabbed";
+    }
+    return "unknown focus mode " + std::to_string(focus_mode);
+}
+}
+
+class mf::XWaylandSceneObserver
+    : public ms::NullObserver
+{
+public:
+    XWaylandSceneObserver(mf::XWaylandWM* wm)
+        : wm{wm}
+    {
+    }
+
+    void surfaces_reordered(ms::SurfaceSet const& affected_surfaces) override
+    {
+        wm->surfaces_reordered(affected_surfaces);
+    }
+
+    mf::XWaylandWM* const wm;
 };
 
-static const struct cursor_alternatives cursors[] = {
-    CURSOR_ENTRY(top_sides),           CURSOR_ENTRY(bottom_sides),         CURSOR_ENTRY(left_sides),
-    CURSOR_ENTRY(right_sides),         CURSOR_ENTRY(top_left_corners),     CURSOR_ENTRY(top_right_corners),
-    CURSOR_ENTRY(bottom_left_corners), CURSOR_ENTRY(bottom_right_corners), CURSOR_ENTRY(left_ptrs)};
-
-namespace mf = mir::frontend;
-
-mf::XWaylandWM::XWaylandWM(std::shared_ptr<WaylandConnector> wayland_connector, wl_client* wayland_client, int fd)
-    : wm_fd{fd},
-      connection{std::make_shared<XCBConnection>(fd)},
+mf::XWaylandWM::XWaylandWM(
+    std::shared_ptr<WaylandConnector> wayland_connector,
+    wl_client* wayland_client,
+    Fd const& fd,
+    std::shared_ptr<dispatch::MultiplexingDispatchable> const& dispatcher,
+    float assumed_surface_scale)
+    : connection{std::make_shared<XCBConnection>(fd)},
+      xfixes{init_xfixes(*connection)},
       wayland_connector(wayland_connector),
-      dispatcher{std::make_shared<mir::dispatch::MultiplexingDispatchable>()},
       wayland_client{wayland_client},
-      wm_shell{std::static_pointer_cast<XWaylandWMShell>(wayland_connector->get_extension("x11-support"))}
+      wm_shell{std::static_pointer_cast<XWaylandWMShell>(wayland_connector->get_extension("x11-support"))},
+      wayland_executor{*wm_shell->wayland_executor},
+      cursors{std::make_unique<XWaylandCursors>(connection)},
+      clipboard_source{std::make_unique<XWaylandClipboardSource>(*connection, dispatcher, wm_shell->clipboard)},
+      clipboard_provider{std::make_unique<XWaylandClipboardProvider>(connection, dispatcher, wm_shell->clipboard)},
+      wm_window{create_wm_window(*connection)},
+      scene_observer{std::make_shared<XWaylandSceneObserver>(this)},
+      client_manager{std::make_shared<XWaylandClientManager>(wm_shell->shell)},
+      assumed_surface_scale{assumed_surface_scale}
 {
-    if (xcb_connection_has_error(*connection))
-    {
-        mir::log_error("XWAYLAND: xcb_connect_to_fd failed");
-        close(wm_fd);
-        return;
-    }
-
-    wm_dispatcher =
-        std::make_shared<mir::dispatch::ReadableFd>(mir::Fd{mir::IntOwnedFd{wm_fd}}, [this]() { handle_events(); });
-    dispatcher->add_watch(wm_dispatcher);
-
-    event_thread = std::make_unique<mir::dispatch::ThreadedDispatcher>(
-        "Mir/X11 WM Reader", dispatcher, []() { mir::terminate_with_current_exception(); });
-
-    wm_get_resources();
-    setup_visual_and_colormap();
-
     uint32_t const attrib_values[]{
         XCB_EVENT_MASK_SUBSTRUCTURE_NOTIFY | XCB_EVENT_MASK_SUBSTRUCTURE_REDIRECT | XCB_EVENT_MASK_PROPERTY_CHANGE};
 
@@ -108,32 +162,83 @@ mf::XWaylandWM::XWaylandWM(std::shared_ptr<WaylandConnector> wayland_connector,
     xcb_composite_redirect_subwindows(*connection, connection->root_window(), XCB_COMPOSITE_REDIRECT_MANUAL);
 
     xcb_atom_t const supported[]{
-        connection->net_wm_moveresize,
-        connection->net_wm_state,
-        connection->net_wm_state_fullscreen,
-        connection->net_wm_state_maximized_vert,
-        connection->net_wm_state_maximized_horz,
-        connection->net_active_window};
+        connection->_NET_WM_MOVERESIZE,
+        connection->_NET_WM_STATE,
+        connection->_NET_WM_STATE_FULLSCREEN,
+        connection->_NET_WM_STATE_MAXIMIZED_VERT,
+        connection->_NET_WM_STATE_MAXIMIZED_HORZ,
+        connection->_NET_ACTIVE_WINDOW};
 
-    connection->set_property<XCBType::ATOM>(connection->root_window(), connection->net_supported, supported);
+    connection->set_property<XCBType::ATOM>(connection->root_window(), connection->_NET_SUPPORTED, supported);
 
     connection->set_property<XCBType::WINDOW>(
         connection->root_window(),
-        connection->net_active_window,
+        connection->_NET_ACTIVE_WINDOW,
         static_cast<xcb_window_t>(XCB_WINDOW_NONE));
-    wm_selector();
 
     connection->flush();
 
-    create_wm_cursor();
-    set_cursor(connection->root_window(), CursorLeftPointer);
+    cursors->apply_default_to(connection->root_window());
 
-    create_wm_window();
-    connection->flush();
+    wm_shell->surface_stack->add_observer(scene_observer);
+
+    // Detect and manage any windows that already exist
+    auto const query_tree_cookie = xcb_query_tree(*connection, connection->root_window());
+    auto const query_tree_reply = make_unique_cptr(xcb_query_tree_reply(*connection, query_tree_cookie, nullptr));
+    if (query_tree_reply)
+    {
+        std::vector<std::function<void()>> window_setup_funcs;
+        for (int i = 0; i < xcb_query_tree_children_length(query_tree_reply.get()); ++i)
+        {
+            xcb_window_t const window = xcb_query_tree_children(query_tree_reply.get())[i];
+            if (!connection->is_ours(window))
+            {
+                if (verbose_xwayland_logging_enabled())
+                {
+                    log_debug("Window %s already exists", connection->window_debug_string(window).c_str());
+                }
+
+                auto const geometry_cookie = xcb_get_geometry(*connection, window);
+                auto const attrs_cookie = xcb_get_window_attributes(*connection, window);
+                window_setup_funcs.push_back([this, window, geometry_cookie, attrs_cookie]()
+                    {
+                        auto const geometry_reply = make_unique_cptr(xcb_get_geometry_reply(*connection, geometry_cookie, nullptr));
+                        auto const attrs_reply = make_unique_cptr(xcb_get_window_attributes_reply(*connection, attrs_cookie, nullptr));
+
+                        if (geometry_reply && attrs_reply)
+                        {
+                            manage_window(
+                                window,
+                                geom::Rectangle{
+                                    {geometry_reply->x, geometry_reply->y},
+                                    {geometry_reply->width, geometry_reply->height}},
+                                attrs_reply->override_redirect);
+                        }
+                        else
+                        {
+                            log_warning(
+                                "Faild to load geometry and attributes for %s",
+                                connection->window_debug_string(window).c_str());
+                        }
+                    });
+            }
+        }
+
+        for (auto const& f : window_setup_funcs)
+        {
+            f();
+        }
+    }
+    else
+    {
+        log_warning("Failed to query initial windows");
+    }
 }
 
 mf::XWaylandWM::~XWaylandWM()
 {
+    wm_shell->surface_stack->remove_observer(scene_observer);
+
     // clear the surfaces map and then destroy all surfaces
     std::map<xcb_window_t, std::shared_ptr<XWaylandSurface>> local_surfaces;
 
@@ -144,12 +249,12 @@ mf::XWaylandWM::~XWaylandWM()
     }
 
     if (verbose_xwayland_logging_enabled())
-        log_debug("Closing %d XWayland surface(s)...", local_surfaces.size());
+        log_debug("Closing %zu XWayland surface(s)...", local_surfaces.size());
 
     for (auto const& surface : local_surfaces)
     {
-        if (surface.second)
-            surface.second->close();
+        surface.second->close();
+        xcb_kill_client(*connection, surface.first);
     }
 
     local_surfaces.clear();
@@ -157,114 +262,39 @@ mf::XWaylandWM::~XWaylandWM()
     if (verbose_xwayland_logging_enabled())
         log_debug("...done closing surfaces");
 
-    if (event_thread)
-    {
-        dispatcher->remove_watch(wm_dispatcher);
-        event_thread.reset();
-    }
-
-    // xcb_cursors == 2 when its empty
-    if (xcb_cursors.size() != 2)
-    {
-        mir::log_info("Cleaning cursors");
-        for (auto xcb_cursor : xcb_cursors)
-        xcb_free_cursor(*connection, xcb_cursor);
-    }
-
-    close(wm_fd);
-}
-
-void mf::XWaylandWM::wm_selector()
-{
-    xcb_selection_request.requestor = XCB_NONE;
-
-    uint32_t const values[]{
-        XCB_EVENT_MASK_PROPERTY_CHANGE};
-
-    xcb_selection_window = xcb_generate_id(*connection);
-
-    xcb_create_window(
-        *connection,
-        XCB_COPY_FROM_PARENT,
-        xcb_selection_window,
-        connection->root_window(),
-        0, 0, // position
-        10, 10, // size
-        0, // border width
-        XCB_WINDOW_CLASS_INPUT_OUTPUT,
-        connection->screen()->root_visual,
-        XCB_CW_EVENT_MASK,
-        values);
-
-    xcb_set_selection_owner(*connection, xcb_selection_window, connection->clipboard_manager, XCB_TIME_CURRENT_TIME);
-
-    uint32_t const mask =
-        XCB_XFIXES_SELECTION_EVENT_MASK_SET_SELECTION_OWNER |
-        XCB_XFIXES_SELECTION_EVENT_MASK_SELECTION_WINDOW_DESTROY |
-        XCB_XFIXES_SELECTION_EVENT_MASK_SELECTION_CLIENT_CLOSE;
-
-    xcb_xfixes_select_selection_input(*connection, xcb_selection_window, connection->clipboard, mask);
-}
-
-void mf::XWaylandWM::set_cursor(xcb_window_t id, const CursorType &cursor)
-{
-    if (xcb_cursor == cursor)
-        return;
-
-    xcb_cursor = cursor;
-    uint32_t cursor_value_list = xcb_cursors[cursor];
-    xcb_change_window_attributes(*connection, id, XCB_CW_CURSOR, &cursor_value_list);
+    xcb_destroy_window(*connection, wm_window);
     connection->flush();
 }
 
-void mf::XWaylandWM::create_wm_cursor()
+
+void mf::XWaylandWM::handle_events()
 {
-    const char *name;
-    int count = ARRAY_LENGTH(cursors);
+    bool got_events = false;
 
-    xcb_cursors.clear();
-    xcb_cursors.reserve(count);
+    connection->verify_not_in_error_state();
 
-    for (int i = 0; i < count; i++)
+    while (xcb_generic_event_t* const event = xcb_poll_for_event(*connection))
     {
-        for (size_t j = 0; j < cursors[i].count; j++)
+        try
         {
-            name = cursors[i].names[j];
-            xcb_cursors.push_back(xcb_cursor_library_load_cursor(name));
-            if (xcb_cursors[i] != static_cast<xcb_cursor_t>(-1))
-                break;
+            handle_event(event);
         }
+        catch (...)
+        {
+            log(
+                logging::Severity::warning,
+                MIR_LOG_COMPONENT,
+                std::current_exception(),
+                "Error processing XCB event");
+        }
+        free(event);
+        got_events = true;
     }
-}
 
-void mf::XWaylandWM::create_wm_window()
-{
-    std::string const wm_name{"Mir XWM"};
-
-    wm_window = xcb_generate_id(*connection);
-    xcb_create_window(
-        *connection,
-        XCB_COPY_FROM_PARENT,
-        wm_window,
-        connection->root_window(),
-        0, 0, 10, 10, 0,
-        XCB_WINDOW_CLASS_INPUT_OUTPUT,
-        connection->screen()->root_visual,
-        0, NULL);
-
-    connection->set_property<XCBType::WINDOW>(wm_window, connection->net_supporting_wm_check, wm_window);
-    connection->set_property<XCBType::UTF8_STRING>(wm_window, connection->net_wm_name, wm_name);
-
-    connection->set_property<XCBType::WINDOW>(
-        connection->root_window(),
-        connection->net_supporting_wm_check,
-        wm_window);
-
-    /* Claim the WM_S0 selection even though we don't support
-     * the --replace functionality. */
-    xcb_set_selection_owner(*connection, wm_window, connection->wm_s0, XCB_TIME_CURRENT_TIME);
-
-    xcb_set_selection_owner(*connection, wm_window, connection->net_wm_cm_s0, XCB_TIME_CURRENT_TIME);
+    if (got_events)
+    {
+        connection->flush();
+    }
 }
 
 auto mf::XWaylandWM::get_wm_surface(
@@ -273,7 +303,7 @@ auto mf::XWaylandWM::get_wm_surface(
     std::lock_guard<std::mutex> lock{mutex};
 
     auto const surface = surfaces.find(xcb_window);
-    if (surface == surfaces.end() || !surface->second)
+    if (surface == surfaces.end())
         return std::experimental::nullopt;
     else
         return surface->second;
@@ -313,7 +343,7 @@ void mf::XWaylandWM::set_focus(xcb_window_t xcb_window, bool should_be_focused)
     {
         connection->set_property<XCBType::WINDOW>(
             connection->root_window(),
-            connection->net_active_window,
+            connection->_NET_ACTIVE_WINDOW,
             xcb_window);
 
         if (auto const surface = get_wm_surface(xcb_window))
@@ -325,7 +355,7 @@ void mf::XWaylandWM::set_focus(xcb_window_t xcb_window, bool should_be_focused)
     {
         connection->set_property<XCBType::WINDOW>(
             connection->root_window(),
-            connection->net_active_window,
+            connection->_NET_ACTIVE_WINDOW,
             static_cast<xcb_window_t>(XCB_WINDOW_NONE));
 
         xcb_set_input_focus_checked(
@@ -338,43 +368,160 @@ void mf::XWaylandWM::set_focus(xcb_window_t xcb_window, bool should_be_focused)
     connection->flush();
 }
 
-void mf::XWaylandWM::run_on_wayland_thread(std::function<void()>&& work)
+void mf::XWaylandWM::remember_scene_surface(std::weak_ptr<scene::Surface> const& scene_surface, xcb_window_t window)
 {
-    wayland_connector->run_on_wayland_display([work = move(work)](auto){ work(); });
+    std::lock_guard<std::mutex> lock{mutex};
+    scene_surfaces.insert(std::make_pair(scene_surface, window));
+    scene_surface_set.insert(scene_surface);
 }
 
-/* Events */
-void mf::XWaylandWM::handle_events()
+void mf::XWaylandWM::forget_scene_surface(std::weak_ptr<scene::Surface> const& scene_surface)
 {
-    bool got_events = false;
+    std::lock_guard<std::mutex> lock{mutex};
+    scene_surfaces.erase(scene_surface);
+    scene_surface_set.erase(scene_surface);
+}
+
+void mf::XWaylandWM::surfaces_reordered(scene::SurfaceSet const& affected_surfaces)
+{
+    bool our_surfaces_affected = false;
 
-    while (xcb_generic_event_t* const event = xcb_poll_for_event(*connection))
     {
-        try
+        std::lock_guard<std::mutex> lock{mutex};
+        for (auto const& surface : affected_surfaces)
         {
-            handle_event(event);
+            if (scene_surfaces.find(surface) != scene_surfaces.end())
+            {
+                our_surfaces_affected = true;
+                break;
+            }
         }
-        catch (...)
-        {
-            log(
-                logging::Severity::warning,
-                MIR_LOG_COMPONENT,
-                std::current_exception(),
-                "Failed to handle xcb event.");
-        }
-        free(event);
-        got_events = true;
     }
 
-    if (got_events)
+    if (our_surfaces_affected)
     {
-        connection->flush();
+        restack_surfaces();
     }
 }
 
+void mf::XWaylandWM::restack_surfaces()
+{
+    std::vector<xcb_window_t> new_order;
+
+    {
+        std::lock_guard<std::mutex> lock{mutex};
+        auto const new_surface_order = wm_shell->surface_stack->stacking_order_of(scene_surface_set);
+        for (auto const& surface : new_surface_order)
+        {
+            auto const surface_window = scene_surfaces.find(surface);
+            if (surface_window != scene_surfaces.end())
+            {
+                new_order.push_back(surface_window->second);
+            }
+        }
+    }
+
+    xcb_window_t window_below = 0;
+    for (auto const& window : new_order)
+    {
+        if (window_below)
+        {
+            if (verbose_xwayland_logging_enabled())
+            {
+                log_debug(
+                    "Stacking %s on top of %s",
+                    connection->window_debug_string(window).c_str(),
+                    connection->window_debug_string(window_below).c_str());
+            }
+
+            connection->configure_window(
+                window,
+                std::experimental::nullopt,
+                std::experimental::nullopt,
+                window_below,
+                XCB_STACK_MODE_ABOVE);
+        }
+        window_below = window;
+    }
+
+    connection->flush();
+}
+
+void mf::XWaylandWM::manage_window(xcb_window_t window, geom::Rectangle const& geometry, bool override_redirect)
+{
+    if (verbose_xwayland_logging_enabled())
+    {
+        auto const props_cookie = xcb_list_properties(*connection, window);
+        auto const props_reply = xcb_list_properties_reply(*connection, props_cookie, nullptr);
+        if (props_reply)
+        {
+            std::vector<std::function<void()>> functions;
+            int const prop_count = xcb_list_properties_atoms_length(props_reply);
+            for (int i = 0; i < prop_count; i++)
+            {
+                auto const atom = xcb_list_properties_atoms(props_reply)[i];
+
+                auto const log_prop = [this, atom](std::string const& value)
+                    {
+                        auto const prop_name = connection->query_name(atom);
+                        log_debug(
+                            "  | %s: %s",
+                            prop_name.c_str(),
+                            value.c_str());
+                    };
+
+                functions.push_back(connection->read_property(
+                    window,
+                    atom,
+                    {
+                        [this, log_prop](xcb_get_property_reply_t* reply)
+                        {
+                            auto const reply_str = connection->reply_debug_string(reply);
+                            log_prop(reply_str);
+                        },
+                        [log_prop](std::string const& message)
+                        {
+                            log_prop("error getting value: " + message);
+                        }
+                    }));
+            }
+            free(props_reply);
+
+            log_debug("%s has %d initial propertie(s):", connection->window_debug_string(window).c_str(), prop_count);
+            for (auto const& f : functions)
+                f();
+        }
+        else
+        {
+            log_debug("%s's initial properties failed to load", connection->window_debug_string(window).c_str());
+        }
+    }
+
+    std::lock_guard<std::mutex> lock{mutex};
+
+    if (surfaces.find(window) != surfaces.end())
+    {
+        // If a window is created during startup, we may be double-notified of it
+        return;
+    }
+
+    surfaces[window] = std::make_shared<XWaylandSurface>(
+        this,
+        connection,
+        *wm_shell,
+        client_manager,
+        window,
+        geometry,
+        override_redirect,
+        assumed_surface_scale);
+}
+
 void mf::XWaylandWM::handle_event(xcb_generic_event_t* event)
 {
-    int type = event->response_type & ~0x80;
+    // see https://www.systutorials.com/docs/linux/man/3-xcb-requests/
+    int const xcb_error_type = 0;
+
+    auto const type = event->response_type & ~0x80;
     switch (type)
     {
     case XCB_BUTTON_PRESS:
@@ -437,9 +584,36 @@ void mf::XWaylandWM::handle_event(xcb_generic_event_t* event)
         break;
     case XCB_FOCUS_IN:
         handle_focus_in(reinterpret_cast<xcb_focus_in_event_t*>(event));
+        break;
+    case XCB_SELECTION_REQUEST:
+        clipboard_provider->selection_request_event(reinterpret_cast<xcb_selection_request_event_t*>(event));
+        break;
+    case XCB_SELECTION_NOTIFY:
+        clipboard_source->selection_notify_event(reinterpret_cast<xcb_selection_notify_event_t*>(event));
+        break;
+    case xcb_error_type:
+        handle_error(reinterpret_cast<xcb_generic_error_t*>(event));
+        break;
     default:
         break;
     }
+
+    if (xfixes)
+    {
+        auto const xfixes_type = event->response_type - xfixes->first_event;
+        switch (xfixes_type)
+        {
+        case XCB_XFIXES_SELECTION_NOTIFY:
+            // Order of calls should not matter
+            clipboard_provider->xfixes_selection_notify_event(
+                reinterpret_cast<xcb_xfixes_selection_notify_event_t*>(event));
+            clipboard_source->xfixes_selection_notify_event(
+                reinterpret_cast<xcb_xfixes_selection_notify_event_t*>(event));
+            break;
+        default:
+            break;
+        }
+    }
 }
 
 void mf::XWaylandWM::handle_property_notify(xcb_property_notify_event_t *event)
@@ -468,14 +642,16 @@ void mf::XWaylandWM::handle_property_notify(xcb_property_notify_event_t *event)
             auto const reply_function = connection->read_property(
                 event->window,
                 event->atom,
-                [this, log_prop](xcb_get_property_reply_t* reply)
                 {
-                    auto const reply_str = connection->reply_debug_string(reply);
-                    log_prop(reply_str);
-                },
-                [log_prop]()
-                {
-                    log_prop("Error getting value");
+                    [this, log_prop](xcb_get_property_reply_t* reply)
+                    {
+                        auto const reply_str = connection->reply_debug_string(reply);
+                        log_prop(reply_str);
+                    },
+                    [log_prop](std::string const& message)
+                    {
+                        log_prop("error getting value: " + message);
+                    }
                 });
 
             reply_function();
@@ -500,69 +676,14 @@ void mf::XWaylandWM::handle_create_notify(xcb_create_notify_event_t *event)
 
         if (event->border_width)
             log_warning("border width unsupported (border width %d)", event->border_width);
-
-        auto const props_cookie = xcb_list_properties(*connection, event->window);
-        auto const props_reply = xcb_list_properties_reply(*connection, props_cookie, nullptr);
-        if (props_reply)
-        {
-            std::vector<std::function<void()>> functions;
-            int const prop_count = xcb_list_properties_atoms_length(props_reply);
-            for (int i = 0; i < prop_count; i++)
-            {
-                auto const atom = xcb_list_properties_atoms(props_reply)[i];
-
-                auto const log_prop = [this, atom](std::string const& value)
-                    {
-                        auto const prop_name = connection->query_name(atom);
-                        log_debug(
-                            "                  | %s: %s",
-                            prop_name.c_str(),
-                            value.c_str());
-                    };
-
-                functions.push_back(connection->read_property(
-                    event->window,
-                    atom,
-                    [this, log_prop](xcb_get_property_reply_t* reply)
-                    {
-                        auto const reply_str = connection->reply_debug_string(reply);
-                        log_prop(reply_str);
-                    },
-                    [log_prop]()
-                    {
-                        log_prop("Error getting value");
-                    }));
-            }
-            free(props_reply);
-
-            log_debug("                  initial properties: %d", prop_count);
-            for (auto const& f : functions)
-                f();
-        }
-        else
-        {
-            log_debug("                  initial properties: failed to load");
-        }
     }
 
     if (!connection->is_ours(event->window))
     {
-        auto const surface = std::make_shared<XWaylandSurface>(
-            this,
-            connection,
-            wm_shell->seat,
-            wm_shell->shell,
-            event);
-
-        {
-            std::lock_guard<std::mutex> lock{mutex};
-
-            if (surfaces.find(event->window) != surfaces.end())
-                BOOST_THROW_EXCEPTION(
-                    std::runtime_error(connection->window_debug_string(event->window) + " created, but already known"));
-
-            surfaces[event->window] = surface;
-        }
+        manage_window(
+            event->window,
+            geom::Rectangle{{event->x, event->y}, {event->width, event->height}},
+            event->override_redirect);
     }
 }
 
@@ -658,13 +779,13 @@ void mf::XWaylandWM::handle_client_message(xcb_client_message_event_t *event)
     if (auto const surface = get_wm_surface(event->window))
     {
         // TODO: net_active_window?
-        if (event->type == connection->net_wm_moveresize)
+        if (event->type == connection->_NET_WM_MOVERESIZE)
             handle_move_resize(surface.value(), event);
-        else if (event->type == connection->net_wm_state)
+        else if (event->type == connection->_NET_WM_STATE)
             surface.value()->net_wm_state_client_message(event->data.data32);
-        else if (event->type == connection->wm_change_state)
+        else if (event->type == connection->WM_CHANGE_STATE)
             surface.value()->wm_change_state_client_message(event->data.data32);
-        else if (event->type == connection->wl_surface_id)
+        else if (event->type == connection->WL_SURFACE_ID)
             handle_surface_id(surface.value(), event);
     }
 }
@@ -684,14 +805,15 @@ void mf::XWaylandWM::handle_surface_id(
 {
     uint32_t id = event->data.data32[0];
 
-    wayland_connector->run_on_wayland_display([
+    wayland_executor.spawn([
             wayland_connector = wayland_connector,
             client=wayland_client,
+            scale=assumed_surface_scale,
             id,
             weak_surface,
-            weak_shell = std::weak_ptr<shell::Shell>{wm_shell->shell}](auto)
+            weak_shell = std::weak_ptr<shell::Shell>{wm_shell->shell}]()
         {
-            wayland_connector->on_surface_created(client, id, [weak_surface, weak_shell](WlSurface* wl_surface)
+            wayland_connector->on_surface_created(client, id, [scale, weak_surface, weak_shell](WlSurface* wl_surface)
                 {
                     auto const surface = weak_surface.lock();
                     auto const shell = weak_shell.lock();
@@ -700,7 +822,7 @@ void mf::XWaylandWM::handle_surface_id(
                         surface->attach_wl_surface(wl_surface);
 
                         // Will destroy itself
-                        new XWaylandSurfaceRole{shell, surface, wl_surface};
+                        new XWaylandSurfaceRole{shell, surface, wl_surface, scale};
                     }
                     else
                     {
@@ -760,16 +882,10 @@ void mf::XWaylandWM::handle_focus_in(xcb_focus_in_event_t* event)
 {
     if (verbose_xwayland_logging_enabled())
     {
-        std::string mode_str;
-        switch (event->mode)
-        {
-        case XCB_NOTIFY_MODE_NORMAL:        mode_str = "normal focus"; break;
-        case XCB_NOTIFY_MODE_GRAB:          mode_str = "focus grabbed"; break;
-        case XCB_NOTIFY_MODE_UNGRAB:        mode_str = "focus ungrabbed"; break;
-        case XCB_NOTIFY_MODE_WHILE_GRABBED: mode_str = "focus while grabbed"; break;
-        default: mode_str = "unknown focus mode " + std::to_string(event->mode); break;
-        }
-        log_debug("XCB_FOCUS_IN %s on %s", mode_str.c_str(), connection->window_debug_string(event->event).c_str());
+        log_debug(
+            "XCB_FOCUS_IN %s on %s",
+            focus_mode_to_string(event->mode).c_str(),
+            connection->window_debug_string(event->event).c_str());
     }
 
     // Ignore grabs
@@ -782,144 +898,10 @@ void mf::XWaylandWM::handle_focus_in(xcb_focus_in_event_t* event)
     }
 }
 
-// Cursor
-xcb_cursor_t mf::XWaylandWM::xcb_cursor_image_load_cursor(const XcursorImage *img)
+void mf::XWaylandWM::handle_error(xcb_generic_error_t* event)
 {
-    xcb_connection_t *c = *connection;
-    xcb_screen_iterator_t s = xcb_setup_roots_iterator(xcb_get_setup(c));
-    xcb_screen_t *screen = s.data;
-    xcb_gcontext_t gc;
-    xcb_pixmap_t pix;
-    xcb_render_picture_t pic;
-    xcb_cursor_t cursor;
-    int stride = img->width * 4;
-
-    pix = xcb_generate_id(c);
-    xcb_create_pixmap(c, 32, pix, screen->root, img->width, img->height);
-
-    pic = xcb_generate_id(c);
-    xcb_render_create_picture(c, pic, pix, xcb_format_rgba.id, 0, 0);
-
-    gc = xcb_generate_id(c);
-    xcb_create_gc(c, gc, pix, 0, 0);
-
-    xcb_put_image(c, XCB_IMAGE_FORMAT_Z_PIXMAP, pix, gc, img->width, img->height, 0, 0, 0, 32, stride * img->height,
-                  (uint8_t *)img->pixels);
-    xcb_free_gc(c, gc);
-
-    cursor = xcb_generate_id(c);
-    xcb_render_create_cursor(c, cursor, pic, img->xhot, img->yhot);
-
-    xcb_render_free_picture(c, pic);
-    xcb_free_pixmap(c, pix);
-
-    return cursor;
-}
-
-xcb_cursor_t mf::XWaylandWM::xcb_cursor_images_load_cursor(const XcursorImages *images)
-{
-    if (images->nimage != 1)
-        return -1;
-
-    return xcb_cursor_image_load_cursor(images->images[0]);
-}
-
-xcb_cursor_t mf::XWaylandWM::xcb_cursor_library_load_cursor(const char *file)
-{
-    xcb_cursor_t cursor;
-    XcursorImages *images;
-    char *v = NULL;
-    int size = 0;
-
-    if (!file)
-        return 0;
-
-    v = getenv("XCURSOR_SIZE");
-    if (v)
-        size = atoi(v);
-
-    if (!size)
-        size = 32;
-
-    images = XcursorLibraryLoadImages(file, NULL, size);
-    if (!images)
-        return -1;
-
-    cursor = xcb_cursor_images_load_cursor(images);
-    XcursorImagesDestroy(images);
-
-    return cursor;
-}
-
-void mf::XWaylandWM::wm_get_resources()
-{
-    xcb_xfixes_query_version_cookie_t xfixes_cookie;
-    xcb_xfixes_query_version_reply_t *xfixes_reply;
-    xcb_render_query_pict_formats_reply_t *formats_reply;
-    xcb_render_query_pict_formats_cookie_t formats_cookie;
-    xcb_render_pictforminfo_t *formats;
-    uint32_t i;
-
-    xcb_prefetch_extension_data(*connection, &xcb_xfixes_id);
-    xcb_prefetch_extension_data(*connection, &xcb_composite_id);
-
-    formats_cookie = xcb_render_query_pict_formats(*connection);
-
-    xfixes = xcb_get_extension_data(*connection, &xcb_xfixes_id);
-    if (!xfixes || !xfixes->present)
-        log_warning("xfixes not available");
-
-    xfixes_cookie = xcb_xfixes_query_version(*connection, XCB_XFIXES_MAJOR_VERSION, XCB_XFIXES_MINOR_VERSION);
-    xfixes_reply = xcb_xfixes_query_version_reply(*connection, xfixes_cookie, NULL);
-
     if (verbose_xwayland_logging_enabled())
-        log_debug("xfixes version: %d.%d", xfixes_reply->major_version, xfixes_reply->minor_version);
-
-    free(xfixes_reply);
-
-    formats_reply = xcb_render_query_pict_formats_reply(*connection, formats_cookie, 0);
-    if (formats_reply == NULL)
-        return;
-
-    formats = xcb_render_query_pict_formats_formats(formats_reply);
-    for (i = 0; i < formats_reply->num_formats; i++)
     {
-        if (formats[i].direct.red_mask != 0xff && formats[i].direct.red_shift != 16)
-            continue;
-        if (formats[i].type == XCB_RENDER_PICT_TYPE_DIRECT && formats[i].depth == 24)
-            xcb_format_rgb = formats[i];
-        if (formats[i].type == XCB_RENDER_PICT_TYPE_DIRECT && formats[i].depth == 32 &&
-            formats[i].direct.alpha_mask == 0xff && formats[i].direct.alpha_shift == 24)
-            xcb_format_rgba = formats[i];
+        log_warning("XWayland XCB error: %s", connection->error_debug_string(event).c_str());
     }
-
-    free(formats_reply);
-}
-
-void mf::XWaylandWM::setup_visual_and_colormap()
-{
-    xcb_depth_iterator_t depthIterator = xcb_screen_allowed_depths_iterator(connection->screen());
-    xcb_visualtype_t *visualType = nullptr;
-    xcb_visualtype_iterator_t visualTypeIterator;
-    while (depthIterator.rem > 0)
-    {
-        if (depthIterator.data->depth == 32)
-        {
-            visualTypeIterator = xcb_depth_visuals_iterator(depthIterator.data);
-            visualType = visualTypeIterator.data;
-            break;
-        }
-
-        xcb_depth_next(&depthIterator);
-    }
-
-    if (!visualType)
-    {
-        mir::log_warning("No 32-bit visualtype");
-        return;
-    }
-
-    xcb_visual_id = visualType->visual_id;
-    xcb_colormap = xcb_generate_id(*connection);
-    xcb_create_colormap(*connection, XCB_COLORMAP_ALLOC_NONE, xcb_colormap, connection->root_window(), xcb_visual_id);
 }
