@@ -20,84 +20,78 @@
 #define MIR_FRONTEND_XWAYLAND_WM_H
 
 #include "mir/dispatch/threaded_dispatcher.h"
+#include "mir/geometry/rectangle.h"
 #include "wayland_connector.h"
 #include "xcb_connection.h"
 
 #include <map>
+#include <set>
 #include <thread>
 #include <experimental/optional>
 #include <mutex>
 
 #include <wayland-server-core.h>
-
-#include <X11/Xcursor/Xcursor.h>
-#include <xcb/composite.h>
-#include <xcb/xcb.h>
 #include <xcb/xfixes.h>
 
 namespace mir
 {
+namespace scene
+{
+using SurfaceSet = std::set<std::weak_ptr<Surface>, std::owner_less<std::weak_ptr<Surface>>>;
+class Surface;
+}
 namespace dispatch
 {
-class ReadableFd;
-class ThreadedDispatcher;
 class MultiplexingDispatchable;
 }
 namespace frontend
 {
 class XWaylandSurface;
 class XWaylandWMShell;
+class XWaylandCursors;
+class XWaylandClientManager;
+class XWaylandClipboardSource;
+class XWaylandClipboardProvider;
 
-template<typename T, size_t length>
-constexpr size_t length_of(T(&)[length])
-{
-    return length;
-}
+class XWaylandSceneObserver;
 
 class XWaylandWM
 {
 private:
-    int const wm_fd;
     std::shared_ptr<XCBConnection> const connection;
 
 public:
-    XWaylandWM(std::shared_ptr<WaylandConnector> wayland_connector, wl_client* wayland_client, int fd);
+    /// Takes ownership of the given FD
+    XWaylandWM(
+        std::shared_ptr<WaylandConnector> wayland_connector,
+        wl_client* wayland_client,
+        Fd const& fd,
+        std::shared_ptr<dispatch::MultiplexingDispatchable> const& dispatcher,
+        float assumed_surface_scale);
     ~XWaylandWM();
 
+    /// Called by the XWayland connector when there may be new events
+    void handle_events();
+
     auto get_wm_surface(xcb_window_t xcb_window) -> std::experimental::optional<std::shared_ptr<XWaylandSurface>>;
     auto get_focused_window() -> std::experimental::optional<xcb_window_t>;
     void set_focus(xcb_window_t xcb_window, bool should_be_focused);
-    void run_on_wayland_thread(std::function<void()>&& work);
+    void remember_scene_surface(std::weak_ptr<scene::Surface> const& scene_surface, xcb_window_t window);
+    void forget_scene_surface(std::weak_ptr<scene::Surface> const& scene_surface);
+
+    void surfaces_reordered(scene::SurfaceSet const& affected_surfaces);
 
 private:
-    enum CursorType
-    {
-        CursorUnset = -1,
-        CursorTop,
-        CursorBottom,
-        CursorLeft,
-        CursorRight,
-        CursorTopLeft,
-        CursorTopRight,
-        CursorBottomLeft,
-        CursorBottomRight,
-        CursorLeftPointer
-    };
+    XWaylandWM(XWaylandWM const&) = delete;
+    XWaylandWM& operator=(XWaylandWM const&) = delete;
 
-    void create_wm_window();
-    void wm_selector();
+    void restack_surfaces();
 
-    void create_window(xcb_window_t id);
-    void set_cursor(xcb_window_t id, const CursorType &cursor);
-    void create_wm_cursor();
-    void wm_get_resources();
-    void setup_visual_and_colormap();
+    /// Called for all windows at startup and whenever a window is created
+    /// May occasionally be called multiple times for the same window
+    void manage_window(xcb_window_t window, geometry::Rectangle const& geometry, bool override_redirect);
 
-    // Event handeling
-    void handle_events();
     void handle_event(xcb_generic_event_t* event);
-
-    // Events
     void handle_create_notify(xcb_create_notify_event_t *event);
     void handle_motion_notify(xcb_motion_notify_event_t *event);
     void handle_property_notify(xcb_property_notify_event_t *event);
@@ -110,32 +104,33 @@ private:
     void handle_unmap_notify(xcb_unmap_notify_event_t *event);
     void handle_destroy_notify(xcb_destroy_notify_event_t *event);
     void handle_focus_in(xcb_focus_in_event_t* event);
+    void handle_error(xcb_generic_error_t* event);
 
-    std::mutex mutex;
-
-    // Cursor
-    xcb_cursor_t xcb_cursor_image_load_cursor(const XcursorImage *img);
-    xcb_cursor_t xcb_cursor_images_load_cursor(const XcursorImages *images);
-    xcb_cursor_t xcb_cursor_library_load_cursor(const char *file);
-
+    xcb_query_extension_reply_t const* const xfixes; ///< Must not be freed, can be null
     std::shared_ptr<WaylandConnector> const wayland_connector;
-    std::shared_ptr<dispatch::MultiplexingDispatchable> const dispatcher;
     wl_client* const wayland_client;
     std::shared_ptr<XWaylandWMShell> const wm_shell;
+    Executor& wayland_executor;
+    std::unique_ptr<XWaylandCursors> const cursors;
+    std::unique_ptr<XWaylandClipboardSource> const clipboard_source;
+    std::unique_ptr<XWaylandClipboardProvider> const clipboard_provider;
+    xcb_window_t const wm_window;
+    std::shared_ptr<XWaylandSceneObserver> const scene_observer;
+    std::shared_ptr<XWaylandClientManager> const client_manager;
+    /// The scale we assume applications are rendering at. If this doesn't match the scale an app is actually rendering
+    /// at the app will appear the wrong size. If this matches the app but both are smaller than the output scale, the
+    /// app will appear the correct size but blurry.
+    float const assumed_surface_scale;
 
-    xcb_window_t wm_window;
+    std::mutex mutex;
     std::map<xcb_window_t, std::shared_ptr<XWaylandSurface>> surfaces;
+    std::map<std::weak_ptr<
+        scene::Surface>,
+        xcb_window_t,
+        std::owner_less<std::weak_ptr<scene::Surface>>> scene_surfaces;
+    /// Could be regenerated from scene_surfaces at any time, but more efficient to keep this up to date
+    std::set<std::weak_ptr<scene::Surface>, std::owner_less<std::weak_ptr<scene::Surface>>> scene_surface_set;
     std::experimental::optional<xcb_window_t> focused_window;
-    std::shared_ptr<dispatch::ReadableFd> wm_dispatcher;
-    int xcb_cursor;
-    std::vector<xcb_cursor_t> xcb_cursors;
-    xcb_window_t xcb_selection_window;
-    xcb_selection_request_event_t xcb_selection_request;
-    xcb_render_pictforminfo_t xcb_format_rgb, xcb_format_rgba;
-    const xcb_query_extension_reply_t *xfixes;
-    std::unique_ptr<dispatch::ThreadedDispatcher> event_thread;
-    xcb_visualid_t xcb_visual_id;
-    xcb_colormap_t xcb_colormap;
 };
 } /* frontend */
 } /* mir */
