@@ -63,8 +63,17 @@ void mgo::detail::EGLDisplayHandle::initialize()
     if (eglInitialize(egl_display, &major, &minor) == EGL_FALSE)
         BOOST_THROW_EXCEPTION(mg::egl_error("Failed to initialize EGL"));
 
-    if ((major != 1) || (minor != 4))
-        BOOST_THROW_EXCEPTION(std::runtime_error("EGL version 1.4 needed"));
+    if (major > 1)
+    {
+        BOOST_THROW_EXCEPTION((
+            std::runtime_error{"EGL version 1.x required, found: " +
+            std::to_string(major) + "." + std::to_string(minor)}));
+    }
+    if (minor < 4)
+        BOOST_THROW_EXCEPTION(
+            std::runtime_error(
+                std::string{"EGL version ≥ 1.4 needed, found "} +
+                std::to_string(major) + "." + std::to_string(minor)));
 }
 
 mgo::detail::EGLDisplayHandle::~EGLDisplayHandle() noexcept
@@ -150,7 +159,7 @@ void mgo::Display::configure(mg::DisplayConfiguration const& conf)
         {
             if (output.connected && output.preferred_mode_index < output.modes.size())
             {
-                eglBindAPI(MIR_SERVER_EGL_OPENGL_API);
+                eglBindAPI(EGL_OPENGL_ES_API);
                 auto raw_db = new mgo::DisplayBuffer{
                     SurfacelessEGLContext{egl_display, egl_context_shared},
                     output.extents()};
@@ -189,15 +198,10 @@ std::shared_ptr<mg::Cursor> mgo::Display::create_hardware_cursor()
 
 std::unique_ptr<mir::renderer::gl::Context> mgo::Display::create_gl_context() const
 {
-    eglBindAPI(MIR_SERVER_EGL_OPENGL_API);
+    eglBindAPI(EGL_OPENGL_ES_API);
     return std::make_unique<SurfacelessEGLContext>(egl_display, egl_context_shared);
 }
 
-mg::NativeDisplay* mgo::Display::native_display()
-{
-    return this;
-}
-
 mg::Frame mgo::Display::last_frame_on(unsigned) const
 {
     return {};
