@@ -72,9 +72,7 @@ private:
 
 }
 
-class Display : public graphics::Display,
-                public graphics::NativeDisplay,
-                public renderer::gl::ContextSource
+class Display : public graphics::Display
 {
 public:
     Display(EGLNativeDisplayType egl_native_display,
@@ -102,7 +100,6 @@ public:
     std::shared_ptr<Cursor> create_hardware_cursor() override;
     std::unique_ptr<VirtualOutput> create_virtual_output(int width, int height) override;
 
-    NativeDisplay* native_display() override;
     Frame last_frame_on(unsigned output_id) const override;
 
     std::unique_ptr<renderer::gl::Context> create_gl_context() const override;
