@@ -40,6 +40,7 @@
 #include "default_coordinate_translator.h"
 #include "unsupported_coordinate_translator.h"
 #include "timeout_application_not_responding_detector.h"
+#include "basic_clipboard.h"
 #include "mir/options/program_option.h"
 #include "mir/options/default_configuration.h"
 #include "mir/graphics/display_configuration.h"
@@ -194,8 +195,7 @@ mir::DefaultServerConfiguration::the_pixel_buffer()
         {
             auto as_context_source = [](mg::Display* display)
             {
-                auto const ctx = dynamic_cast<renderer::gl::ContextSource*>(
-                        display->native_display());
+                auto const ctx = dynamic_cast<renderer::gl::ContextSource*>(display);
                 if (!ctx)
                     BOOST_THROW_EXCEPTION(std::logic_error("Display does not support GL rendering"));
                 return ctx;
@@ -242,6 +242,16 @@ mir::DefaultServerConfiguration::the_coordinate_translator()
         });
 }
 
+auto mir::DefaultServerConfiguration::the_clipboard()
+-> std::shared_ptr<ms::Clipboard>
+{
+    return clipboard(
+        []()
+        {
+            return std::make_shared<ms::BasicClipboard>();
+        });
+}
+
 auto mir::DefaultServerConfiguration::the_application_not_responding_detector()
 -> std::shared_ptr<scene::ApplicationNotRespondingDetector>
 {
