@@ -99,11 +99,18 @@ msh::AbstractShell::~AbstractShell() noexcept
 
 void msh::AbstractShell::update_focused_surface_confined_region()
 {
-    auto const current_focus = focus_surface.lock();
-
-    if (current_focus && current_focus->confine_pointer_state() == mir_pointer_confined_to_window)
+    if (auto const current_focus = focus_surface.lock())
     {
-        seat->set_confinement_regions({current_focus->input_bounds()});
+        switch (current_focus->confine_pointer_state())
+        {
+        case mir_pointer_confined_oneshot:
+        case mir_pointer_confined_persistent:
+            seat->set_confinement_regions({current_focus->input_bounds()});
+            break;
+
+        default:
+            break;
+        }
     }
 }
 
@@ -148,15 +155,22 @@ auto msh::AbstractShell::create_surface(
     ms::SurfaceCreationParameters const& params,
     std::shared_ptr<ms::SurfaceObserver> const& observer) -> std::shared_ptr<ms::Surface>
 {
-    auto const build = [observer](std::shared_ptr<ms::Session> const& session, ms::SurfaceCreationParameters const& placed_params)
+    // Instead of a shared pointer, a local variable could be used and the lambda could capture a reference to it
+    // This should be safe, but could be the source of nasty bugs and crashes if the wm did something unexpected
+    auto const should_decorate = std::make_shared<bool>(false);
+    auto const build = [observer, should_decorate](std::shared_ptr<ms::Session> const& session, ms::SurfaceCreationParameters const& placed_params)
         {
+            if (placed_params.server_side_decorated.is_set() && placed_params.server_side_decorated.value())
+            {
+                *should_decorate = true;
+            }
             return session->create_surface(session, placed_params, observer);
         };
 
     auto const result = window_manager->add_surface(session, params, build);
     report->created_surface(*session, *result);
 
-    if (params.server_side_decorated.is_set() && params.server_side_decorated.value())
+    if (*should_decorate)
     {
         decoration_manager->decorate(result);
     }
@@ -224,15 +238,21 @@ void msh::AbstractShell::modify_surface(std::shared_ptr<scene::Session> const& s
         }
     }
 
-    if (modifications.confine_pointer.is_set() && focused_surface() == surface)
+    if (modifications.confine_pointer.is_set())
     {
-        if (surface->confine_pointer_state() == mir_pointer_confined_to_window)
+        auto const prev_state = surface->confine_pointer_state();
+        auto const new_state = modifications.confine_pointer.value();
+
+        if ((prev_state != mir_pointer_unconfined) && (new_state != mir_pointer_unconfined))
         {
-            seat->set_confinement_regions({surface->input_bounds()});
+            // TODO need to report "already_constrained"
         }
-        else
+
+        surface->set_confine_pointer_state(new_state);
+
+        if (focused_surface() == surface)
         {
-            seat->reset_confinement_regions();
+            update_confinement_for(surface);
         }
     }
 }
@@ -419,20 +439,36 @@ void msh::AbstractShell::notify_focus_locked(
             if (find(begin(new_focus_tree), end(new_focus_tree), item) == end(new_focus_tree))
             {
                 item->set_focus_state(mir_window_focus_state_unfocused);
+
+                // When a menu loses focus we should close and unmap it
+                if (item->type() == mir_window_type_menu)
+                {
+                    item->request_client_surface_close();
+                    item->hide();
+                }
             }
         }
 
         if (current_focus)
         {
             current_focus->remove_observer(focus_surface_observer);
+
+            switch (current_focus->confine_pointer_state())
+            {
+            case mir_pointer_confined_oneshot:
+            case mir_pointer_locked_oneshot:
+                seat->reset_confinement_regions();
+                current_focus->set_confine_pointer_state(mir_pointer_unconfined);
+                break;
+
+            default:
+                break;
+            }
         }
 
         if (surface)
         {
-            if (surface->confine_pointer_state() == mir_pointer_confined_to_window)
-            {
-                seat->set_confinement_regions({surface->input_bounds()});
-            }
+            update_confinement_for(surface);
 
             // Ensure the surface has really taken the focus before notifying it that it is focused
             input_targeter->set_focus(surface);
@@ -454,6 +490,30 @@ void msh::AbstractShell::notify_focus_locked(
     }
 }
 
+void msh::AbstractShell::update_confinement_for(std::shared_ptr<ms::Surface> const& surface) const
+{
+    switch (surface->confine_pointer_state())
+    {
+    case mir_pointer_locked_oneshot:
+    case mir_pointer_locked_persistent:
+    {
+        auto rectangle = surface->input_bounds();
+        rectangle.top_left = rectangle.top_left + as_displacement(0.5*rectangle.size);
+        rectangle.size = {1,1};
+        this->seat->set_confinement_regions({rectangle});
+        break;
+    }
+    case mir_pointer_confined_oneshot:
+    case mir_pointer_confined_persistent:
+        this->seat->set_confinement_regions({surface->input_bounds()});
+        break;
+
+    case mir_pointer_unconfined:
+        this->seat->reset_confinement_regions();
+        break;
+    }
+}
+
 void msh::AbstractShell::update_focus_locked(
     std::unique_lock<std::mutex> const& /*lock*/,
     std::shared_ptr<ms::Session> const& session,
