@@ -123,6 +123,9 @@ public:
                                     uint32_t pixel_format, uint32_t const bo_handles[4],
                                     uint32_t const pitches[4], uint32_t const offsets[4],
                                     uint32_t *buf_id, uint32_t flags));
+    MOCK_METHOD10(drmModeAddFB2WithModifiers, int(int fd, uint32_t width, uint32_t height,
+        uint32_t fourcc, uint32_t const handles[4], uint32_t const pitches[4],
+        uint32_t const offsets[4], uint64_t const modifiers[4], uint32_t *buf_id, uint32_t flags));
     MOCK_METHOD2(drmModeRmFB, int(int fd, uint32_t bufferId));
 
     MOCK_METHOD5(drmModePageFlip, int(int fd, uint32_t crtc_id, uint32_t fb_id,
