@@ -34,6 +34,7 @@ mtd::FakeX11Resources::FakeX11Resources()
       window{reinterpret_cast<Window>((long unsigned int)9876543210)}
 {
     std::memset(&keypress_event_return, 0, sizeof(XEvent));
+    std::memset(&key_release_event_return, 0, sizeof(XEvent));
     std::memset(&button_release_event_return, 0, sizeof(XEvent));
     std::memset(&expose_event_return, 0, sizeof(XEvent));
     std::memset(&focus_in_event_return, 0, sizeof(XEvent));
@@ -46,6 +47,7 @@ mtd::FakeX11Resources::FakeX11Resources()
     std::memset(&screen, 0, sizeof screen);
     visual_info.red_mask = 0xFF0000;
     keypress_event_return.type = KeyPress;
+    key_release_event_return.type = KeyRelease;
     button_release_event_return.type = ButtonRelease;
     button_release_event_return.xbutton.button = 0;
     expose_event_return.type = Expose;
@@ -92,6 +94,12 @@ mtd::MockX11::MockX11()
                                          return fake_x11.pending_events;
                                      }));
 
+    ON_CALL(*this, XEventsQueued(_,_))
+    .WillByDefault(InvokeWithoutArgs([this]()
+                                     {
+                                         return fake_x11.pending_events;
+                                     }));
+
     ON_CALL(*this, XGetGeometry(fake_x11.display,_,_,_,_,_,_,_,_))
     .WillByDefault(DoAll(SetArgPointee<5>(fake_x11.screen.width),
                          SetArgPointee<6>(fake_x11.screen.height),
@@ -165,6 +173,16 @@ int XNextEvent(Display* display, XEvent* event_return)
     return result;
 }
 
+int XPeekEvent(Display* display, XEvent* event_return)
+{
+    return global_mock->XPeekEvent(display, event_return);
+}
+
+int XEventsQueued(Display* display, int mode)
+{
+    return global_mock->XEventsQueued(display, mode);
+}
+
 int XLookupString(XKeyEvent* event_struct, char* buffer_return, int bytes_buffer, KeySym* keysym_return, XComposeStatus* status_in_out)
 {
     return global_mock->XLookupString(event_struct, buffer_return, bytes_buffer, keysym_return, status_in_out);
