@@ -22,6 +22,7 @@
 
 #include "mir/shared_library.h"
 #include "mir/geometry/rectangle.h"
+#include "mir/graphics/display.h"
 #include "mir_test_framework/executable_path.h"
 
 #include <boost/throw_exception.hpp>
@@ -29,6 +30,12 @@
 namespace geom = mir::geometry;
 namespace mtf = mir_test_framework;
 
+namespace
+{
+// This avoids an intermittent shutdown crash deleting a stub-graphics buffer (LP: #1728931)
+std::shared_ptr<void> delay_unloading_graphics_platform;
+}
+
 mtf::HeadlessTest::HeadlessTest()
 {
     add_to_environment("MIR_SERVER_PLATFORM_GRAPHICS_LIB", mtf::server_platform("graphics-dummy.so").c_str());
@@ -39,13 +46,19 @@ mtf::HeadlessTest::HeadlessTest()
     {
         return std::make_shared<mtf::HeadlessDisplayBufferCompositorFactory>();
     });
+
+    server.add_init_callback([server = &server]
+        {delay_unloading_graphics_platform = server->the_graphics_platform(); });
 }
 
-mtf::HeadlessTest::~HeadlessTest() noexcept = default;
-
-void mtf::HeadlessTest::preset_display(std::shared_ptr<mir::graphics::Display> const& display)
+mtf::HeadlessTest::~HeadlessTest() noexcept
 {
-    mtf::set_next_preset_display(display);
+    delay_unloading_graphics_platform.reset();
+}
+
+void mtf::HeadlessTest::preset_display(std::unique_ptr<mir::graphics::Display> display)
+{
+    mtf::set_next_preset_display(std::move(display));
 }
 
 void mtf::HeadlessTest::initial_display_layout(std::vector<geom::Rectangle> const& display_rects)
