@@ -28,9 +28,11 @@
 #include <mir_test_framework/headless_display_buffer_compositor_factory.h>
 #include <mir/test/doubles/null_logger.h>
 
+#include <mir/default_configuration.h>
 #include <mir/fd.h>
 #include <mir/main_loop.h>
 #include <mir/server.h>
+#include <mir/options/configuration.h>
 #include <mir/options/option.h>
 
 #include <boost/throw_exception.hpp>
@@ -40,13 +42,13 @@ using namespace std::chrono_literals;
 namespace mtf = mir_test_framework;
 namespace msh = mir::shell;
 namespace ml = mir::logging;
+namespace mo = mir::options;
 namespace mtd = mir::test::doubles;
 
 namespace
 {
 std::chrono::seconds const timeout{20};
 char const* dummy_args[2] = { "TestServer", nullptr };
-char const* const trace_option = "window-management-trace";
 }
 
 miral::TestDisplayServer::TestDisplayServer() :
@@ -57,6 +59,7 @@ miral::TestDisplayServer::TestDisplayServer() :
 miral::TestDisplayServer::TestDisplayServer(int argc, char const** argv) :
     runner{argc, argv}
 {
+    unsetenv("WAYLAND_DISPLAY");    // We don't want to conflict with any existing server
     add_to_environment("MIR_SERVER_PLATFORM_GRAPHICS_LIB", mtf::server_platform("graphics-dummy.so").c_str());
     add_to_environment("MIR_SERVER_PLATFORM_INPUT_LIB", mtf::server_platform("input-stub.so").c_str());
     add_to_environment("MIR_SERVER_NO_FILE", "on");
@@ -85,9 +88,19 @@ void miral::TestDisplayServer::start_server()
 
             auto init = [this](mir::Server& server)
                 {
-                    server.add_configuration_option(trace_option, "log trace message", mir::OptionType::null);
                     server.add_configuration_option(mtd::logging_opt, mtd::logging_descr, false);
 
+                    // These options are needed to test through the legacy mirclient API
+                    server.add_configuration_option(mo::server_socket_opt,
+                                                    "Socket filename [string:default=$XDG_RUNTIME_DIR/mir_socket or /tmp/mir_socket]",
+                                                    mir::default_server_socket);
+                    server.add_configuration_option(mo::no_server_socket_opt,
+                                                    "Do not provide a socket filename for client connections", mir::OptionType::null);
+                    server.add_configuration_option(mo::prompt_socket_opt,
+                                                    "Provide a \"..._trusted\" filename for prompt helper connections", mir::OptionType::null);
+                    server.add_configuration_option(mo::enable_mirclient_opt,
+                                                    "Enable deprecated mirclient socket", mir::OptionType::null);
+
                     server.add_init_callback([&]
                         {
                             auto const main_loop = server.the_main_loop();
