@@ -21,7 +21,6 @@
 #include "mir/graphics/display_buffer.h"
 #include "mir/graphics/graphic_buffer_allocator.h"
 #include "mir/graphics/platform_authentication.h"
-#include "mir/graphics/platform_operation_message.h"
 #include "mir/options/program_option.h"
 #include "mir/shared_library.h"
 #include "mir/renderer/gl/context_source.h"
@@ -36,17 +35,17 @@
 #include "mir/emergency_cleanup.h"
 
 #include <EGL/egl.h>
-#include MIR_SERVER_GL_H
+#include <GLES2/gl2.h>
 #include <boost/exception/diagnostic_information.hpp>
 #include <chrono>
 #include <iostream>
 #include <mir/geometry/displacement.h>
-#include <mir/graphics/wayland_allocator.h>
 #include <mir/renderer/gl/render_target.h>
 #include <thread>
 #include <unistd.h>
 
 namespace mg = mir::graphics;
+namespace mrs = mir::renderer::software;
 
 namespace
 {
@@ -129,11 +128,6 @@ public:
             {
                 return {};
             }
-            auto platform_operation(unsigned int /*op*/, mir::graphics::PlatformOperationMessage const& /*request*/)
-                -> mg::PlatformOperationMessage override
-            {
-                BOOST_THROW_EXCEPTION((std::runtime_error{"No support for mirclient"}));
-            }
         };
 
         return std::make_shared<NullPlatformAuthentication>();
@@ -240,7 +234,7 @@ auto test_display_construction(mir::graphics::DisplayPlatform& platform, Minimal
 
 auto test_display_supports_gl(mg::Display& display) -> bool
 {
-    if (dynamic_cast<mir::renderer::gl::ContextSource*>(display.native_display()))
+    if (dynamic_cast<mir::renderer::gl::ContextSource*>(&display))
     {
         std::cout << "Display supports GL context creation" << std::endl;
         return true;
@@ -357,33 +351,25 @@ void basic_display_swapping(mg::Display& display)
         });
 }
 
-std::unique_ptr<unsigned char[]> make_pixel_buffer(size_t size, uint32_t value)
-{
-    auto pixels = std::make_unique<unsigned char[]>(size);
-
-    std::fill(
-        reinterpret_cast<uint32_t*>(pixels.get()),
-        reinterpret_cast<uint32_t*>(pixels.get() + size),
-        value);
-
-    return pixels;
-}
-
 std::shared_ptr<mg::Buffer> alloc_and_fill_sw_buffer(
     mg::GraphicBufferAllocator& allocator,
     mir::geometry::Size size,
     MirPixelFormat format,
     uint32_t fill_value)
 {
-    auto const buffer = allocator.alloc_software_buffer(size, format);
-    auto const writable_buffer = std::dynamic_pointer_cast<mir::renderer::software::PixelSource>(buffer);
+    auto const num_px = size.width.as_uint32_t() * size.height.as_uint32_t();
+    auto buffer = std::make_unique<unsigned char[]>(num_px* MIR_BYTES_PER_PIXEL(format));
+    std::fill(
+        reinterpret_cast<uint32_t*>(buffer.get()),
+        reinterpret_cast<uint32_t*>(buffer.get() + num_px),
+        fill_value);
 
-    auto const size_in_bytes = writable_buffer->stride().as_uint32_t() * size.height.as_uint32_t();
-    auto const pixels = make_pixel_buffer(size_in_bytes, fill_value);
-
-    writable_buffer->write(pixels.get(), size_in_bytes);
-
-    return buffer;
+    return mrs::alloc_buffer_with_content(
+        allocator,
+        buffer.get(),
+        size,
+        mir::geometry::Stride(size.width.as_uint32_t() * MIR_BYTES_PER_PIXEL(format)),
+        format);
 }
 
 template<typename Period, typename Rep>
@@ -562,20 +548,6 @@ void basic_software_buffer_drawing(
     }
 }
 
-auto test_platform_supports_accelerated_wayland_clients(mg::GraphicBufferAllocator const& allocator) -> bool
-{
-    if (dynamic_cast<mg::WaylandAllocator const*>(&allocator))
-    {
-        std::cout << "Platform supports Wayland EGL" << std::endl;
-        return true;
-    }
-    else
-    {
-        std::cout << "Platform does *not* support Wayland EGL" << std::endl;
-        return false;
-    }
-}
-
 void print_diagnostic_information(std::exception const& err)
 {
     std::cout << "Error: " << err.what() << std::endl;
@@ -623,7 +595,6 @@ int main(int argc, char const** argv)
                 basic_display_swapping(*display);
 
                 auto buffer_allocator = platform->create_buffer_allocator(*display);
-                success &= test_platform_supports_accelerated_wayland_clients(*buffer_allocator);
 
                 basic_software_buffer_drawing(
                     *display,
